/**
 * 
 */
package deploy

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.sas.management.deployment.DeployMetadata;
import com.sas.management.deployment.DeploymentException;
import com.sas.management.deployment.MetadataUtility;
import com.sas.metadata.remote.MdException;

/**
 * @author brswee
 *
 */
public class AddMetadata {
	
	public boolean RUN_UPDATES = true;
	public boolean DEBUG = false;
	
	public static final String IDENTITY_GROUP_TYPE = "IdentityGroup";
	public static final String IDENTITY_GROUP = "<IdentityGroup name=\"\$INNER_ID_GROUP_NAME\$\" "+
	"publicType=\"UserGroup\" usageVersion=\"1000000\"/>";
	
	public static final String IDENTITY_ROLE = "<IdentityGroup name=\"\$INNER_ID_ROLE_NAME\$\" "+
	"GroupType=\"ROLE\" publicType=\"Role\" usageVersion=\"1000000\"/>";
	
	public String addItem(String xml, Map variables, String reposId) {
		StringBuffer buffer = new StringBuffer();
		String flags = "268435456"
		buffer.append("<AddMetadata>\n<Metadata>");
		
		String replaced = xml;
		for (Iterator iterator = variables.entrySet().iterator(); iterator.hasNext(); ) {
			Map.Entry entry = (Entry) iterator.next();
//			            System.out.println("replacing "+entry.getKey()+" with "+entry.getValue());
			replaced = replaced.replaceAll((String) entry.getKey(), (String) entry.getValue());
		}
		buffer.append(replaced);
		
		buffer.append("</Metadata>\n<ReposId>");
		buffer.append(reposId);
		buffer.append("</ReposId>\n<Ns>SAS</Ns>\n<Flags>");
		buffer.append(flags);
		buffer.append("</Flags>\n<Options/>\n</AddMetadata>");
		return buffer.toString();
		
	}
	
	public String addGroup(String xml, Map variables, String name, String displayName, String reposId) {
		String result = addItem(xml, variables, reposId);
		//do it this way so we don't modify the variables map.
		String namePattern = "\\\$INNER_ID_GROUP_NAME\\\$";
		result = result.replaceAll(namePattern,name);
		result = result.replaceAll("\\\$INNER_ID_GROUP_DISPLAY_NAME\\\$",name);
		return result;
	}
	
	public String addGroupIfNotExisting(String name, Map vars, MetadataExecution executor, String foundationId) {
		String analystId = executor.getIdByName(IDENTITY_GROUP_TYPE, "@Name='"+name+"'", foundationId );
		if(null == analystId) {
			String xmlToExecute = addGroup(AddMetadata.IDENTITY_GROUP, vars, name, name, foundationId);
			if(DEBUG) {
				println(xmlToExecute);
			}
			if(RUN_UPDATES) {
				analystId = executor.execute(xmlToExecute, IDENTITY_GROUP_TYPE);
			} else {
				return "A0000001.A0000001";
			}
		}
		return analystId;
	}
	
	public String addRole(String xml, Map variables, String name, String displayName, String reposId) {
		String result = addItem(xml, variables, reposId);
		//do it this way so we don't modify the variables map.
		String namePattern = "\\\$INNER_ID_ROLE_NAME\\\$";
		result = result.replaceAll(namePattern,name);
		result = result.replaceAll("\\\$INNER_ID_ROLE_DISPLAY_NAME\\\$",name);
		return result;
	}
	
	public String addRoleIfNotExisting(String name, Map vars, MetadataExecution executor, String foundationId) {
		String analystId = executor.getIdByName(IDENTITY_GROUP_TYPE, "@Name='"+name+"'", foundationId );
		if(null == analystId) {
			String xmlToExecute = addRole(AddMetadata.IDENTITY_ROLE, vars, name, name, foundationId);
			if(DEBUG) {
				println(xmlToExecute);
			}
			if(RUN_UPDATES) {
				analystId = executor.execute(xmlToExecute, IDENTITY_GROUP_TYPE);
			} else {
				return "A0000001.A0000001";
			}
		}
		return analystId;
	}
	
	public String addRawMetadata(String repositoryId, String fileName, Map mapping, MetadataExecution executor) throws DeploymentException {
	    return executeRawMetadata(repositoryId, fileName, mapping, executor, "Add");
	}
	public String updateRawMetadata(String repositoryId, String fileName, Map mapping, MetadataExecution executor) throws DeploymentException {
	    return executeRawMetadata(repositoryId, fileName, mapping, executor, "Update");
	}
	
	public String executeRawMetadata(String repositoryId, String fileName, Map mapping, MetadataExecution executor, String method) throws DeploymentException {
		String result = null;
		FileInputStream fs = null;
		try {
			fs = new FileInputStream(new File(fileName));
		}
		catch (FileNotFoundException e) {
			throw new RuntimeException(e);
		}
		InputStreamReader ir=null;
		try {
			ir = new InputStreamReader(fs, "UTF-8");
		}
		catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
		BufferedReader br = new BufferedReader(ir);
		
		StringBuffer buffer = new StringBuffer();
		buffer.append("<"+method+"Metadata>\n<Metadata>");
		int commentIndex = -1;
		try {
			String inline;
			boolean beginTransform = false;
			while ((inline = br.readLine()) != null) {
				int index = inline.indexOf("subxml");
				if (index >= 0) {
					//
					// really fragile, using : as a tokenizer
					//
					int firstColon = inline.indexOf(':');
					int secondColon = inline.indexOf(':', firstColon + 1);
					int thirdColon = inline.indexOf(':', secondColon + 1);
					String key = inline.substring(firstColon+1, secondColon-1).trim();
					key = "\\" + key.substring(0, key.length()-1) + "\\\$"; // since we are using it as a regex later need to escape the $
					String type = inline.substring(secondColon+1, thirdColon-1).trim();
					String searchString = inline.substring(thirdColon+1).trim();
					try {
						mapping.put(key, executor.getIdByName(type, searchString, repositoryId));
					}catch (NullPointerException e) {
						println("could not find an ID for "+key);
						throw e;
					}
					println(key+"->"+mapping.get(key));
				}
				
				if (beginTransform)
					buffer.append(inline + "\n");
				commentIndex = inline.indexOf("-->");  // end of the comments
				if (commentIndex >= 0)
					beginTransform = true;
			}
			
			buffer.append("</Metadata>\n<Reposid>");
			buffer.append(repositoryId);
			// TODO might have to change what flags are set...
			buffer.append("</Reposid>\n<Ns>SAS</Ns>\n<Flags>268435456</Flags>\n<Options/>\n</"+method+"Metadata>");
			
			//  This code is not very efficient to say the least!!!!
			
			result = buffer.toString();
			for (Iterator iterator = mapping.entrySet().iterator(); iterator.hasNext(); ) {
				Map.Entry entry = (Entry) iterator.next();
//				println("replacing "+entry.getKey()+" with "+entry.getValue());
				result = result.replaceAll((String) entry.getKey(), (String) entry.getValue());
			}
			
		}
		catch (IOException e) {
			throw new RuntimeException(e);
		}
		return result;
	}
	
	public static String wrapForUpdate(String xmlIn, String reposId) {
		StringBuffer buffer = new StringBuffer();
		String flags = "268435456"
		buffer.append("<UpdateMetadata>\n<Metadata>");

		buffer.append(xmlIn);
		
		buffer.append("</Metadata>\n");
//		buffer.append("<ReposId>");
//		buffer.append(reposId);
//		buffer.append("</ReposId>\n");
		buffer.append("<Ns>SAS</Ns>\n<Flags>");
		buffer.append(flags);
		buffer.append("</Flags>\n<Options/>\n</UpdateMetadata>");
		return buffer.toString();

	}
	
	public static String wrapForGet(String type, String searchString, String reposId, String flags) {
	    String xml = "<GetMetadataObjects> <ReposId>" + 
	    reposId +
        "</ReposId>" +
        "<Type>" + type + "</Type>" +
        "<Objects/>" +
        "<ns>SAS</ns>" +
        "<Flags>"+flags+"</Flags>" +  //TODO may have to update the Flags - orig 8320 
        "<Options>" +
        "<XMLSelect search=\"" + searchString + "\"/>" +
        "</Options>" +
        "</GetMetadataObjects>";
	}
	
}
