package deploy;

import com.sas.services.webdav.DAVResource;
import com.sas.services.webdav.Acl;
import com.sas.services.webdav.Principal;
import com.sas.services.webdav.Ace;
import com.sas.services.webdav.Privilege;

import java.util.List;
import java.util.Iterator;

try
{
  boolean DEBUG = true;
  String user = attributes.get("admin.user");
  String password = attributes.get("admin.pass");
  String targetUrl = attributes.get("dav.target.url");

  if(DEBUG)
  {
    println("--- INPUT ATTRIBUTES ---");
    println("USER: " + user);
    println("TARGET URL: " + targetUrl);    
    println("");
  }

  DAVResource resource = new DAVResource(targetUrl, user, password);

  if(resource.exists())
  {
    if(DEBUG)
      println("Setting ACL on resource: " + targetUrl);

    Principal authenticated = new Principal(Principal.AUTHENTICATED);
    Privilege [] privileges = [ Privilege.PRIV_WRITE, 
                                Privilege.PRIV_SCS_DELETE,
                                Privilege.PRIV_SCS_INHERIT_WRITE,
                                Privilege.PRIV_SCS_INHERIT_DELETE 
                               ];
    
    InstallDAVPermissions.setPermissions(resource, authenticated, privileges);

    if(DEBUG)
      println("Done setting ACL.");    
  }
  else
  {
    println("WARN: Could not set ACL on " + targetUrl);
  }
  
}
catch (Exception e)
{
  e.printStackTrace();
}



public class InstallDAVPermissions
{
  static void setPermissions(DAVResource resource, Principal principal, Privilege [] privileges)
  {
     Acl acl = resource.getAcl();

     List aceList = acl.getAceList();
     Iterator items = aceList.iterator();
     boolean aceFound = false;
     while (items.hasNext()) {
       Ace aceItem = (Ace) items.next();
       if (aceItem.getPrincipal().equals(principal)) {
         Privilege [] existingPrivileges = aceItem.getPrivileges();
         for(int i = 0; i < privileges.length; i++)
         {
           addPrivilege(aceItem, existingPrivileges, privileges[i]);           
         }        
         aceFound = true;
         break;
       }
     }
     if (!aceFound) {
       println("WARN: Could not find access control entry for jcr:authenticated.  Could not set ACL.");
       return;
     }

     resource.setAcl(acl);
  }

  private static void addPrivilege(Ace aceItem, Privilege [] existingPriv, Privilege thePrivilege)
  {
    boolean found = false;
    for(int i = 0; i < existingPriv.length && !found; i++)
    {
      if(existingPriv[i].equals(thePrivilege))
        found = true;
    }

    if(!found)
    {
      aceItem.addPrivilege(thePrivilege);      
    }
  }
}