package deploy;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import com.sas.iom.SASIOMDefs.GenericError;
import com.sas.metadata.MdObjectFactory;
import com.sas.metadata.MetadataUtil;
import com.sas.metadata.remote.AssociationList;
import com.sas.metadata.remote.Column;
import com.sas.metadata.remote.CMetadata;
import com.sas.metadata.remote.PhysicalTable;
import com.sas.metadata.remote.TextStore;
import com.sas.metadata.remote.MdException;
import com.sas.metadata.remote.MdFactory;
import com.sas.metadata.remote.MdFactoryImpl;
import com.sas.metadata.remote.MdObjectStore;
import com.sas.metadata.remote.MetadataObjects;
import com.sas.metadata.remote.SoftwareComponent;
import com.sas.metadata.remote.ServiceComponent;
import com.sas.metadata.remote.DeployedComponent;
import com.sas.metadata.remote.PropertyGroup;
import com.sas.metadata.remote.PropertySet;
import com.sas.metadata.remote.Tree;
import com.sas.management.deployment.MetadataUtility;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.SAXException;
import java.io.StringReader;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;

public class JomaSupport
{
  /**
     * The ClassIdentifier attribute value for the SoftwareComponent that's
     * the root of the tree structures used by SAS applications.
     */
    protected final static String ROOT_CLASSIDENTIFIER   = "E5F27790-2149-11D6-8828-AA0004006D06"/* I18nOK:EMS */;
    private static final String   EX_MSG_CONNECTIONABORT = "connection abort"/* I18nOK:EMS */;
    private static final String   EX_MSG_EOFEXCEPTION    = "EOFException"/* I18nOK:EMS */;
    private static final String   EX_MSG_SOCKETEXCEPTION = "SocketException"/* I18nOK:EMS */;

    private static final String SOLUTIONS_REPOS_NM = "Solutions";
    private static final String FINANCE_REPOS_NM = "Finance";
    private static final String SPM_REPOS_NM = "Performance Management";


    // from com.sas.metadata.MetadataUtil
   /**
    * (268435456)Determines if the client can call this method. This flag is currently required.
    */
   public static final int OMI_TRUSTED_CLIENT = 0x10000000; /*   268435456 */

    protected MdFactory           _factory;
    protected MdObjectStore       _store;
    protected String              _foundationReposName   = null;
    protected Map                 _repositories          = new HashMap();
    protected Map                 _id2NameMap            = new HashMap();


    public JomaSupport()
    {
    }

    public void connect(String host, String port, String userid, String
            userpw) throws Exception
    {

        try {
            /*
             * THIS IS jOMA stuff
             */
            _factory = new MdFactoryImpl();
            _factory.makeConnection2(host, port, userid, userpw);
            _store = _factory.createObjectStore();
        } catch (MdException ex) {
            throw new Exception(ex.getMessage(), ex);
        } catch (RemoteException ex) {
            throw new Exception(ex.getMessage(), ex);
        }
        loadRepositoryData();
    }

    /**
     * Disonnect from the metadata server. Close the connection and dispose
     * of the factory and store.
     */
    public void disconnect()
    {
        try {
            _store.dispose();
            _factory.dispose();
            _repositories.clear();
            _id2NameMap.clear();
        } catch (RemoteException ex) {
        }
    }

    /**
     * Return a List of RepositoryEntry objects with descriptions of the repositories managed by the connected metadata
     * server.
     *
     * @return A List of RepositoryEntry objects.
     */
    protected List getRepositories()
    {
        return new ArrayList(_repositories.values());
    }

    /**
     * Get the BIP Service SoftwareComponent for the repository given. If no repository is provided, the foundation
     * repository is assumed.
     *
     * @param repository The FQID of the repository to get the BIP Service SoftwareComponent from.
     * @return The BIP Service SoftwareComponent, if found. Null if none was located.
     * @throws Exception In the event of a repository error.
     */
    public SoftwareComponent getBipRoot(String repository) throws Exception
    {
        try {
            String type = MetadataObjects.SOFTWARECOMPONENT;
            String filterString = "<XMLSelect search=\"SoftwareComponent[@Name='BIP Service' and @ClassIdentifier='"
                                  + ROOT_CLASSIDENTIFIER + "']\"/>";
            if (repository == null) {
                RepositoryEntry re = (RepositoryEntry) _repositories.get(_foundationReposName);
                if (re == null)
                    throw new Exception("Damn!  The foundation repository wasn't found!");
                repository = re.fqid;
            }
            int my_flags = MetadataUtil.OMI_XMLSELECT | MetadataUtil.OMI_GET_METADATA | MetadataUtil.OMI_TEMPLATE | MetadataUtil.OMI_ALL_SIMPLE;
            List rootComponents = (_factory.getOMIUtil().getMetadataObjectsSubset(_store, repository, type, my_flags,
                                                                                  filterString));
            if (rootComponents == null || rootComponents.size() == 0) { return null; }
            return (SoftwareComponent) rootComponents.get(0);
        } catch (MdException mex) {
            throw new Exception(mex.getMessage(), mex);
        } catch (RemoteException rex) {
            throw new Exception(rex.getMessage(), rex);
        }
    }

    /**
     * Read the data about the repositories that are managed by the connected server. It isn't returned by this method,
     * it just loads it up so it's available when getRepositories or other methods are called. This method is called
     * from connect(), so unless the data needs reloading for some reason, it shouldn't need to be called by other
     * methods.
     */
    protected void loadRepositoryData()
    {
        Map reposData = new HashMap();
        StringBuffer xmlBuffer = new StringBuffer();
        xmlBuffer.append("<GetMetadataObjects>" + "<Reposid>A0000001.A0000001</Reposid>"
                         + "<Type>RepositoryBase</Type>" + "<Objects/>" + "<ns>REPOS</ns>" + "<Flags>264</Flags>"
                         + "<options/>" + "</GetMetadataObjects>");
        String inxml = xmlBuffer.toString();/* I18nOK:LSM */
        org.omg.CORBA.StringHolder outXml = new org.omg.CORBA.StringHolder();
        try {
            _factory.getConnection().getCMRHandle().DoRequest(inxml, outXml);
        } catch (RemoteException rex) {
            return;
        } catch (GenericError ex) {
            // ex.printStackTrace(System.out);
            return;
        }
        String xmlOut = outXml.value;
        int index1 = 0;
        index1 = xmlOut.indexOf("<RepositoryBase ");
        while (index1 > -1) {
            int index3 = xmlOut.indexOf("Id=", index1);
            String fqid = xmlOut.substring(index3 + 4, index3 + 21);
            index3 = xmlOut.indexOf("Name=", index1);
            int index4 = xmlOut.indexOf('"', index3 + 6);
            String name = xmlOut.substring(index3 + 6, index4);
            index3 = xmlOut.indexOf("RepositoryType=", index1);
            index4 = xmlOut.indexOf('"', index3 + 16);
            String repositoryType = xmlOut.substring(index3 + 16, index4);
            if ("Foundation".equalsIgnoreCase(repositoryType)) {
                _foundationReposName = name;
            }
            RepositoryEntry entry = new RepositoryEntry(name, fqid, repositoryType);
            reposData.put(name, entry); // map name to repository entry
            _id2NameMap.put(fqid, name); // Map FQID to repository name
            index1 = xmlOut.indexOf("<RepositoryBase ", index1 + 20);
        }
        _repositories = reposData;
    }

    /**
     * Test an exception to see if it was caused by a connection failure.
     *
     * @param ex A Throwable to test for connection causes.
     * @return True if the cause of the error was a connection failure. False otherwise.
     */
    protected boolean isConnectionException(Throwable ex)
    {
        String mexMessage = ex.getMessage();
        if ((mexMessage != null)
            && ((mexMessage.indexOf(EX_MSG_EOFEXCEPTION) > -1) || (mexMessage.indexOf(EX_MSG_SOCKETEXCEPTION) > -1) || (mexMessage
                .indexOf(EX_MSG_CONNECTIONABORT) > -1)) || ioExceptionInChain(ex)) { return true; }
        return false;
    }

    private boolean ioExceptionInChain(Throwable ex)
    {
        while (ex != null) {
            if (ex instanceof java.io.IOException) { return true; }
            Throwable t = ex.getCause();
            if (t == ex)
                break; // avoid infinite recursion.
            ex = t;
        }
        return false;
    }

    public static String escapeXML(String s)
    {
        return s
            .replaceAll("&", "&amp;")
            .replaceAll("<", "&lt;")
            .replaceAll(">", "&gt;")
            .replaceAll("\"", "&quot;")
            ;
    }

    public String getFoundationRepositoryId()
    {
        RepositoryEntry re = (RepositoryEntry)_repositories.get(
                _foundationReposName);
        if (re == null)
            throw new Exception("The foundation repository wasn't found");
        return re.fqid;
    }

    public String getRepositoryId(String repositoryName)
    {
        RepositoryEntry re = (RepositoryEntry)_repositories.get(repositoryName);
        if (re == null)
            return null;
        return re.fqid;
    }

    public String getRepositoryName(String repositoryID)
    {
        return _id2NameMap.get(repositoryID);
    }

    public String doRequest(String xmlString, String type)
    {
        TypeHandler_groovy typeHandler = new TypeHandler_groovy(type);

        String result = "";
        try
        {
            result = MetadataUtility.DoRequest(_factory, xmlString);
        }
        catch (Exception e)
        {
            CommonUtils.info("Could not find object for query: " + xmlString);
            return "";
        }

        SAXParser saxParser = null;
        try {
            saxParser = SAXParserFactory.newInstance().newSAXParser();
            saxParser.parse(new InputSource(new StringReader(result)), typeHandler);
        }
        catch (SAXException e) {
            throw new RuntimeException(e);
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }
        return typeHandler.id;
    }

    public List getSoftwareComponentsByName(String repositoryID, String name)
    {
        if (repositoryID == null || repositoryID.length() == 0)
            repositoryID = getFoundationRepositoryId();
        String sOptions = "<XMLSELECT Search=\"@Name='" + escapeXML(name) +
            "'\"/>";
        List objects =
            (this._factory.getOMIUtil()).getMetadataObjectsSubset(
                this._store,
                repositoryID, MdObjectFactory.SOFTWARECOMPONENT,
                (MetadataUtil.OMI_XMLSELECT |
                 MetadataUtil.OMI_GET_METADATA),
                sOptions);
        return objects;
    }

    public CMetadata getSoftwareComponentByName(String repositoryID, String name)
    {
        List l = getSoftwareComponentsByName(repositoryID, name);

        if(l.size() == 0)
        {
            CommonUtils.info("Could not find Software Component: " + name + ".");
            return null;
        }

        CMetadata softwareComponent = (CMetadata) l.get(0);
        return softwareComponent;
    }

    public List getObjectsByName(String repositoryID, String typeName,
            String name)
    {
        if (repositoryID == null || repositoryID.length() == 0)
            repositoryID = getFoundationRepositoryId();
        String sOptions = "<XMLSELECT Search=\"@Name='" + escapeXML(name) +
            "'\"/>";
        List objects =
            (this._factory.getOMIUtil()).getMetadataObjectsSubset(
                this._store,
                repositoryID, typeName,
                (MetadataUtil.OMI_XMLSELECT |
                 MetadataUtil.OMI_GET_METADATA),
                sOptions);
        return objects;
    }

    public CMetadata getObjectByName(String repositoryID, String typeName, String name)
    {
        List l = getObjectsByName(repositoryID, typeName, name);
        if(l.size() == 0)
        {
            CommonUtils.info("Object " + name + " of type " + typeName + " does not exist.");
            return null;
        }
        return (CMetadata) l.get(0);
    }

    public CMetadata getSoftwareComponentByClsid(String repositoryID, String classIdentifier)
    {
        List l = getObjectsByAttrVal(repositoryID, "SoftwareComponent", "ClassIdentifier", classIdentifier);
        if(l.size() == 0)
        {
            CommonUtils.info("Could not find Software Component for ClassIdentifier: " + classIdentifier + ".");
            return null;
        }

        CMetadata softwareComponent = (CMetadata) l.get(0);
        return softwareComponent;
    }


    public List getObjectsByAttrVal(String repositoryID, String typeName,
            String attrName, String attrVal)
    {
        if (repositoryID == null || repositoryID.length() == 0)
            repositoryID = getFoundationRepositoryId();
        String sOptions = "<XMLSELECT Search=\"@" + attrName + "='" +
            escapeXML(attrVal) + "'\"/>";
        List objects =
            (this._factory.getOMIUtil()).getMetadataObjectsSubset(
                this._store,
                repositoryID, typeName,
                (MetadataUtil.OMI_XMLSELECT |
                 MetadataUtil.OMI_GET_METADATA),
                sOptions);
        return objects;
    }


    /**
    * Find Metadata objects matching an XPath query.
    * @param repositoryID ID of repository to search, null for Foundation
    * @param typeName Exact type of metadata object to find
    * @param xmlSelectQuery An "XPath" metadata query. Examples:
    *     "@Name='Fred'"
    *     "*[@Name='XYZ']"
    *     See http://sww.sas.com/~sassyp/oma/omaref.hlp/xmlselect.htm
    */
    public List getMetadataObjects(String repositoryID, String typeName, String xmlSelectQuery)
    {
       return getMetadataObjects(repositoryID, typeName, xmlSelectQuery, null);
    }


    public List getMetadataObjects(String repositoryID, String typeName, String xmlSelectQuery, String template)
    {
        if (repositoryID == null || repositoryID.length() == 0)
            repositoryID = getFoundationRepositoryId();
        String sOptions = "<XMLSELECT Search=\"" + escapeXML(xmlSelectQuery)+ "\"/>";
        int flags = MetadataUtil.OMI_XMLSELECT | MetadataUtil.OMI_GET_METADATA;

        if(template != null)
        {
           sOptions = sOptions + template;
           flags = MetadataUtil.OMI_XMLSELECT | MetadataUtil.OMI_TEMPLATE | MetadataUtil.OMI_GET_METADATA;
        }
        CommonUtils.debug(sOptions);
        List objects =
            (this._factory.getOMIUtil()).getMetadataObjectsSubset(this._store, repositoryID, typeName,
                flags,
                sOptions);
        return objects;
    }

    // http://java.na.sas.com/reference/api/sas/bip/v920/com/sas/metadata/remote/MdFactory.html#createComplexMetadataObject(com.sas.metadata.remote.MdObjectStore,%20java.lang.String,%20java.lang.String,%20java.lang.String)
    // http://sww.sas.com/~sassyp/oma/jmiug.hlp/jomaobj.htm
    public CMetadata createObject(String repositoryID, String typeName,
                               String name)
    {
        CMetadata obj = createObjectNoUpdate(repositoryID, typeName, name);
        obj.updateMetadataAll();
        return obj;
    }

    /**
     * There are some objects that have required associations.
     * (e.g. ExternalIdentity)  This means that they cannot be
     * updated (i.e. obj.updateMetadataAll(), until associations
     * are set.
     *
     * Caller will need to:
     *   > run this method
     *   > set the association
     *   > then run obj.updateMetadataAll()
     */
    public CMetadata createObjectNoUpdate(String repositoryID, String typeName,
                               String name)
    {
        if (repositoryID == null || repositoryID.length() == 0)
            repositoryID = getFoundationRepositoryId();
        String shortReposID = repositoryID.substring(
                repositoryID.indexOf('.') + 1, repositoryID.length());
        CMetadata obj = this._factory.createComplexMetadataObject(
                this._store,
                name,
                typeName,
                shortReposID);
        return obj;
    }

    public void deleteObjects(List objsToDel)
    {
        if (objsToDel == null || objsToDel.size() == 0)
        {
            return;
        }
        CommonUtils.info("Deleting " + objsToDel.size() + " objects.");
        _factory.deleteMetadataObjects(objsToDel);
    }

    public void createBIPFolder(String repositoryName, String folderName, String parentFolder)
    {
        String repositoryID = getRepositoryId(repositoryName);
        Tree tree = (Tree) createObject(repositoryID, MetadataObjects.TREE, folderName);

        if(parentFolder == null)
        {
            //Set the BIP Service SoftwareComponents association
            SoftwareComponent bipService = (SoftwareComponent) getSoftwareComponentByName(null, "BIP Service");
            AssociationList softwareComponents = new AssociationList("SoftwareComponents");
            softwareComponents.add(bipService);
            tree.setSoftwareComponents(softwareComponents);
        }
        else
        {
            Tree parentTree = (Tree) getObjectByName(repositoryID, MetadataObjects.TREE, parentFolder);
            tree.setParentTree(parentTree);
        }

        //SET THE PUBLIC TYPE ATTRIBUTE TO "Folder"
        tree.setPublicType("Folder");

        //SET THE USAGE VERSION UsageVersion TO 1000000
        tree.setUsageVersion(1000000);

        //SET THE TreeType TO "BIP Folder"
        tree.setTreeType("BIP Folder");

        //UPDATE METADATA
        tree.updateMetadataAll();
    }

     public static CMetadata getAssocObjByName(CMetadata obj,
            String assocName, String objName)
    {
        if(obj == null)
        {
            CommonUtils.info("Parent object is null.  Can not do lookup for " + assocName + "." + objName);
            return null;
        }

        AssociationList objects = obj.getAssociatedObjects(assocName);
        for (Iterator it=objects.iterator(); it.hasNext(); )
        {
            CMetadata o = (CMetadata)it.next();
            if (objName.equalsIgnoreCase(o.getName()))
                return o;
        }
        return null;
    }


    public static CMetadata getAssocObjByName(CMetadata obj,
            String assocName, String objName, List list)
    {
        CMetadata assocObject = getAssocObjByName(obj, assocName, objName);
        if(assocObject != null)
           list.add(assocObject);
        else
           CommonUtils.info("Could not find " + objName + " associated with " + obj.getName());
        return assocObject;
    }


    public static CMetadata [] getAssocObjByName(CMetadata obj,
            String assocName, String [] objNames, List list)
    {
       CMetadata [] result = new CMetadata[objNames.length];
       for(int i = 0; i < objNames.length; i++)
       {
          CMetadata assocObject = getAssocObjByName(obj, assocName, objNames[i]);
          result[i] = assocObject;
          if(assocObject != null)
             list.add(assocObject);
          else
             CommonUtils.info("Could not find " + objNames[i] + " associated with " + obj.getName());
       }
       return result;
    }


    public static String getAttribute(CMetadata obj, String attrName)
    {
        return (String)obj.getAttrs().get(attrName);
    }

    // Remove obj from the association, if it is associated.
    // Return the object removed from the association, or null if it is not
    // associated.
    public static CMetadata removeAssociatedObject(CMetadata parent,
            String assocName, CMetadata obj)
    {
        String id = obj.getId();
        CMetadata result = null;
        AssociationList objects = parent.getAssociatedObjects(assocName);
        int i = 0;
        for (Iterator it=objects.iterator(); it.hasNext(); ++i)
        {
            CMetadata o = (CMetadata)it.next();
            if (id.equals(o.getId()))
            {
                result = o;
                break;
            }
        }
        if (result != null)
        {
            objects.remove(i);
            parent.updateMetadataAll();
        }
        return result;
    }


    public static CMetadata addRegisteredSSASSolution(CMetadata softwareComponent, List list)
    {

        DeployedComponent registeredSS = (DeployedComponent) getAssocObjByName(softwareComponent, "DeployedComponents", "Registered SAS Solution");
        if(registeredSS == null)
        {
            CommonUtils.info("Could not find Registered SAS Solution for " + softwareComponent.getName());
            return null;
        }
        list.add(registeredSS);

        AssociationList properties = registeredSS.getProperties();
        for(int i = 0; i < properties.size(); i ++)
        {
            CMetadata property = properties.get(i);
            if(property != null)
            {
                list.add(property);
            }
        }

        AssociationList sourceConnections = registeredSS.getSourceConnections();
        for(int i = 0; i < sourceConnections.size(); i ++)
        {
            CMetadata connection = sourceConnections.get(i);
            if(connection != null)
            {
                list.add(connection);
            }
        }

        return registeredSS;
    }

    public static CMetadata addApplicationConfiguration(CMetadata softwareComponent, List list)
    {
        PropertySet applicationConfig = (PropertySet) getAssocObjByName(softwareComponent, "PropertySets", "Application Configuration");

        if(applicationConfig == null)
        {
            CommonUtils.info("Could not find Application Configuration for " + softwareComponent.getName());
            return null;
        }
        list.add(applicationConfig);

        AssociationList properties = applicationConfig.getSetProperties();
        for(int i = 0; i < properties.size(); i ++)
        {
            CMetadata property = properties.get(i);
            if(property != null)
            {
                list.add(property);
            }
        }
        return applicationConfig;
    }

    public static Map getApplicationConfigurationProperties(CMetadata softwareComponent)
    {
        PropertySet applicationConfig = (PropertySet) getAssocObjByName(softwareComponent, "PropertySets", "Application Configuration");

        if(applicationConfig == null)
        {
            CommonUtils.info("Could not find Application Configuration for " + softwareComponent.getName());
            return null;
        }


        AssociationList properties = applicationConfig.getSetProperties();
        Map propMap = new HashMap(properties.size);
        for(int i = 0; i < properties.size(); i ++)
        {
            CMetadata property = properties.get(i);
            if(property != null)
            {
                String propName = property.getName();
                propMap.put(propName, property);
            }
        }
        return propMap;
    }

    public static void addDirectiveServiceComponents(List serviceComponents, List list)
    {
        for(int i = 0; i < serviceComponents.size(); i++)
        {
            ServiceComponent sc = serviceComponents.get(i);
            if(sc == null)
                continue;

            list.add(sc);

            AssociationList propertySets = sc.getPropertySets();
            for(int k = 0; k < propertySets.size(); k++)
            {
                PropertySet propertySet = (PropertySet) propertySets.get(k);
                if(propertySet != null)
                {
                    list.add(propertySet);

                    AssociationList properties = propertySet.getProperties();
                    for(int j = 0; j < properties.size(); j ++)
                    {
                        CMetadata property = properties.get(j);
                        if(property != null)
                       {
                           list.add(property);
                       }
                    }
                 }
            }


            AssociationList properties = sc.getProperties();
            for(int j = 0; j < properties.size(); j ++)
            {
                CMetadata property = properties.get(j);
                if(property != null)
                {
                    list.add(property);
                }
            }
        }
    }


    public static CMetadata addObjectIdentifierServiceComponent(CMetadata softwareComponent, List list)
    {
        ServiceComponent objectIdentifier = (ServiceComponent) getAssocObjByName(softwareComponent, "DeployedComponents", "ObjectIdentifier");

        if(objectIdentifier == null)
        {
            CommonUtils.info("Could not find ObjectIdentifier for " + softwareComponent.getName());
            return null;
        }

        list.add(objectIdentifier);

        AssociationList implementors = objectIdentifier.getImplementors();
        for(int i = 0; i < implementors.size(); i ++)
        {
            CMetadata implementor = implementors.get(i);
            if(implementor != null)
            {
                list.add(implementor);
            }
        }

        PropertyGroup historyAction = (PropertyGroup) getAssocObjByName(objectIdentifier, "PropertyGroups", "HistoryAction", list);
        AssociationList properties = historyAction.getProperties();
        for(int i = 0; i < properties.size(); i ++)
        {
            CMetadata property = properties.get(i);
            if(property != null)
            {
                list.add(property);
            }
        }
        return objectIdentifier;
    }

    public static String getSolutionsRepositoryName()
    {
        return SOLUTIONS_REPOS_NM;
    }

    public static String getFinanceRepositoryName()
    {
        return FINANCE_REPOS_NM;
    }

    public static String getPerformanceManagementRepositoryName()
    {
        return SPM_REPOS_NM;
    }

    public CMetadata newObject(String repositoryID, String typeName,
            String objectName)
    {
        if (repositoryID == null || repositoryID.length() == 0)
            repositoryID = getFoundationRepositoryId();
        String shortReposID = repositoryID.substring(
                repositoryID.indexOf('.') + 1, repositoryID.length());
        CMetadata obj = this._factory.createComplexMetadataObject(
                this._store, null, objectName, typeName, shortReposID);
        obj.updateMetadataAll();
        return obj;
    }

    public CMetadata newProperty(String repositoryID, String propName)
    {
        return newObject(repositoryID, MdObjectFactory.PROPERTY, propName);
    }


}