package deploy;

import java.rmi.RemoteException;

import com.sas.metadata.logical.ServicesUtil;
import com.sas.services.ServiceException;
import com.sas.services.deployment.CorePlatformServices;
import com.sas.services.information.InformationServiceInterface;
import com.sas.services.session.SessionContextInterface;
import com.sas.services.user.UserContextInterface;
import com.sas.services.user.UserServiceInterface;


public class SDWPlatformServices
{
    private static final String DOMAIN = "DefaultAuth";

    
    private SDWPlatformServices()
    {
    }


    public static void startLocalPlatformServices(String host, String port)
            throws ServiceException, RemoteException
    {
        CommonUtils.info("-->Starting metadata connection to host: " + host);
       
        ServicesUtil.setLoginConfiguration(host, port);
        ServicesUtil.startLocalServices();
        getInformationService().setIsolated();
        ServicesUtil.validateServices();

        CommonUtils.info("<--Metadata connection complete.");
    }

    
    public static InformationServiceInterface getInformationService()
    {
        InformationServiceInterface rval = CorePlatformServices.getInformationService();
        if (rval == null) { throw new IllegalStateException("Platform services is not started"/* I18nOK:EMS */); }
        return rval;
    }
    
    
    public static void verifyPlatformServicesStarted() throws ServiceException
    {
        ServicesUtil.validateServices();
    }

 
    public static void stopPlatformServices()
    {
        try {
            ServicesUtil.terminateServices();
        } catch (ServiceException e) {
            e.printStackTrace();
        }
    }

    
    public static UserContextInterface newUser(String uid, String pwd) throws RemoteException,
            ServiceException
    {
        UserContextInterface rval;
        UserServiceInterface userService = CorePlatformServices.getUserService();

        rval = userService.newUser(uid, pwd, DOMAIN);
        SessionContextInterface session = CorePlatformServices.getSessionService().newSessionContext(rval);
        rval.setSessionContext(session);
        
        return rval;
    }
}
