package deploy.StMBIDProvider;

import com.sas.framework.services.metadata.BaseAssociationFilter;
import com.sas.metadata.remote.MdOMIUtil;
import com.sas.metadata.remote.MetadataObjects;
import com.sas.report.repository.AdapterFactory;
import com.sas.report.repository.EntryInterface;
import com.sas.services.information.RepositoryInterface;
import com.sas.services.information.metadata.MetadataInterface;
import com.sas.services.user.UserContextInterface;
import java.util.Iterator;
import java.util.List;

import deploy.SDWPlatformServices;

//----- RUN THE UPGRADE CODE -----

/*----------------------------------------------------------------------------*
  Groovy code to upgrade Stm installation release metadata
  
  The old_stmurl/new_stmurl are passed in as attribute properties for running
  the script.
  
 *----------------------------------------------------------------------------*/

boolean DEBUG = true;
int rc = 0;

try {
    
    String user = attributes.get("admin.user");
    String password = attributes.get("admin.pass");
    String host= attributes.get("omr.host");
    String port = attributes.get("omr.port");
    String repositoryName = attributes.get("foundation.repos.nm");
    String old_stmurl = attributes.get("from.url");
    String new_stmurl = attributes.get("to.url");
     
    

    if(DEBUG)
    {
      println("--- INPUT ATTRIBUTES ---");
      println("USER: " + user);
      println("OMR HOST: " + host);    
      println("OMR PORT: " + port);
      println("FROM StM URL: " + old_stmurl);
      println("TO StM URL: "   + new_stmurl);
    }
        
    UpgradeStMBIDProviderServerUrl upgrade = new UpgradeStMBIDProviderServerUrl();  
    upgrade.upgradeAll(host, port, user, password, repositoryName, old_stmurl, new_stmurl);
    
}
catch (Exception e)
{
    rc = 1;
    project.setNewProperty("outputproperty", e.toString());     
    e.printStackTrace();    
}
finally
{
  if(DEBUG) println("DONE.");
}

project.setNewProperty("resultsproperty", "" + rc);

// ----- END RUN THE UPGRADE CODE -----



class UpgradeStMBIDProviderServerUrl {

    boolean DEBUG = true;
    
    public UpgradeStMBIDProviderServerUrl() {}

    
    public void upgradeAll(String host, String port, String user, String password, String repositoryName, String old_stmurl,
                           String new_stmurl) {
       // get a usercontext.
       SDWPlatformServices.startLocalPlatformServices(host, port);
       UserContextInterface userContext = SDWPlatformServices.newUser(user, password);

       if(userContext == null) { 
	   logError("Unable to migrate documents.  Problem with user context.");
	   throw new RuntimeException("Unable to migrate documents.  Problem with user context.");
       }

	// locate the providers in metadata.
	BaseAssociationFilter filter = new BaseAssociationFilter();
	filter.setType(MetadataObjects.TRANSFORMATION);
    	filter.setOMRFlags(MdOMIUtil.OMI_MATCH_CASE);
	filter.setXMLSelect("<XMLSELECT search=\"Transformation[@PublicType='IndicatorData']\"/>");
        RepositoryInterface repositoryInterface = userContext.getAuthServer().getRepository(repositoryName);
        List transformations = repositoryInterface.search(filter); 
        
	// go trhough the indicators from metadata.
	MetadataInterface transformation;
	if (transformations != null) {
		String origContent;
		String modContent;
		byte[] readData;
		byte[] writeData;
		InputStream istr;
		OutputStream ostr;
		
		Object serviceObject;
		EntryInterface entry;
		
		Iterator iter = transformations.iterator();
		while (iter.hasNext()) {
			// do mysterious BID stuff to get an input stream.
			transformation = (MetadataInterface) iter.next();
			serviceObject = transformation.newServiceObject();
			entry = AdapterFactory.adaptEntry(serviceObject);
			istr = entry.getInputStream();

			// get the string value from the object
			readData = new byte[istr.available()];
			istr.read(readData);
			istr.close();
			origContent = new String(readData);
			
			// now operate on the string making necessary changes to the server url.
			if (origContent != null && origContent.length() > 0) {
				
				// modify the server name if it's an StM provider.
				modContent = fixUpString(origContent, old_stmurl, new_stmurl);

				// persist the change if there was a change.
				if (!origContent.equals(modContent)) {
					info("modifying provider: " + transformation.getPathUrl());
					ostr = entry.getOutputStream();
					writeData = modContent.getBytes();
					ostr.write(writeData);
					ostr.close();
				}
			}
		}
	}
       

    }  // upgradeAll
    

    private String fixUpString(String content, String oldUrl, String newUrl) {

	// fix up the string.
	String newContent = content;
	int p = content.indexOf("<ModelProvider>");
	if (p >= 0) {
		int pp = content.indexOf("</", p);
		String providerType = content.substring(p+15, pp);
		if (providerType.equalsIgnoreCase("spm2")) {
			// this is an StM provider, check url.
			p = content.indexOf("authenticationServiceURL=\"");
			if (p >= 0) {
				pp = content.indexOf("\"", p+26);
				String url = content.substring(p+26, pp);

				if (url != null && url.equals(oldUrl)) {
					newContent = content.substring(0, p+26) + newUrl + content.substring(pp); 
				}
			}
		}
	}
	return newContent;
   }
	
    
   private void debug(String s) {
      if (DEBUG)
      {
           println(s);
      }
   }
     
   private void info(String s) {
        println(s);
   }
   
   private void warn(String s) {
        println(s);
   }
   
   private void error(String s) {
        println(s);
   }

}  // class UpgradeStmMetadataRelease