 
package deploy.identities
import com.sas.metadata.MetadataObjects;
import deploy.JomaSupport;
import com.sas.metadata.MdObjectFactory;
import com.sas.iom.SASIOMDefs.GenericError;
import com.sas.metadata.MdObjectFactory;
import com.sas.metadata.MetadataUtil;
import com.sas.metadata.remote.AssociationList;
import com.sas.metadata.remote.Column;
import com.sas.metadata.remote.CMetadata;
import com.sas.metadata.remote.MdException;
import com.sas.metadata.remote.MdFactory;
import com.sas.metadata.remote.MdFactoryImpl;
import com.sas.metadata.remote.MdObjectStore;
import com.sas.metadata.remote.MetadataObjects;
import com.sas.metadata.remote.SoftwareComponent;
import com.sas.metadata.remote.IdentityGroup;
import com.sas.metadata.remote.Identity;
import java.lang.Exception;
import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.List;

 //----- RUN THE MIGRATION CODE -----
 
 JomaSupport joma = null;
 boolean DEBUG = true;
 int rc = 0;
 
                      
 
 try
 {
     
     String user = attributes.get("admin.user");
     String password = attributes.get("admin.pass");
     String host= attributes.get("omr.host");
     String port = attributes.get("omr.port");
      
     
     // The following maps the properties which are NOT migrated from 9.1.3 to 9.2
     // Either they are no longer supported or the properties require 9.2 settings
 
     if(DEBUG)
     {
       println("--- INPUT ATTRIBUTES ---");
       println("USER: " + user);
       println("OMR HOST: " + host);    
       println("OMR PORT: " + port);
     }
         
     joma = new JomaSupport();
     joma.connect(host, port, user, password);
     
     MigrateSPMIdentities53 migrate = new MigrateSPMIdentities53(joma);
     migrate.migrateIdentities();
 

 } catch (Throwable t) {
     rc = 1;
     project.setNewProperty("outputproperty", t.toString());    
     t.printStackTrace();    
 }
 finally
 {
   if(DEBUG) println("DONE.");
   if(joma != null)
     joma.disconnect();
 }
 
 project.setNewProperty("resultsproperty", "" + rc);
 
 // ----- END RUN THE MIGRATION CODE -----


class MigrateSPMIdentities53
{
    boolean DEBUG = true;
    JomaSupport joma;
    Properties lclz_props = null; 
    static final String VERSION = "1000000";
    
        
    public MigrateSPMIdentities53(JomaSupport joma)
    {
        println("entered MigrateSPMIdentities53()");
        this.joma = joma;
        println("exit MigrateSPMIdentities53()");
    }
    
    
    
    public void migrateIdentities()
    {
         String repositoryID = joma.getFoundationRepositoryId(); 
        
        def oldRoleToNewRoleMap = ['Scorecard Modeler' : 'Strategy Management Modeler',
                                   'Analyst' : 'Strategy Management Analyst',  
                                   'Scorecard Data Entry' : 'Strategy Management Data Entry',
                                   'Information Consumer' : 'Strategy Management Information Consumer'
                                     ]
    
        def oldGroupToNewGroupMap = ['SPM Users': 'Strategy Management Users',
                                     'Solutions Users': 'Strategy Management Users',
                                     'Solutions Administrators Group': 'Strategy Management Administrators Group'];

        def deleteRoles = ['Scorecard Modeler', 'Scorecard Data Entry'];

                       
        List alreadyExists;
        List deleteList = null;
        IdentityGroup oldGroup = null;
        IdentityGroup newGroup = null;
        String oldGroupName = null;
        String newGroupName = null;
        AssociationList newMemberList = null;

        for (aGroup in oldGroupToNewGroupMap)
        {

           oldGroupName   = aGroup.key;      // old group name
           newGroupName   = aGroup.value;    // new groupName
           
           try
           {
                oldGroup = (IdentityGroup) joma.getObjectsByName(null, MetadataObjects.IDENTITYGROUP, oldGroupName).get(0);
                newGroup = (IdentityGroup) joma.getObjectsByName(null, MetadataObjects.IDENTITYGROUP, newGroupName).get(0);
           }
           catch (Exception e) {};

           if (oldGroup == null)
           {
                   println("Warning: Old Group: " + oldGroupName + " not found in metadata repository.  Migration may already be completed");
                   continue;
           }
           else if (newGroup == null)
           {
                   println("Warning: New Group: " + newGroupName + " not found in metadata repository");
                   continue;
           }

           newMemberList = newGroup.getMemberIdentities();
           if (!newMemberList.contains(oldGroup))
           {  // Old groups need to be added to new to maintain permissions.
              newMemberList.add(oldGroup);
              newGroup.updateMetadataAll();
              println("Info: " + "Group: " + oldGroup + " been added as a member to group: " + newGroup.getName());
           }
           else 
           {
               println("Info: " + "Group: " + oldGroup + " is already a member of: " + newGroup.getName());
           }
        }


        for (aRole in oldRoleToNewRoleMap)
        {  // Old role membership needs to be transferred to new role membershkp

           String oldRoleName   = aRole.key;      // old
           String newRoleName   = aRole.value;    // role name
           IdentityGroup oldRole = null;
           IdentityGroup newRole = null;
           AssociationList memberList;
           AssociationList newmList;



           try
           {
                   oldRole = (IdentityGroup) joma.getObjectsByName(null, MetadataObjects.IDENTITYGROUP, oldRoleName).get(0);
                   newRole = (IdentityGroup) joma.getObjectsByName(null, MetadataObjects.IDENTITYGROUP, newRoleName).get(0);
           }
           catch (Exception e) {};

           if (oldRole == null)
           {
                   println("Warning: Old Role: " + oldRoleName + " not found in metadata repository.  Migration may already be completed");
                   continue;
           }
           else if (newRole == null)
           {
                   println("Warning: New Role: " + newRoleName + " not found in metadata repository");
                   continue;
           }

           memberList = oldRole.getMemberIdentities();
           newmList = newRole.getMemberIdentities();
           Identity member = null;

            for (Iterator iter = memberList.iterator(); iter.hasNext();)
            {
                member = (Identity) iter.next();
                if (!newmList.contains(member))
                {
                    newmList.add(member)
                    println("Info: " + oldRoleName + " role member: " + member.getName() + " been added as a member to role: " + newRoleName);
                    
                }
                
            }
     
               memberList.clear();
               newRole.setMemberIdentities(newmList);
               oldRole.setMemberIdentities(memberList);
               newRole.updateMetadataAll();
               oldRole.updateMetadataAll();
       } 

       deleteList = new ArrayList(); 
       CMetadata cm = null;
       for (aRole in deleteRoles)
       {        
          List ml =  joma.getObjectsByName(null, MetadataObjects.IDENTITYGROUP, aRole);
          cm = null;
          if (ml != null && ml.size() > 0)
        	  cm = ml.get(0);
          if (cm != null)
              deleteList.add(cm);   
          else println("Warning: Role/Group name to delete not found: " + aRole );
          
       }   
       
       showDeleteList(deleteList);
       if (deleteList.size() == 0)
    	   println("Warning: No Roles/Groups to delete");
       // else joma.deleteObjects(deleteList);  // Clean up old group 

   }
        
  
   public void showDeleteList(List deleteList)
   {
         for(Iterator i = deleteList.iterator(); i.hasNext();)
         {
               CMetadata obj = (CMetadata) i.next();
               println("Deleting: " + obj.getName());            
         }        
   }

}