
package deploy.identities


import deploy.AddMetadata
import deploy.MetadataExecution
import deploy.JomaSupport;

import com.sas.metadata.MdObjectFactory;
import com.sas.iom.SASIOMDefs.GenericError;

import com.sas.metadata.MetadataObjects;
import com.sas.metadata.MdObjectFactory;
import com.sas.metadata.MetadataUtil;
import com.sas.metadata.remote.AssociationList;
import com.sas.metadata.remote.CMetadata;
import com.sas.metadata.remote.MdException;
import com.sas.metadata.remote.MdFactory;
import com.sas.metadata.remote.MdFactoryImpl;
import com.sas.metadata.remote.MdObjectStore;
import com.sas.metadata.remote.MetadataObjects;
import com.sas.metadata.remote.SoftwareComponent;
import com.sas.metadata.remote.IdentityGroup;

import java.lang.Exception;
import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;



 //----- RUN THE MIGRATION CODE -----

 JomaSupport joma = null;
 boolean DEBUG = true;
 int rc = 0;

 // This is the 5.3 to 5.4 Identity migration.
 // This code adds 2 new groups/roles, Data Entry Approver role, Data Entry Portlet Administrator
 //
 // Deletes then re-adds all the capabilities




 try
 {

     String user = attributes.get("admin.user");
     String password = attributes.get("admin.pass");
     String host= attributes.get("omr.host");
     String port = attributes.get("omr.port");
     String applicationsActionsFile = attributes.get("application.actions.file");
     String spmmidtier_localization_res_path = attributes.get("spmmidtier.localization.res.path");


     if(DEBUG)
     {
       println("--- INPUT ATTRIBUTES ---");
       println("USER: " + user);
       println("OMR HOST: " + host);
       println("OMR PORT: " + port);
       println("APPLICATIONACTIONSFILE: " + applicationsActionsFile);
       println("SPMMIDTIER LOCALIZATION RES PATH: " + spmmidtier_localization_res_path.toString());

     }

     joma = new JomaSupport();
     joma.connect(host, port, user, password);

     String foundReposId = joma.getFoundationRepositoryId();

     MigrateSPMIdentities migrate = new MigrateSPMIdentities(joma, spmmidtier_localization_res_path);
     migrate.updateIdentities();
     migrate.updateCapabilities(foundReposId, host, port, user, password, applicationsActionsFile);


 } catch (Throwable t) {
     rc = 1;
     project.setNewProperty("outputproperty", t.toString());
     t.printStackTrace();
 }
 finally
 {
   if(DEBUG) println("DONE.");
   if(joma != null)
     joma.disconnect();
 }

 project.setNewProperty("resultsproperty", "" + rc);

 // ----- END RUN THE MIGRATION CODE -----


class MigrateSPMIdentities
{
    boolean DEBUG = true;
    JomaSupport joma;
    Properties lclz_props = null;
    static final String VERSION = "1000000";
    static DUMMY_ROLE = "DUMMY";


    public MigrateSPMIdentities(JomaSupport joma, String localizationFilePath)
    {
    	// println("entered MigrateSPMIdentities()");
        this.joma = joma;
        lclz_props = new Properties();

	FileInputStream propsIn = new FileInputStream(localizationFilePath);
	lclz_props.load(propsIn);
	println("exit MigrateSPMIdentities()");

    }

    public void updateCapabilities(
		               String foundationId,
                       String omr_host,
                       String omr_port,
                       String admin_user,
                       String admin_pass,
                       String applicationsActionsFile
                       )
    {
		String repositoryID = joma.getFoundationRepositoryId();
		CMetadata cMd = null;
		String result;
		MetadataExecution executor = new MetadataExecution(omr_host, omr_port, admin_user, admin_pass);

		AddMetadata adder = new AddMetadata();

		Hashtable vars = new Hashtable();
		lclz_props.each() { key, value -> vars.put("@"+key+"@", value) };  // Setup localization substitution


		String toExecute = adder.addRawMetadata(foundationId, applicationsActionsFile, vars, executor);
	    println(toExecute);


		// Delete the old releases Application actions


	    List trees = joma.getMetadataObjects(null, "Tree", "Tree[@Name='ApplicationActions'][SoftwareComponents/SoftwareComponent[@ClassIdentifier='spmmidtier']]");
	    if (trees == null || trees.size() != 1)
	    {
           	println("upgradeCapabilites failed:  ApplicationAction tree is null or not just one tree found " + trees);
           	throw new Exception("upgradeCapabilites:  Strategy Management ApplicationAction tree is null or not just one tree found");
		}

		cMd = trees.get(0);
        ArrayList deleteList = new ArrayList(1)
        addToDeleteList(cMd, "ApplicationActions", deleteList);
        showDeleteList(deleteList);
        joma.deleteObjects(deleteList);  // delete old Application Actions  so new one can be imported

		result = executor.execute(toExecute, "Tree");

		println("STM Capabilities Update executed, result: " + result);
	}



    public void updateIdentities()
    {

		String repositoryID = joma.getFoundationRepositoryId();

		// Define new roles, and groups and their relation ship

		def mapNameToDisplayName = [
			'Strategy Management Data Entry Approver'       	: lclz_props.get("I18N_ScorecardDataEntryApproverDisplayName.txt"),
			'Strategy Management Data Entry Approver Group' 	: lclz_props.get("I18N_ScorecardDataEntryApproverGroupDisplayName.txt"),
			'Strategy Management Data Entry Portlet Admin Group' : lclz_props.get("I18N_ScorecardDataEntryPAGroupDisplayName.txt")

		 ];

		def mapNameToDesc = [
			'Strategy Management Data Entry Approver' 			 : lclz_props.get("I18N_ScorecardDataEntryApproverDesc.txt"),
			'Strategy Management Data Entry Approver Group' 	 : lclz_props.get("I18N_ScorecardDataEntryApproverGroupDesc.txt"),
			'Strategy Management Data Entry Portlet Admin Group' : lclz_props.get("I18N_ScorecardDataEntryPAGroupDesc.txt")
		 ];




		def groupToRoleMap = [
			'Strategy Management Data Entry Approver Group' : 'Strategy Management Data Entry Approver',
			'Strategy Management Data Entry Portlet Admin Group' : DUMMY_ROLE
		];


		for (aGroup in groupToRoleMap)
		{

			String key   = aGroup.key;      // group name
			String value = aGroup.value;    // role name
		   // println("Key: " + key + ", Value: " + value);

			IdentityGroup role = null;
			IdentityGroup group = null;
			IdentityGroup group2 = null;

			try
			{
				group = (IdentityGroup) joma.getObjectsByName(repositoryID, MetadataObjects.IDENTITYGROUP, key).get(0);
			}
			catch (Exception e) {};

			if (group != null)
			{  // already defined

				continue;
			}


			CMetadata fixture = joma.createObject(repositoryID, MetadataObjects.IDENTITYGROUP, 	key);
			println(fixture.class.getName());
			group = (IdentityGroup) fixture;
			group.setDisplayName(mapNameToDisplayName.get(key)?:key); //elvis operator in groovy
			group.setGroupType("GROUP");
			group.setPublicType("UserGroup");
			group.setUsageVersion(VERSION);
			group.setDesc(mapNameToDesc.get(key)?:"");
			group.updateMetadataAll();

			if (!DUMMY_ROLE.equals(value))
			{
				fixture = joma.createObject(repositoryID, MetadataObjects.IDENTITYGROUP, 	value);
				println(fixture.class.getName());
				role = (IdentityGroup) fixture;
				role.setDisplayName(mapNameToDisplayName.get(value)?:value); //elvis operator in groovy
				role.setGroupType("ROLE");
				role.setPublicType("Role");
				role.setUsageVersion(VERSION);
				role.setDesc(mapNameToDesc.get(value)?:"");

				AssociationList ident_groups = role.getMemberIdentities();
				ident_groups.clear();
				ident_groups.add(group);
				role.setMemberIdentities(ident_groups);  // Make the group a member
				role.updateMetadataAll();
				println("Added role " + value + "  as " + role.getId() + " DisplayName is " + role.getDisplayName());
				println("Added group " + key  + " as " + group.getId() + " DisplayName is " + group.getDisplayName());
				println("Added member " + key + " as  member of " + role.getDisplayName());

			}
			else
			{
				if ('Strategy Management Data Entry Portlet Admin Group'.equals(key))
				{
					AssociationList ident_groups = group.getMemberIdentities();
					ident_groups.clear();
					group2 = (IdentityGroup) joma.getObjectsByName(repositoryID, MetadataObjects.IDENTITYGROUP, 'Strategy Management Data Entry Group').get(0);
					ident_groups.add(group2);
					group2 = (IdentityGroup) joma.getObjectsByName(repositoryID, MetadataObjects.IDENTITYGROUP, 'Strategy Management Data Entry Approver Group').get(0);
					ident_groups.add(group2);
					group.setMemberIdentities(ident_groups);  // Make the group a member
					group.updateMetadataAll();
				}

				println("Added group "+ key + " as " + group.getId() + " DisplayName is " + group.getDisplayName());
			}
		} // for (aGroup in associations)

    }

    public String addToDeleteList(CMetadata mdObj, String mdName, ArrayList deleteList)
    {

        if (mdObj == null || mdName == null || mdName.length() ==0)
        {
            warn("\nWARN: addToDeletList failed for metaObj = " + mdObj + " Name = " +  mdName);
            return mdname;
        }

        String oldName = mdObj.getName();
        String type = mdObj.getCMetadataType();

        if (oldName.equals(mdName))
        {
                // info("\nINFO: Metadata " + type + " object: " + mdName + "  has been added to delete list");
                deleteList.add(mdObj);
        }
        else
        {
            info("\nINFO: Metadata " + type + " object: " + oldName + "  does not match requested deletion for object name: " + mdName);
        }

    }

    public void showDeleteList(List deleteList)
    {
          for(Iterator i = deleteList.iterator(); i.hasNext();)
          {
                CMetadata obj = (CMetadata) i.next();
                println("Deleting: " + obj.getName());
          }
    }

}