 
package deploy.identities

import com.sas.metadata.MetadataObjects;
import deploy.JomaSupport;
import com.sas.metadata.MdObjectFactory;
import com.sas.iom.SASIOMDefs.GenericError;
import com.sas.metadata.MdObjectFactory;
import com.sas.metadata.MetadataUtil;
import com.sas.metadata.remote.AssociationList;
import com.sas.metadata.remote.Column;
import com.sas.metadata.remote.CMetadata;
import com.sas.metadata.remote.PhysicalTable;
import com.sas.metadata.remote.TextStore;
import com.sas.metadata.remote.MdException;
import com.sas.metadata.remote.MdFactory;
import com.sas.metadata.remote.MdFactoryImpl;
import com.sas.metadata.remote.MdObjectStore;
import com.sas.metadata.remote.MetadataObjects;
import com.sas.metadata.remote.SoftwareComponent;
import com.sas.metadata.remote.IdentityGroup;
import java.lang.Exception;
import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;



 //----- RUN THE MIGRATION CODE -----
 
 JomaSupport joma = null;
 boolean DEBUG = true;
 int rc = 0;
 
                      
 
 try
 {
     
     String user = attributes.get("admin.user");
     String password = attributes.get("admin.pass");
     String host= attributes.get("omr.host");
     String port = attributes.get("omr.port");
     String spmmidtier_localization_res_path = attributes.get("spmmidtier.localization.res.path");
      
     
     // The following maps the properties which are NOT migrated from 9.1.3 to 9.2
     // Either they are no longer supported or the properties require 9.2 settings
 
     if(DEBUG)
     {
       println("--- INPUT ATTRIBUTES ---");
       println("USER: " + user);
       println("OMR HOST: " + host);    
       println("OMR PORT: " + port);
       println("SPMMIDTIER LOCALIZATION RES PATH: " + spmmidtier_localization_res_path.toString());
     }
         
     joma = new JomaSupport();
     joma.connect(host, port, user, password);
     
     MigrateSPMIdentities migrate = new MigrateSPMIdentities(joma, spmmidtier_localization_res_path);
     migrate.migrateIdentities();
 

 } catch (Throwable t) {
     rc = 1;
     project.setNewProperty("outputproperty", t.toString());	
     t.printStackTrace();    
 }
 finally
 {
   if(DEBUG) println("DONE.");
   if(joma != null)
     joma.disconnect();
 }
 
 project.setNewProperty("resultsproperty", "" + rc);
 
 // ----- END RUN THE MIGRATION CODE -----


class MigrateSPMIdentities
{
    boolean DEBUG = true;
    JomaSupport joma;
    Properties lclz_props = null; 
    static final String VERSION = "1000000";
    
        
    public MigrateSPMIdentities(JomaSupport joma, String localizationFilePath)
    {
    	// println("entered MigrateSPMIdentities()");
        this.joma = joma;
        lclz_props = new Properties();
        
	FileInputStream propsIn = new FileInputStream(localizationFilePath);
	lclz_props.load(propsIn);
	println("exit MigrateSPMIdentities()");

    }
    
    
    
    public void migrateIdentities()
    {
	
	String repositoryID = joma.getFoundationRepositoryId(); 
	
	def mapNameToDisplayName = [
	    'Scorecard Modeler Group'        : lclz_props.get("I18N_ScorecardModelerGroupDisplayName.txt"),
	    'Scorecard Data Entry Group'     : lclz_props.get("I18N_ScorecardDataEntryGroupDisplayName.txt"),
	    'Solutions Administrators Group' : lclz_props.get("I18N_SystemAdministratorsGroupDisplayName.txt"),
	    'Analyst Group'                  : lclz_props.get("I18N_AnalystGroupDisplayName.txt"),
	    'Information Consumer Group'     : lclz_props.get("I18N_InformationConsumerGroupDisplayName.txt"),	           
		
	    'Scorecard Modeler'              : lclz_props.get("I18N_ScorecardModelerDisplayName.txt"),
	    'Scorecard Data Entry'           : lclz_props.get("I18N_ScorecardDataEntryDisplayName.txt"),
	    'System Administrator'           : lclz_props.get("I18N_SystemAdministratorDisplayName.txt"),
	    'Analyst'                        : lclz_props.get("I18N_AnalystDisplayName.txt"),
	    'Information Consumer'           : lclz_props.get("I18N_InformationConsumerDisplayName.txt")	           
	 ];

	def mapNameToDesc = [
	    'Scorecard Modeler Group'        : lclz_props.get("I18N_ScorecardModelerGroupDesc.txt"),
	    'Scorecard Data Entry Group'     : lclz_props.get("I18N_ScorecardDataEntryGroupDesc.txt"),
	    'Solutions Administrators Group' : lclz_props.get("I18N_SolutionsAdministratorGroupDesc.txt"),
	    'Analyst Group'                  : lclz_props.get("I18N_AnalystGroupDesc.txt"),
	    'Information Consumer Group'     : lclz_props.get("I18N_InformationConsumerGroupDesc.txt"),	           
		
	    'Scorecard Modeler'              : lclz_props.get("I18N_ScorecardModelerDesc.txt"),
	    'Scorecard Data Entry'           : lclz_props.get("I18N_ScorecardDataEntryDesc.txt"),
	    'System Administrator'           : lclz_props.get("I18N_SystemAdministratorDesc.txt"),
	    'Analyst'                        : lclz_props.get("I18N_AnalystDesc.txt"),
	    'Information Consumer'           : lclz_props.get("I18N_InformationConsumerDesc.txt")	           
	 ];




        def groupToRoleMap = ['Solutions Administrators Group': 'System Administrator',
		              'Scorecard Modeler Group'       : 'Scorecard Modeler',
		              'Analyst Group'                 : 'Analyst',
		              'Scorecard Data Entry Group'    : 'Scorecard Data Entry',
		              'Information Consumer Group'    : 'Information Consumer'
		             ];
		       
	List alreadyExists;
	
		       
	for (aGroup in groupToRoleMap)
	{
	   	
	   String key   = aGroup.key;      // group name
	   String value = aGroup.value;    // role name
	   // println("Key: " + key + ", Value: " + value);
	  	
	   IdentityGroup role = null;
	   IdentityGroup group = null;
	   
	   alreadyExists = joma.getObjectsByName(repositoryID, MetadataObjects.IDENTITYGROUP, value);
	   
	   if (alreadyExists.isEmpty())
	   {
   	      println("Role: " + value + " not found in metadata repository");
   	      continue;   // Without the role to process we can't do anything except go on to the next
           }
            
           role = (IdentityGroup) alreadyExists.get(0); 
	   
	   try
	   {
	   	group = (IdentityGroup) joma.getObjectsByName(repositoryID, MetadataObjects.IDENTITYGROUP, key).get(0);
	   }
	   catch (Exception e) {};
	       
	   if (group != null)
	   {  // Delete Same Named Group since we are going to transform its matching role into a group
	      
               AssociationList gMems = group.getMemberIdentities();
               if (!gMems.isEmpty())
               {
               	   println("Group: " + key + " has associated member identities and has previously been migrated");
               	   continue;
               }
               else 
               {
                   List deleteList = new ArrayList();
                   deleteList.add(group);
                   showDeleteList(deleteList);
	           joma.deleteObjects(deleteList);  // Clean up old group
	           group = null;
	       }    

           }
           
           if (group == null) 
           {  // Transform the existing role into a group to preserve its membership and permissions gorp
           
	       role.setName(key);  // rename
	       role.setDisplayName(mapNameToDisplayName.get(key)?:key); //elvis operator in groovy
	       role.setGroupType("Group");
	       role.setPublicType("UserGroup");
	       role.setUsageVersion(VERSION);
	       role.setDesc(mapNameToDesc.get(key)?:"");
	       
	       AssociationList protoList = role.getUsedByPrototypes();
	       if (!protoList.isEmpty())
	       {
	           CMetadata proto = protoList.get(0);
	           String protoName = proto.getName();
	           if (protoName.equals(value + " Permissions Tree"))
	           {
	              String newName = key + " Permissions Tree";
	              println("Prototype newName: " + newName);
	              proto.setName(newName);
	           }   
	       }
	       
	       role.updateMetadataAll();
	       println("Role: " + value + " transformed to Group: " + key + " as "+role.getId() +" DisplayName is "+role.getDisplayName());
	       
	       group = role;  
	       
	    }
	    
	    // At this point there should be a single IdentityGroup (formerly the role), and we need to
	    // create a new role
	    
	    
	    alreadyExists =  joma.getObjectsByName(repositoryID, MetadataObjects.IDENTITYGROUP, value);
	    if (!alreadyExists.isEmpty())
	    {
	        String msg = "Error in migration processing role: " + value + " still exists in metadata, and should have become a group";
	    	throw new Exception(msg);
	    }
	    
	    CMetadata fixture = joma.createObject(repositoryID, MetadataObjects.IDENTITYGROUP, 	value);
	    println(fixture.class.getName());
	    role = (IdentityGroup) fixture;
	    role.setDisplayName(mapNameToDisplayName.get(value)?:value); //elvis operator in groovy
	    role.setGroupType("ROLE");
	    role.setPublicType("Role");
	    role.setUsageVersion(VERSION);
	    role.setDesc(mapNameToDesc.get(value)?:"");

	    AssociationList ident_groups = role.getMemberIdentities();
	    ident_groups.clear();
	    ident_groups.add(group);
	    role.setMemberIdentities(ident_groups);  // Make the group (which was previously a role) a member
	    role.updateMetadataAll();

	    println("Added role "+value+" as "+role.getId()+" DisplayName is "+role.getDisplayName());
            
	} // for (aGroup in associations)

   }
  
   public void showDeleteList(List deleteList)
   {
         for(Iterator i = deleteList.iterator(); i.hasNext();)
         {
               CMetadata obj = (CMetadata) i.next();
               println("Deleting: " + obj.getName());            
         }        
   }

}