package deploy.migrateSASSDM;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;

import java.sql.Connection;
import java.sql.DriverManager;

import deploy.CommonUtils;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;

import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

int rc = CommonUtils.RC_OK;
CommonUtils.init(self);

CommonUtils.info("----> SASSDM Formula MIGRATION STARTED");

String resultFileNameProperty = attributes.get("result.filename");
String resultsProperty = attributes.get("resultproperty");


//---- SET UP THE TEMP FILE ----
BufferedWriter writer = null;
File outputFile = null;
String outputFileName="";
try
{
    outputFile = File.createTempFile("MIGRATE_SASSDM_FORMULA", ".sql");
    writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(outputFile),"UTF-8"));
    outputFileName = outputFile.getAbsolutePath();
    CommonUtils.info("RESULTS WILL BE WRITTEN TO: " + outputFileName);    
}
catch (Exception e)
{
    rc = CommonUtils.RC_ERROR;
    CommonUtils.error("A problem occurred while creating a temporary file:  " + e.getMessage());    
}

if(writer != null)
{
    rc = MigrateFormulaImpl.generateFormulaSQL(attributes, writer);   
}

project.setNewProperty(resultsProperty, "" + rc);
project.setNewProperty(resultFileNameProperty, outputFileName);

CommonUtils.info("<---- SASSDM FORMULA MIGRATION COMPLETE RC=" + rc);


public class MigrateFormulaImpl
{
    
    private static boolean isTableServer = false;
    
    private static final int MAX_RECORDS_PER_INSERT = 10;
    
    private static String spm_SoftwareComponentName_old = "Strategic Performance Management";
    private static String spm_SoftwareComponentName_new = "Strategy Mgmt 5.3";

    static Integer ONE = new Integer(1);
    static Integer ZERO = new Integer(0);
    static Integer FF = new Integer(255); 

    static Integer  USERIDENTITYTYPE = new Integer(1);
    static Integer  GROUPIDENTITYTYPE = new Integer(2);
    


    
    public static int generateFormulaSQL(Map attributes, BufferedWriter writer)
    {
      
        int rc = CommonUtils.RC_OK;
        
        Statement statement = null;
        ResultSet rs = null;
        Connection connection = null;
        
        int MAX_FORMULA_PROC = 10000;
        
        try
        {           
            String user = attributes.get("mysql.user");
            String password = attributes.get("mysql.pass");
            String url= attributes.get("mysql.url");
            String driver = attributes.get("mysql.driver");
            String jarDir = attributes.get("mysql.jdbc.jar");
            String targetDBType = attributes.get("spm.db.type");
            String targetDBName = attributes.get("spm.db.name");
            if ((targetDBName == null) || (targetDBName.length() == 0))
                 targetDBName = "spm";
            
            // CommonUtils.info("***DEBUG*** (user, password, url, driver, jarDir, targetDBType, targetDBName) " + user + ", " + password + ", " + url + ", " + driver + ", " + jarDir + ", " + targetDBType + ", " + targetDBName);
                        
            
            File jar = new File(jarDir);
            if (jar == null)
                return CommonUtils.RC_ERROR;
            
            //--------------------------------------------------
            //---- ADD THE MYSQL JAR FILE TO CLASS PATH --------
            //--------------------------------------------------
            ClassLoader loader = Thread.currentThread().getContextClassLoader();
            loader.parent.addURL(jar.toURI().toURL());
   
            Class.forName(driver);
            connection = DriverManager.getConnection(url, user, password);

            CommonUtils.info("START MIGRATION FOR SASSDM SAS_USER_ACTIONS");

            
            /*---------------------------------------------------------------------------------------+
            
            Convert the sassdm sas_formula to spm sas_formula
            
            The following is the column mappings from the sassdm formula table to the spm table.
            FORMULA_ID          --> FORMULA_ID
            NAMESPACE           --> NAMESPACE
            FORMULA_TYPE_ENUM,  --> FORMULA_TYPE_ENUM
            TARGET,             --> TARGET
            RESULT_TYPE,        --> RESULT_TYPE
            FORMULA_TEXT,       --> FORMULA_TEXT
            BYTE_CODE,          --> BYTE_CODE
                                               
            +-----------------------------------------------------------------------------------------*/

            String sassdm_query = null;
 
            sassdm_query = "SELECT MIN(FORMULA_ID) FROM sassdm.sas_formula s WHERE NAMESPACE='S';" ;
            statement = connection.createStatement();
	    rs = statement.executeQuery(sassdm_query);
            rs.next();
 	    int minFormulaId  = Integer.parseInt(rs.getObject(1).toString());

            sassdm_query = "SELECT MAX(FORMULA_ID) FROM sassdm.sas_formula s WHERE NAMESPACE='S';" ;
            statement = connection.createStatement();
 	    rs = statement.executeQuery(sassdm_query);
            rs.next();
	    int maxFormulaId  = Integer.parseInt(rs.getObject(1).toString());
	    	    
            // insert the next formula id key value in to the keygen table.
            StringBuffer sas_keygen = new StringBuffer();
            sas_keygen.append("\nINSERT INTO sas_keygen (IDENTIFIER_CD, KEY_CNT) VALUES \n('spmformula', ");
	    sas_keygen.append(maxFormulaId+1);
	    sas_keygen.append(");");
	    writer.write(sas_keygen.toString());
	    
            for (int formulaIx = minFormulaId; formulaIx <= maxFormulaId; formulaIx = formulaIx + MAX_FORMULA_PROC)
            {
                    int IXMAX = formulaIx + MAX_FORMULA_PROC;
		    sassdm_query = "SELECT FORMULA_ID, NAMESPACE, FORMULA_TYPE_ENUM, TARGET, RESULT_TYPE, FORMULA_TEXT, BYTE_CODE FROM sassdm.sas_formula s WHERE NAMESPACE='S'" + 
		                   " AND (FORMULA_ID >= " + formulaIx + ") AND (FORMULA_ID < "  + IXMAX + ") ;" ;                       
		    statement = connection.createStatement();
		    CommonUtils.info(sassdm_query);
		    rs = statement.executeQuery(sassdm_query);
		    processRS(rs, USERIDENTITYTYPE, writer);
		    close(rs);
		    close(statement);

		    /*---------------------------------------------------------------------------------------+

		    Convert the sassdm sas_formula_operand to spm sas_formula_operand

		    The follow is the column mappings from the sassdm table to the spm table
		    FORMULA_ID        --> FORMULA_ID
		    NAMESPACE         --> NAMESPACE
		    ORDINAL,          --> ORDINAL
		    OPERAND_GROUP,    --> OPERAND_GROUP
		    OPERAND_ELEMENT,  --> OPERAND_ELEMENT
		    TABLE_NM,         --> TABLE_NM

		    +-----------------------------------------------------------------------------------------*/

		    sassdm_query = "SELECT FORMULA_ID, NAMESPACE, ORDINAL, OPERAND_GROUP, OPERAND_ELEMENT, TABLE_NM FROM sassdm.sas_formula_operand s WHERE NAMESPACE='S' " +                        
	                           " AND (FORMULA_ID >= " + formulaIx + ") AND (FORMULA_ID < "  + IXMAX + ") ;" ; 
	            statement = connection.createStatement();
	            CommonUtils.info(sassdm_query);
		    rs = statement.executeQuery(sassdm_query);
		    processRS(rs, GROUPIDENTITYTYPE, writer); 
		    close(rs);
		    close(statement);
            }

        }
        catch (Exception e)
        {
            rc = CommonUtils.RC_ERROR;
            CommonUtils.error("A problem occurred while generating migration data from SASSDM:  " + e.getMessage());
            e.printStackTrace();
        }
        finally
        {        
            close(writer);
            close(rs);
            close(statement);
            close(connection);            
        }
        
        return rc;        
    }
    
 
    public static final int processRS(ResultSet rs, Integer identityType, BufferedWriter writer) {
    
        StringBuffer sas_formula_update = new StringBuffer();
                       
        int sas_formula_count = 0;
                  
        String rulePermString = null;
        int[] deny = new int[1];

        String identityColumn = "GROUP_IDENTIFIER";
        
        if (identityType == USERIDENTITYTYPE)
                identityColumn = "USER_IDENTIFIER";

        String tableName = rs.getMetaData().getTableName(1);
        
        int formulaId;
           
        while(rs.next()) {
            
            if(sas_formula_count%MAX_RECORDS_PER_INSERT == 0){
                if(sas_formula_count > 0){
                    sas_formula_update.append(";");
                    writer.write(sas_formula_update.toString());
                    writer.newLine();       
                    sas_formula_update.setLength(0); // reduce memory load
                 }
                 
                sas_formula_update.append("\nINSERT INTO ");
                sas_formula_update.append(tableName);
                if ("sas_formula".equals(tableName))
                        sas_formula_update.append(" (FORMULA_ID, NAMESPACE, FORMULA_TYPE_ENUM, TARGET, RESULT_TYPE, FORMULA_TEXT, BYTE_CODE) VALUES \n(");
                else if ("sas_formula_operand".equals(tableName))
                        sas_formula_update.append(" (FORMULA_ID, NAMESPACE, ORDINAL, OPERAND_GROUP, OPERAND_ELEMENT, TABLE_NM) VALUES \n(");

            }
            else    sas_formula_update.append(",\n(");
               
            deny[0]  = 0;

            
            if ("sas_formula".equals(tableName)) {
                    formulaId = rs.getInt("FORMULA_ID");  
                      
                    processObject(sas_formula_update, formulaId, "0", false);  
                    processObject(sas_formula_update, rs.getObject("NAMESPACE"), "", false);        
                    processObject(sas_formula_update, rs.getObject("FORMULA_TYPE_ENUM"), "", false); 
                    processObject(sas_formula_update, rs.getObject("TARGET"), null, true);               
                    processObject(sas_formula_update, fixResultType(rs.getObject("RESULT_TYPE")), "", false);               
                    processObject(sas_formula_update, fixFormulaText(rs.getObject("FORMULA_TEXT")), null, false);               
                    processObject(sas_formula_update, rs.getObject("BYTE_CODE"), null, false, false);               
            }
            else if ("sas_formula_operand".equals(tableName)) {
                    processObject(sas_formula_update, rs.getObject("FORMULA_ID"), "0", false);  
                    processObject(sas_formula_update, rs.getObject("NAMESPACE"), "0", false);        
                    processObject(sas_formula_update, rs.getObject("ORDINAL"), "", false); 
                    processObject(sas_formula_update, rs.getObject("OPERAND_GROUP"), null, true);               
                    processObject(sas_formula_update, rs.getObject("OPERAND_ELEMENT"), null, true);               
                    processObject(sas_formula_update, rs.getObject("TABLE_NM"), null, true, false);               
            }
                
                
            sas_formula_update.append(")");
            sas_formula_count++;                
        }  
        
           
        CommonUtils.info(sas_formula_count + " RECORDS WILL BE MIGRATED FOR SASSDM TABLE " + tableName + " .");
   
        if(sas_formula_count > 0)
        {
            writer.write(sas_formula_update.toString());
            writer.write(";");
            writer.newLine();   
            sas_formula_update.setLength(0); // reduce memory load          
        }  

        return sas_formula_count;
    }   

    
    
    public static String fixResultType(String origResultType) {
      if ("O".equalsIgnoreCase(origResultType))
        return "N";
      else
        return origResultType;
    }
    
    
    public static String fixFormulaText(String origFormulaText) {
      int p = origFormulaText.indexOf("tableMap");
      
      if (p < 0)
        return origFormulaText;
        
      // first change tableMap to tableN.
      Pattern tmpPattern = Pattern.compile("tableMap");
      Matcher matcher = tmpPattern.matcher(origFormulaText);
      origFormulaText = matcher.replaceFirst("tableN");
      
      // now look for the second comma.
      p = origFormulaText.indexOf(",");
      if (p >= 0) {
        p = origFormulaText.indexOf(",", p+1);
        if (p >= 0) {
          // locate opening quote.
          int pQuoteLeft = origFormulaText.indexOf("'", p+1);
          // locate the parameter list and get the extraneous params.
          int pSpace = origFormulaText.indexOf(" ", pQuoteLeft+1);
          int pQuoteRight = origFormulaText.indexOf("'", pSpace+1);

          if (pQuoteRight > pSpace) {
            // get rid of the extra parameter(s).
            String removeStr = origFormulaText.substring(pSpace, pQuoteRight);
            tmpPattern = Pattern.compile(removeStr);
            matcher = tmpPattern.matcher(origFormulaText);
            matcher.region(pQuoteLeft, pQuoteRight);
            origFormulaText = matcher.replaceFirst("");
          }
        }
      }
      return origFormulaText;
    }
 
 
    public static final void close(BufferedWriter writer)
    {
        if (writer != null)
        {
            try
            {
                writer.close();
            }
            catch (IOException ex)
            {              
            }
        }
    }
    
   
    public static final void close(ResultSet rs)
    {
        if (rs != null)
        {
            try
            {
                rs.close();
            }
            catch (SQLException ex)
            {              
            }
        }
    }

    
    public static final void close(Statement stmt)
    {
        if (stmt != null)
        {
            try
            {
                stmt.close();
            }
            catch (SQLException ex)
            {               
            }
        }
    }

    
    public static final void close(Connection con)
    {
        if (con != null)
        {
            try
            {
                con.close();
            }
            catch (SQLException ex)
            {                
            }
        }
    }
    
    
    public static void processObject(StringBuffer sb, Object obj, Object theDefault, boolean allowNull)
    {
        processObject(sb, obj, theDefault, allowNull, true);   
    }
    
    
    public static void processObject(StringBuffer sb, Object obj, Object theDefault, boolean allowNull, boolean comma)
    {       
        String value = "";
        
        if(obj == null && allowNull)
        {
            sb.append("NULL");
        }
        else if(obj == null)
        {
            appendValue(sb, theDefault);
        }
        else
        {   
            appendValue(sb, obj);
        }
        
        if(comma)
            sb.append(", ");        
    }
    
    public static String appendValue(StringBuffer sb, Object obj)
    {
        if(obj instanceof Integer)
        {
            sb.append(obj.toString());                
        }
        else if (obj instanceof byte[]) {
            // this is the Blob / compiled formula.
            // put a minimal place holder since all formulas need to be re-compiled.
            sb.append(0xFF);
        }
        else if (obj instanceof Timestamp && isTableServer)
        {            
            sb.append("timestamp '");            
            sb.append(escape(obj.toString()));
            sb.append("'");
        }
        else if (obj instanceof String && ((String) obj).startsWith("@"))
        { 
                sb.append(obj)
        }       
        else
        {
            sb.append("'");            
            sb.append(escape(obj.toString()));
            sb.append("'");                
        }        
    }
    
    
    public static String escape(String str)
    {
        String result = "";
        if(isTableServer)
            result = str.replaceAll("\\\\", "\\\\\\\\").replaceAll("\n", "\\\\n").replaceAll("\r", "\\\\r").replaceAll("\"", "\\\\\"").replaceAll("'", "''").replaceAll("\0", "\\\\0");
        else
            result = str.replaceAll("\\\\", "\\\\\\\\").replaceAll("\n", "\\\\n").replaceAll("\r", "\\\\r").replaceAll("\"", "\\\\\"").replaceAll("'", "\\\\'").replaceAll("\0", "\\\\0");
        
        return result;
    }
}