package deploy.migrateSASSDM;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;

import java.sql.Connection;
import java.sql.DriverManager;

import deploy.CommonUtils;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;

import java.util.Map;


int rc = CommonUtils.RC_OK;
CommonUtils.init(self);

CommonUtils.info("----> SASSDM Security MIGRATION STARTED");

String resultFileNameProperty = attributes.get("result.filename");
String resultsProperty = attributes.get("resultproperty");

//---- SET UP THE TEMP FILE ----
BufferedWriter writer = null;
File outputFile = null;
String outputFileName="";
try
{
    outputFile = File.createTempFile("MIGRATE_SASSDM_SECURITY", ".sql");
    writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(outputFile),"UTF-8"));
    outputFileName = outputFile.getAbsolutePath();
    CommonUtils.info("RESULTS WILL BE WRITTEN TO: " + outputFileName);    
}
catch (Exception e)
{
    rc = CommonUtils.RC_ERROR;
    CommonUtils.error("A problem occurred while creating a temporary file:  " + e.getMessage());    
}

if(writer != null)
{
    rc = MigrateSecurityImpl.generateSQL(attributes, writer);   
}

project.setNewProperty(resultsProperty, "" + rc);
project.setNewProperty(resultFileNameProperty, outputFileName);

CommonUtils.info("<---- SASSDM SECURITY MIGRATION COMPLETE RC=" + rc);


public class MigrateSecurityImpl
{
    
    private static boolean isTableServer = false;
    
    private static final int MAX_RECORDS_PER_INSERT = 10;
    
    private static String spm_SoftwareComponentName_old = "Strategic Performance Management";
    private static String spm_SoftwareComponentName_new = "Strategy Mgmt 5.3";

    static Integer ONE = new Integer(1);
    static Integer ZERO = new Integer(0);
    static Integer FF = new Integer(255); 

    static Integer  USERIDENTITYTYPE = new Integer(1);
    static Integer  GROUPIDENTITYTYPE = new Integer(2);


    
    public static int generateSQL(Map attributes, BufferedWriter writer)
    {
      
        int rc = CommonUtils.RC_OK;
        
        Statement statement = null;
        ResultSet rs = null;
        Connection connection = null;
        
        try
        {           
            String user = attributes.get("mysql.user");
            String password = attributes.get("mysql.pass");
            String url= attributes.get("mysql.url");
            String driver = attributes.get("mysql.driver");
            String jarDir = attributes.get("mysql.jdbc.jar");
            String targetDBType = attributes.get("spm.db.type");
            String targetDBName = attributes.get("spm.db.name");
            if ((targetDBName == null) || (targetDBName.length() == 0))
                 targetDBName = "spm";
            
            // CommonUtils.info("***DEBUG*** (user, password, url, driver, jarDir, targetDBType, targetDBName) " + user + ", " + password + ", " + url + ", " + driver + ", " + jarDir + ", " + targetDBType + ", " + targetDBName);
                        
            
            File jar = new File(jarDir);
            if (jar == null)
                return CommonUtils.RC_ERROR;
            
            //--------------------------------------------------
            //---- ADD THE MYSQL JAR FILE TO CLASS PATH --------
            //--------------------------------------------------
            ClassLoader loader = Thread.currentThread().getContextClassLoader();
            loader.parent.addURL(jar.toURI().toURL());
   
            Class.forName(driver);
            connection = DriverManager.getConnection(url, user, password);

            CommonUtils.info("START MIGRATION FOR SASSDM SAS_USER_ACTIONS");

            
            /*---------------------------------------------------------------------------------------+
            
            Convert the sassdm sas_user_actions, sas_group_actions to spm sas_permission_rule 
            
            The follow is the column mappings from the sassdm actions table to the permission rule
            USER_IDENTIFIER, GROUP_IDENTIFIER  --> IDENTITY
                                               --> IDENTITY_TYPE (1 = User, 2 = Group)
            OBJ_IDENTIFIER,                    --> OBJECT_ID
            OBJ_TYPE,                          --> OBJ_TYPE
            OBJ_HIER,                          --> Not Used
            START_DT,                          --> START_DT
            END_DT,                            --> END_DT
            APP_IDENTIFIER,                    --> Not Used
            PERMISSIONS (CRUD-Aru)             --> PERMISSIONS (CRrUuDdA)    Bit mappings in ()
                        (----N---)             --> PERMISSION_TYPE   0 - Grant, 1 - DENY (Negate)
                                               --> 0 - Grant if
                                               --> Permission Mask (Constant B'11111111')
            ACTIONS                            --> Not Used
                                               --> ORDERBY (Constant 0)
                                               
            +-----------------------------------------------------------------------------------------*/

            String sassdm_query = null;
            
            
            sassdm_query = "SELECT USER_ACTION_ID, USER_IDENTIFIER, OBJ_IDENTIFIER, OBJ_TYPE, OBJ_HIER, START_DT, END_DT, APP_IDENTIFIER, PERMISSIONS, ACTIONS FROM sassdm.sas_user_actions s WHERE OBJ_TYPE='SPMTemplate' OR OBJ_TYPE='SPMProject' OR OBJ_TYPE='SPMScorecard' OR OBJ_TYPE='SPMElement' ;" ;                       
            statement = connection.createStatement();
            rs = statement.executeQuery(sassdm_query);
            processRS(rs, USERIDENTITYTYPE, writer);

            sassdm_query = "SELECT GROUP_ACTION_ID, GROUP_IDENTIFIER, OBJ_IDENTIFIER, OBJ_TYPE, OBJ_HIER, START_DT, END_DT, APP_IDENTIFIER, PERMISSIONS, ACTIONS FROM sassdm.sas_group_actions s WHERE OBJ_TYPE='SPMTemplate' OR OBJ_TYPE='SPMProject' OR OBJ_TYPE='SPMScorecard' OR OBJ_TYPE='SPMElement' ;" ;                       
            statement = connection.createStatement();
            rs = statement.executeQuery(sassdm_query);
            processRS(rs, GROUPIDENTITYTYPE, writer);           

        }
        catch (Exception e)
        {
            rc = CommonUtils.RC_ERROR;
            CommonUtils.error("A problem occurred while generating migration data from SASSDM:  " + e.getMessage());
            e.printStackTrace();
        }
        finally
        {        
            close(writer);
            close(rs);
            close(statement);
            close(connection);            
        }
        
        return rc;        
    }
    
 
    public static final int processRS(ResultSet rs, Integer identityType, BufferedWriter writer)
    {
    
        StringBuffer sas_permrule_update = new StringBuffer();
                       
        int sas_permrule_count = 0;
                  
        String rulePermString = null;
        int[] deny = new int[1];

        String identityColumn = "GROUP_IDENTIFIER";
        
        if (identityType == USERIDENTITYTYPE)
        	identityColumn = "USER_IDENTIFIER";

        String tableName = rs.getMetaData().getTableName(1);
                        
           
        while(rs.next())
        {
            
            if(sas_permrule_count%MAX_RECORDS_PER_INSERT == 0)
            {
                if(sas_permrule_count > 0)
                {
                    sas_permrule_update.append(";");
                    writer.write(sas_permrule_update.toString());
                    writer.write(";");
                    writer.newLine();       
                    sas_permrule_update.setLength(0); // reduce memory load
                 }
                 
                sas_permrule_update.append("\nINSERT INTO SAS_PERMISSIONRULE (OBJECT_ID, OBJECT_TYPE, IDENTITY, IDENTITY_TYPE, PERMISSION_TYPE, PERMISSIONS_MASK, PERMISSIONS, ORDERBY) VALUES \n(");
            }
            else    sas_permrule_update.append(",\n(");
               
            deny[0]  = 0;
            int rulePermBits = convertPermissions(rs.getObject("PERMISSIONS"), deny);

            processObject(sas_permrule_update, rs.getObject("OBJ_IDENTIFIER"), "", false);   // OBJECT_ID 
            processObject(sas_permrule_update, rs.getObject("OBJ_TYPE"), "", false);         // OBJECT_TYPE
                
            processObject(sas_permrule_update, rs.getObject(identityColumn), "", false);  // IDENTITY
            processObject(sas_permrule_update, identityType, "", false);                     // IDENTITY_TYPE

            processObject(sas_permrule_update, new Integer(deny[0]), ZERO, false);           // PERMISSION_TYPE GRANT = 1, DENY = 0
            processObject(sas_permrule_update, FF, FF, false);                               // PERMISSION_MASK

            processObject(sas_permrule_update, new Integer(rulePermBits), ZERO, false);      // PERMISSIONS
                
            processObject(sas_permrule_update, ZERO, ZERO, false, false);                    // ORDERBY             
                
                
            sas_permrule_update.append(")");
            sas_permrule_count++;                
        }  
           
        CommonUtils.info(sas_permrule_count + " RECORDS WILL BE MIGRATED FOR SASSDM TABLE " + tableName + " .");
   
        if(sas_permrule_count > 0)
        {
            writer.write(sas_permrule_update.toString());
            writer.write(";");
            writer.newLine();   
            sas_permrule_update.setLength(0); // reduce memory load          
        }  

        return sas_permrule_count;
    }   

    
    // Static constants from PermissionDistller.java in reverse order

    public static final int CREATE  = 0x0080;
	public static final int READMETADATA    = 0x0040;
	public static final int UPDATEMETADATA  = 0x0020;
	public static final int DELETE  = 0x0010;
	public static final int NEGATE = 0x0008;
	public static final int ADMIN = 0x0004;
	public static final int READDATA = 0x0002;
	public static final int UPDATEDATA = 0x0001;

    
    private static int convertPermissions(String inPermissions, int[] deny) 
    { 
       // Takes the old pemrission string from solutions common which is
       // CRUDNAru  e.g. "11111011" and   C - Create R - ReadMetadata U - UpdateMeadata, D - Delete, N - Negate, A - Administer, r - read data, u - update data
       // convert it to a bit string string right to left of the form
       // CRrUuDdA  with   (D is Delete Metadata, d is delete data)  generally the Rr Uu Dd are paired the same in 99.9% to the cases.
       // and if N set the Deny array to 1.
       deny[0] = 0;
       int newPerm = 0;
       if (inPermissions == null)
          return newPerm;
          
       int nPermissions = inPermissions.size(); 
       int bitOn = 0;
       
       for (int ix = 0; ix < nPermissions; ix++)
       {
           bitOn = 0;
           if (inPermissions.charAt(ix) != '0')
               bitOn = 1;
               
           switch (ix) 
           {
              case 0: newPerm += bitOn * CREATE;  break; // Create permission
                      
              case 1: newPerm += bitOn * READMETADATA ;  break;  // ReadMetadata permission
                  
              case 2: newPerm += bitOn * UPDATEMETADATA;  break;  // UpdateMetadata permission
             
              case 3: newPerm += bitOn * DELETE ; break; // DeleteMetadata + delete data
              
              case 4: deny[0] = bitOn;  break;          // Negate Deny = 1; Grant = 0; 
              
              case 5: newPerm += bitOn * ADMIN ; break;     // Admin permission
              
              case 6: newPerm += bitOn * READDATA; break;    // Read data;
              
              case 7: newPerm += bitOn * UPDATEDATA;  break;    // Update data
           }   
       }
      //  println("inPermissions: " + inPermissions + " newPerm: " + newPerm);       
       return newPerm;
    
    }
    
    
    public static final void close(BufferedWriter writer)
    {
        if (writer != null)
        {
            try
            {
                writer.close();
            }
            catch (IOException ex)
            {              
            }
        }
    }
    
   
    public static final void close(ResultSet rs)
    {
        if (rs != null)
        {
            try
            {
                rs.close();
            }
            catch (SQLException ex)
            {              
            }
        }
    }

    
    public static final void close(Statement stmt)
    {
        if (stmt != null)
        {
            try
            {
                stmt.close();
            }
            catch (SQLException ex)
            {               
            }
        }
    }

    
    public static final void close(Connection con)
    {
        if (con != null)
        {
            try
            {
                con.close();
            }
            catch (SQLException ex)
            {                
            }
        }
    }
    
    
    public static void processObject(StringBuffer sb, Object obj, Object theDefault, boolean allowNull)
    {
        processObject(sb, obj, theDefault, allowNull, true);   
    }
    
    
    public static void processObject(StringBuffer sb, Object obj, Object theDefault, boolean allowNull, boolean comma)
    {       
        String value = "";
        
        if(obj == null && allowNull)
        {
            sb.append("NULL");
        }
        else if(obj == null)
        {
            appendValue(sb, theDefault);
        }
        else
        {   
            appendValue(sb, obj);
        }
        
        if(comma)
            sb.append(", ");        
    }
    
    public static String appendValue(StringBuffer sb, Object obj)
    {
        if(obj instanceof Integer)
        {
            sb.append(obj.toString());                
        }
        else if (obj instanceof Timestamp && isTableServer)
        {            
            sb.append("timestamp '");            
            sb.append(escape(obj.toString()));
            sb.append("'");
        }
        else if (obj instanceof String && ((String) obj).startsWith("@"))
        { 
                sb.append(obj)
        }       
        else
        {
            sb.append("'");            
            sb.append(escape(obj.toString()));
            sb.append("'");                
        }        
    }
    
    
    public static String escape(String str)
    {
        String result = "";
        if(isTableServer)
            result = str.replaceAll("\\\\", "\\\\\\\\").replaceAll("\n", "\\\\n").replaceAll("\r", "\\\\r").replaceAll("\"", "\\\\\"").replaceAll("'", "''").replaceAll("\0", "\\\\0");
        else
            result = str.replaceAll("\\\\", "\\\\\\\\").replaceAll("\n", "\\\\n").replaceAll("\r", "\\\\r").replaceAll("\"", "\\\\\"").replaceAll("'", "\\\\'").replaceAll("\0", "\\\\0");
        
        return result;
    }
}