package deploy.portal;

import com.sas.metadata.MetadataObjects;

import com.sas.metadata.remote.AssociationList;
import com.sas.metadata.remote.CMetadata;
import com.sas.metadata.remote.Document;
import com.sas.metadata.remote.Group;
import com.sas.metadata.remote.Property;
import com.sas.metadata.remote.PropertySet;
import com.sas.metadata.remote.PropertyType;
import com.sas.metadata.remote.Prototype;
import com.sas.metadata.remote.PSLayoutComponent
import com.sas.metadata.remote.PSPortlet;
import com.sas.metadata.remote.Tree;

import deploy.CommonUtils;
import deploy.JomaSupport;



/**
 * Migrate Stm PerformanceDataEntry portlet properties
 * 
 *
 */

 int rc = CommonUtils.RC_OK;
       
 // Set up script results vars
 String resultsProperty = attributes.get("resultsproperty");
 String outputProperty = attributes.get("outputproperty");
 
 try
 {
     PerformanceDataEntry pde = new PerformanceDataEntry(attributes);
     pde.migrate();
     rc = pde.getReturnCode();
 }
 catch (Exception e)
 {
     rc = CommonUtils.RC_ERROR;
     project.setNewProperty(outputProperty, e.toString());
     e.printStackTrace();
 }
 finally
 {
     CommonUtils.info("migrateStmDataEntry finished");
 }

 project.setNewProperty(resultsProperty, "" + rc);
 
 // ----- END RUN THE UPGRADE CODE -----
 
class PerformanceDataEntry
{
    String user;
    String password;
    String host;
    String port;
    
    String webappRootUri;

    String foundReposId;

    JomaSupport joma;

    

    List objsToDel = new ArrayList();
    
    private static final String PORTLET_TYPE_NM = "PerformanceDataEntry";
    private static final String PORTLET_CONFIG_ROOT_NM = "PORTLET_CONFIG_ROOT";
    private static final String DIRECTIVE_PREFIX = "DIRECTIVE_";
    private static final String ITEM_PREFIX = "ITEM_";
    

    

    
    private int warnings = 0;
    private int errors = 0;
    
    public PerformanceDataEntry(Map attributes)
    {
        this.user = attributes.get("admin.user");
        this.password = attributes.get("admin.pass");
        this.host = attributes.get("omr.host");
        this.port = attributes.get("omr.port");
        // this.webappRootUri = attributes.get("webapp.rooturi");
        webappRootUri="Director?_directive=SPMPortletsDirective&directto=";   // Use SPMPortlets diureticive instead
    }
    
    public void migrate()
    {
        try
        {
            CommonUtils.info("--- INPUT ATTRIBUTES ---");
            CommonUtils.info("User: " + user);
            // CommonUtils.info("Password: " + password);
            CommonUtils.info("OMR Host: " + host);
            CommonUtils.info("OMR Port: " + port);
            CommonUtils.info("OMR user: " + user);
            CommonUtils.info("OMR password: " + password);
            
            CommonUtils.info("Webapp Root URI: " + webappRootUri);
            CommonUtils.info("------------------------");
            CommonUtils.info("");

            joma = new JomaSupport();
            joma.connect(host, port, user, password);

            foundReposId = joma.getFoundationRepositoryId();
            
            boolean isMigrated = false;
            
            // Get the PerformanceDataEntry Portlets
            List portlets = getPSPortletObjects(foundReposId);
            for (Iterator it = portlets.iterator(); it.hasNext(); )
            {
                PSPortlet portlet = (PSPortlet) it.next();
                 
                isMigrated = migratePortlet(foundReposId, portlet);
                
                if (isMigrated)
                {
                    CommonUtils.info("Migrated PSPortlet: '" + portlet.getName() +
                                        "', Id: " + portlet.getFQID());
                    CommonUtils.info(" ");                    
                                        
                }
                else
                {
                     CommonUtils.info("PSPortlet: '" + portlet.getName() +
                                                        "', Id: " + portlet.getFQID() + " contained no Stm Items");
                }

            }
                                  
        }
        catch (Exception e)
        {
          throw e;
        }
        finally
        {
            if (joma != null)
            {
                joma.disconnect();
            }
        }
    }
    
    // PerformanceDataEntry
    
    private List getPSPortletObjects(String reposId)
    {
        String xmlSelect = "@portletType='" + PORTLET_TYPE_NM + "'";
        List portlets = joma.getMetadataObjects(reposId, 
                MetadataObjects.PSPORTLET, xmlSelect);

        CommonUtils.info("Found " + portlets.size() + " " + PORTLET_TYPE_NM +  "  portlets to migrate.");
        
        return portlets;
    }

    private boolean migratePortlet(String reposId, PSPortlet portlet)
    {
        // Find the 'PropertySet' object named:  PORTLET_CONFIG_ROOT
        // should only be one, but we'll look for more
        boolean bIsMigrated = false;
        
        AssociationList objs = portlet.getPropertySets();
        for (Iterator it = objs.iterator(); it.hasNext(); )
        {
            CMetadata cmeta = (CMetadata) it.next();
            if (cmeta.getCMetadataType().equals("PropertySet"))
            {
                PropertySet ps = (PropertySet) cmeta;
                if (ps.getName().equals(PORTLET_CONFIG_ROOT_NM))
                {
                    migratePropertySet(reposId, ps, portlet);
                    bIsMigrated = true;
                        
                }
            }
        }
        
        return bIsMigrated;
    }
    
    private migratePropertySet(String reposId, PropertySet propSet, PSPortlet portlet)
    {
        String propName, propValue, propDesc;
        String itemName, itemDesc, itemURI;
        CMetadata cmeta;
        Property prop;
        objsToDel.clear();
        
        String pName = portlet.getName();
        

        AssociationList objs = propSet.getSetProperties();
        for (Iterator it = objs.iterator(); it.hasNext(); )
        {
            cmeta = (CMetadata) it.next();
               
            if (!"Property".equals(cmeta.getCMetadataType() ))
               continue;
            prop = (Property) cmeta;   
            propName =  prop.getName();
            propValue = prop.getDefaultValue();
            propDesc = prop.getDesc();
            
	    // setProperty(propSet, "show-description", "true");
	    // setProperty(propSet "show-location", "false");
	    // setProperty(propSet, "ascending-packageSortOrder", "UsePreferences");
 
            logInfo("Portlet: " + pName + " Property Name: " + propName + " Value: " + propValue + " + Desc: " + propDesc);
                

            // deleteProperty(prop, objsToDel);  // Mark the property for deletion (if needed)
             
        }
        if (!objsToDel.isEmpty())                // Remove deleted properties
        {
            joma.deleteObjects(objsToDel);
        }
        
        return;
    }
    

    
 
    

    
    private void deleteMetadataObject(CMetadata cMd, List objsToDel)
    {
            CommonUtils.info("   Delete Metadata object(" + cmeta.getCMetadataType() + ") : " +  cMd.getName() + ", Id: " + prop.getFQID());
            objsToDel.add(prop);
    }


    public String setProperty(PropertySet propSet, String propName, String propVal)
    {
           String oldValue = "";
           CMetadata propObj = joma.getAssocObjByName(propSet, "SetProperties", propName);
           if (propObj == null)
           {
               propObj = joma.newProperty(null, propName);
               propSet.getSetProperties().add(propObj);
           }
           else oldValue = "" + propObj.getDefaultValue();
           
           propObj.setPropertyName(propName);
           propObj.setDefaultValue(propVal);
           propSet.updateMetadataAll();
           return oldValue;
    } 


    private void deleteProperty(Property prop, List objsToDel)
    {
        CommonUtils.info("        **Delete " + PORTLET_TYPE_NM + ": " +   prop.getName() + ", Id: " + prop.getFQID());
        objsToDel.add(prop);
    }
    
           
    
    public int getReturnCode()
    {
        CommonUtils.info("");
        int rc = CommonUtils.RC_OK;
        if(errors > 0)
        {
            rc = CommonUtils.RC_ERROR;
        }
        else if (warnings > 0)
        {
            rc = CommonUtils.RC_WARN;
        }
        CommonUtils.info("RETURN CODE: " + rc + " ERRORS: " + errors + 
                " WARNINGS: " + warnings);
        return rc;
    }

    private void logInfo(String msg)
    {
        CommonUtils.info(msg);
    }
    
    private void logWarning(String msg)
    {
        CommonUtils.warn(msg);
        warnings++;
    }
    
    private void logError(String msg)
    {
        CommonUtils.error(msg);
        errors++;        
    }
    
}
 
