package deploy.portal;

import com.sas.metadata.MetadataObjects;

import com.sas.metadata.remote.AssociationList;
import com.sas.metadata.remote.CMetadata;
import com.sas.metadata.remote.Document;
import com.sas.metadata.remote.Group;
import com.sas.metadata.remote.Property;
import com.sas.metadata.remote.PropertySet;
import com.sas.metadata.remote.PropertyType;
import com.sas.metadata.remote.Prototype;
import com.sas.metadata.remote.PSLayoutComponent
import com.sas.metadata.remote.PSPortlet;
import com.sas.metadata.remote.Tree;

import deploy.CommonUtils;
import deploy.JomaSupport;



/**
 * Migrate Solutions My Favorites Directive STM content
 * to Collections portlets
 * 
 *
 * SPM_NewProject                     no chg
 * 
 * SPM_Manage_TemplatesAndProjects    no chg
 * 
 * SPM Data Entry links
 * 
 */

 int rc = CommonUtils.RC_OK;
       
 // Set up script results vars
 String resultsProperty = attributes.get("resultsproperty");
 String outputProperty = attributes.get("outputproperty");
 
 try
 {
     MyFavorites fav = new MyFavorites(attributes);
     fav.migrate();
     rc = fav.getReturnCode();
 }
 catch (Exception e)
 {
     rc = CommonUtils.RC_ERROR;
     project.setNewProperty(outputProperty, e.toString());
     e.printStackTrace();
 }
 finally
 {
     CommonUtils.info("migrateStmMyFavorites finished");
 }

 project.setNewProperty(resultsProperty, "" + rc);
 
 // ----- END RUN THE UPGRADE CODE -----
 
class MyFavorites
{
    String user;
    String password;
    String host;
    String port;
    
    String webappRootUri;
    String webappManageTxt;
    String webappNewProjectTxt;

    String foundReposId;

    JomaSupport joma;

    

    List objsToDel = new ArrayList();
    
    private static final String FAV_NM = "Favorites";
    private static final String PORTLET_CONFIG_ROOT_NM = "PORTLET_CONFIG_ROOT";
    private static final String DIRECTIVE_PREFIX = "DIRECTIVE_";
    private static final String ITEM_PREFIX = "ITEM_";
    
    private static final String SPM_MANAGETEMPLATES_VALUE = "SPM_Manage_TemplatesAndProjects";
    private static final String SPM_NEWPROJECT_VALUE = "SPM_NewProject";
    private static final String SPM_DATAENTRYFORM_VALUE = "ScorecardDataEntryForm&";

    

    
    private int warnings = 0;
    private int errors = 0;
    
    public MyFavorites(Map attributes)
    {
        this.user = attributes.get("admin.user");
        this.password = attributes.get("admin.pass");
        this.host = attributes.get("omr.host");
        this.port = attributes.get("omr.port");
        // this.webappRootUri = attributes.get("webapp.rooturi");
        webappRootUri="Director?_directive=SPMPortletsDirective&directto=";   // Use SPMPortlets diureticive instead
        this.webappManageTxt = attributes.get("webapp.manage.txt");
        this.webappNewProjectTxt = attributes.get("webapp.newproject.txt");
    }
    
    public void migrate()
    {
        try
        {
            CommonUtils.info("--- INPUT ATTRIBUTES ---");
            CommonUtils.info("User: " + user);
            // CommonUtils.info("Password: " + password);
            CommonUtils.info("OMR Host: " + host);
            CommonUtils.info("OMR Port: " + port);
            CommonUtils.info("OMR Port: " + port);
            CommonUtils.info("Webapp Root URI: " + webappRootUri);
            CommonUtils.info("Webapp Manage Projects Text: " + webappManageTxt);
            CommonUtils.info("Webapp New Project Text: " + webappNewProjectTxt);
            CommonUtils.info("------------------------");
            CommonUtils.info("");

            joma = new JomaSupport();
            joma.connect(host, port, user, password);

            foundReposId = joma.getFoundationRepositoryId();
            
            boolean isMigrated = false;
            
            // Get the MyFavorites Portlets
            List portlets = getPSPortletObjects(foundReposId);
            for (Iterator it = portlets.iterator(); it.hasNext(); )
            {
                PSPortlet portlet = (PSPortlet) it.next();
                 
                isMigrated = migratePortlet(foundReposId, portlet);
                
                if (isMigrated)
                {
                    CommonUtils.info("Migrated PSPortlet: '" + portlet.getName() +
                                        "', Id: " + portlet.getFQID());
                    CommonUtils.info(" ");                    
                                        
                }
                else
                {
                     CommonUtils.info("PSPortlet: '" + portlet.getName() +
                                                        "', Id: " + portlet.getFQID() + " contained no Stm Items");
                }

            }
                                  
        }
        catch (Exception e)
        {
          throw e;
        }
        finally
        {
            if (joma != null)
            {
                joma.disconnect();
            }
        }
    }
    
    
    
    private List getPSPortletObjects(String reposId)
    {
        String xmlSelect = "@portletType='" + FAV_NM + "'";
        List portlets = joma.getMetadataObjects(reposId, 
                MetadataObjects.PSPORTLET, xmlSelect);

        CommonUtils.info("Found " + portlets.size() + 
                " Favorite Portlets to migrate.");
        
        return portlets;
    }

    private boolean migratePortlet(String reposId, PSPortlet portlet)
    {
        // Find the 'PropertySet' object named:  PORTLET_CONFIG_ROOT
        // should only be one, but we'll look for more
        boolean bCreatedNewPortlet = false;
        
        AssociationList objs = portlet.getPropertySets();
        for (Iterator it = objs.iterator(); it.hasNext(); )
        {
            CMetadata cmeta = (CMetadata) it.next();
            if (cmeta.getCMetadataType().equals("PropertySet"))
            {
                PropertySet ps = (PropertySet) cmeta;
                if (ps.getName().equals(PORTLET_CONFIG_ROOT_NM))
                {
                    if (null != migratePropertySet(reposId, ps, portlet))
                        bCreatedNewPortlet = true;
                }
            }
        }
        
        return bCreatedNewPortlet;
    }
    
    private PSPortlet migratePropertySet(String reposId, PropertySet ps, PSPortlet portlet)
    {
        String propName, propValue, propDesc;
        String itemName, itemDesc, itemURI;
        CMetadata cmeta;
        Property prop;
        PSPortlet newCollPortlet = null;
        boolean needNewCollectionsPortlet = true;  // Set to true initial
        objsToDel.clear();
        

        AssociationList objs = ps.getSetProperties();
        for (Iterator it = objs.iterator(); it.hasNext(); )
        {
            cmeta = (CMetadata) it.next();
               
            if (!"Property".equals(cmeta.getCMetadataType() ))
               continue;
            prop = (Property) cmeta;   
            propName =  prop.getName();
            propValue = prop.getDefaultValue();
            propDesc = prop.getDesc();
                
            if (!propName.startsWith(DIRECTIVE_PREFIX))
               continue;
               
            if (!propValue.equals(SPM_MANAGETEMPLATES_VALUE) &&
                !propValue.equals(SPM_NEWPROJECT_VALUE)      &&
                !propValue.startsWith(SPM_DATAENTRYFORM_VALUE))
               continue;    
                        
            if (newCollPortlet == null)
            {
                newCollPortlet = createCollectionsPortlet(portlet);
                if (newCollPortlet == null)
                {
                   logError("New Collection portlet is null -- portlet creation failed, cannot continue with migration");
                   return newCollPortlet;  // Don't bother continuing
                }   
            }
            
            // println("migratePropertySet1" + " propName: " + propName + " propValue: " + propValue + " propDesc: " + propDesc);     
                                         
            itemName = ""; 
            itemDesc = propDesc;
            itemURI = ""; 
            
            if (propValue.equals(SPM_MANAGETEMPLATES_VALUE))
            {  // http://midtier:9080/SASStrategyManagement/PrepareManageTemplates.do  
               itemName = webappManageTxt;
               itemURI = webappRootUri + "PrepareManageTemplates.do" + "&fromPortal=true";
            }       
            else if (propValue.equals(SPM_NEWPROJECT_VALUE))
            {  // http://midtier:9080/SASStrategyManagement/NewProjectLocation.do
               itemName = webappNewProjectTxt;
               itemURI = webappRootUri + "NewProjectLocation.do" + "&fromPortal=true";
            }            
            else if (propValue.startsWith(SPM_DATAENTRYFORM_VALUE))
            {  // http://midtier:9080/SASStrategyManagement/SDESimpleFormEntry.do?Spm_Id=4c1de41c-0a15-0f45-4a8f-79ba4639b8ba1&fromPortal=true
               
               itemName = propName.substring(DIRECTIVE_PREFIX.length());;
               itemURI = webappRootUri + "SDESimpleFormEntry.do" + "&" + propValue.substring(SPM_DATAENTRYFORM_VALUE.length()) + "&fromPortal=true";
            }
            else 
            {
               logError("migratePropertySet propValue is not an Stm MyFavorites link item " + propValue); 
               continue;
            }   
             
            addDocumentToCollectionsPortlet(itemName, itemDesc, itemURI, "Portal Web Application", newCollPortlet);
            deleteProperty(prop, objsToDel);  // Remove the Stm property as its now in an a collections portlet
             
        }
        if (!objsToDel.isEmpty())
        {
            joma.deleteObjects(objsToDel);
        }
        
        return newCollPortlet;
    }
    
    private addDocumentToCollectionsPortlet(String itemName, String itemDesc, String itemURI,
                                            String itemTextRole, PSPortlet collPortlet)
    {
    
        Document doc = joma.newObject(null, "Document", itemName);
        
        doc.setDesc(itemDesc);
        doc.setURI(itemURI);
        doc.setTextRole(itemTextRole);
        doc.updateMetadataAll();
        
        for (Iterator iter = collPortlet.getGroups().iterator(); iter.hasNext();)
        {
            Group group = (Group) iter.next();
            AssociationList members = group.getMembers();
            members.add(doc);
            group.updateMetadataAll();
        }
        
        for (Iterator iter = collPortlet.getTrees().iterator(); iter.hasNext();)
        {
           Tree tree = (Tree) iter.next();
           AssociationList members = tree.getMembers();
           try
           {
              members.add(doc);
              tree.updateMetadataAll(); 
           } 
           catch (Exception permEx)
           {
               logError(permEx.getMessage());  // Permission error adding memebers to a read-only permission tree
           }
        } 
        
        CommonUtils.info("Item: " + doc.getName() + "  added to Collection portlet:  " + collPortlet.getName() + "(" + collPortlet.getId() + ") " );
    
    }
    
    private PSPortlet createCollectionsPortlet(PSPortlet myFav)
    {   
        // ------------------------------------------------------------------------------
        // Create a new Collections Portlet based upon the old MyFavorites portlet
        //
        //     Create new PSPortlet:  Copy name and description from MyFavorites 
        //     Create new Group: Portal Collection
        //     Create a PropertySet: PORTLET_CONFIG_ROOT
        // 
        //     Add Association to Collections template Prototype
        //     Copy the MyFavorites Trees Association
        //     Copy the MyFavorites LayoutComponents
        // ------------------------------------------------------------------------------
        
        String newPortletName = myFav.getName();
        String newPortletDesc = myFav.getDesc();
                
        PSPortlet collPortlet = joma.newObject(null, "PSPortlet", newPortletName);
        collPortlet.setDesc(newPortletDesc);       
        collPortlet.setportletType("Collections");
        collPortlet.updateMetadataAll();
        
        Group group = joma.newObject(null, "Group", "Portal Collection"); 
        AssociationList groupMembers = group.getMembers();
        if (groupMembers == null)
        {
           List gList = new ArrayList(1)
           gList.add(collPortlet)
           groupMembers = new AssociationList("Members", gList);           
        }   
        else groupMembers.add(collPortlet);
        
        group.setMembers(groupMembers);
        group.updateMetadataAll();
        
        PropertySet propSet = joma.createObjectNoUpdate(null, "PropertySet", "PORTLET_CONFIG_ROOT");
        propSet.setOwningObject(collPortlet);
        propSet.updateMetadataAll();        
        setProperty(propSet, "show-description", "true");
        setProperty(propSet, "show-location", "false");
        setProperty(propSet, "ascending-packageSortOrder", "UsePreferences");
                        
        Prototype proto = joma.getObjectByName(null, "Prototype", "Collection template");
        AssociationList protoMembers = proto.getAssociatedObjects("UsedByPrototypes");
        protoMembers.add(collPortlet)
        proto.updateMetadataAll();

        PSLayoutComponent psl;
        AssociationList portletMembers;
        for (Iterator iter = myFav.getLayoutComponents().iterator(); iter.hasNext();)
        {
           psl = (PSLayoutComponent) iter.next();
           portletMembers = psl.getPortlets();
           try
           {
               portletMembers.add(collPortlet);
               psl.updateMetadataAll(); 
               
           } 
           catch (Exception permEx)
           {
              logError(permEx.getMessage());  // Permission error adding memebers to a read-only permission tree
           }
           
        }
               
 
        for (Iterator iter = myFav.getTrees().iterator(); iter.hasNext();)
        {
           Tree tree = (Tree) iter.next();
           AssociationList members = tree.getMembers();
           try
           {
               members.add(collPortlet);
               members.add(group);
               tree.updateMetadataAll();        
           } 
           catch (Exception permEx)
           {
              logError(permEx.getMessage());  // Permission error adding memebers to a read-only permission tree
           }
        }               


        //  Remove the Association portlet from the layout
        AssociationList layoutComponents = myFav.getLayoutComponents();
        layoutComponents.clear();
        // myFav.setLayoutComponents(layoutComponents);
        // myFav.updateMetadataAll();
        
        CommonUtils.info("Created New Collections portlet: " + collPortlet.getName() + " (" + collPortlet.getId() + ") " );
        
        return collPortlet;
    
    }
    

    
    private void deleteMetadataObject(CMetadata cMd, List objsToDel)
    {
            CommonUtils.info("   Delete Metadata object(" + cmeta.getCMetadataType() + ") : " +  cMd.getName() + ", Id: " + prop.getFQID());
            objsToDel.add(prop);
    }


    public String setProperty(PropertySet propSet, String propName, String propVal)
    {
           String oldValue = "";
           CMetadata propObj = joma.getAssocObjByName(propSet, "SetProperties", propName);
           if (propObj == null)
           {
               propObj = joma.newProperty(null, propName);
               propSet.getSetProperties().add(propObj);
           }
           else oldValue = "" + propObj.getDefaultValue();
           
           propObj.setPropertyName(propName);
           propObj.setDefaultValue(propVal);
           propSet.updateMetadataAll();
           return oldValue;
    } 


    private void deleteProperty(Property prop, List objsToDel)
    {
        CommonUtils.info("        Del My Favs Property: " + 
                prop.getName() + ", Id: " + prop.getFQID());
        objsToDel.add(prop);
    }
    
           
    
    public int getReturnCode()
    {
        CommonUtils.info("");
        int rc = CommonUtils.RC_OK;
        if(errors > 0)
        {
            rc = CommonUtils.RC_ERROR;
        }
        else if (warnings > 0)
        {
            rc = CommonUtils.RC_WARN;
        }
        CommonUtils.info("RETURN CODE: " + rc + " ERRORS: " + errors + 
                " WARNINGS: " + warnings);
        return rc;
    }
    
    private void logWarning(String msg)
    {
        CommonUtils.warn(msg);
        warnings++;
    }
    
    private void logError(String msg)
    {
        CommonUtils.error(msg);
        errors++;        
    }
    
}
 
