package deploy.softwareComponents;

import com.sas.metadata.MdObjectFactory;
import com.sas.metadata.remote.SoftwareComponent;
import com.sas.metadata.remote.IdentityGroup;
import com.sas.metadata.remote.DeployedComponent;
import com.sas.metadata.remote.Property;
import com.sas.metadata.remote.PropertySet;
import com.sas.metadata.remote.AssociationList;
import com.sas.metadata.remote.CMetadata;
import com.sas.metadata.remote.ServiceComponent;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import deploy.JomaSupport;

//----- RUN THE MIGRATION CODE -----

JomaSupport joma = null;
boolean DEBUG = true;
int rc = 0;

                     

try
{
    
    String user = attributes.get("admin.user");
    String password = attributes.get("admin.pass");
    String host= attributes.get("omr.host");
    String port = attributes.get("omr.port");
    String spmmidtier_swc_name = attributes.get("spmmidtier.software.component.name");
     
    
    // The following maps the properties which are NOT migrated from 9.1.3 to 9.2
    // Either they are no longer supported or the properties require 9.2 settings

    if(DEBUG)
    {
      println("--- INPUT ATTRIBUTES ---");
      println("USER: " + user);
      println("OMR HOST: " + host);    
      println("OMR PORT: " + port);
      println("SPMMIDTIER SWC NAME: " + spmmidtier_swc_name.toString());
    }
        
    joma = new JomaSupport();
    joma.connect(host, port, user, password);

    MigrateSPMSoftwareComponent migrate = new MigrateSPMSoftwareComponent(joma);
    
    migrate.migrateAppConfigProperties(spmmidtier_swc_name);
    migrate.deleteSPMSoftwareComponent();
    migrate.deleteSPMWSSoftwareComponent();
    
}
catch (Exception e)
{
    rc = 1;
    project.setNewProperty("outputproperty", e.toString());	
    e.printStackTrace();    
}
finally
{
  if(DEBUG) println("DONE.");
  if(joma != null)
    joma.disconnect();
}

project.setNewProperty("resultsproperty", "" + rc);

// ----- END RUN THE MIGRATION CODE -----



class MigrateSPMSoftwareComponent
{
    boolean DEBUG = true;
    JomaSupport joma;
    private static String softwareComponentName = "Strategic Performance Management";
    private static String wsSoftwareComponentName = "Strategic Performance Management WebServices";
    
    def excludedProps = ["WebApp.DocSet":"1", 
                         "WebApp.SystemDocSet":"1",
                         "WebApp.Resources":"1",
                         "WebApp.Actions":"1",
                         "WebApp.MenuPrimary":"1",
                         "WebApp.MenuSecondary":"1",
                         "Version.Class":"1",
                         "PortalSoftwareComponentName":"1",
                         "SPMProvider.Midtier.trusted.domain":"1",
                         "SPMProvider.Midtier.trusted.URLs":"1"];

        
    public MigrateSPMSoftwareComponent(JomaSupport joma)
    {
        this.joma = joma;
    }
    
    public void migrateAppConfigProperties(String toSWCName)
    {
       String pmReposName = JomaSupport.getPerformanceManagementRepositoryName();
       String pmReposID = joma.getRepositoryId(pmReposName);   
       CMetadata oldSoftwareComponent = joma.getSoftwareComponentByName(pmReposID, softwareComponentName); 
       
       if (oldSoftwareComponent == null)
       {
        	String msg = softwareComponentName + " SoftwareComponent not found in " + pmReposName +  
                         " repository.  A previous migration run may have deleted it.";  
                debug(msg);         
		throw new Exception(msg);
       }
       
       List oldPropList = new ArrayList();
       JomaSupport.addApplicationConfiguration(oldSoftwareComponent, oldPropList);
       
       // debug(oldPropList.toString());

       CMetadata newSoftwareComponent = joma.getSoftwareComponentByName(null, toSWCName);       
       PropertySet newAppConfig = (PropertySet) JomaSupport.getAssocObjByName(newSoftwareComponent, "PropertySets", "Application Configuration");        
   
       String propName;
       String newPropValue, oldPropValue;
       Property migProperty;
       int pix;
              
       for (pix=1; pix < oldPropList.size(); pix++)
       {  
       
       	   migProperty = (Property) oldPropList.get(pix);
           propName = migProperty.getPropertyName();
           
           // debug("properyName, pix: " + propName + ", " + pix); 
           
           if (propName == null || propName.length() == 0)
              propName = migProperty.getName();
              
           if (propName == null || propName.length() == 0)
           {
           	info("\nCould not obtain migration PropertyName:" + softwareComponentName + ".Application Configuration." + propName);
           	pix++; continue;  // Groovy continue does not increment loop index
           }	
           
           if (null != excludedProps.get(propName))
           {
		info("\nApplication Configuration Property: " + propName + " is in the excluded migration property list");
		pix++; continue;  // Groovy continue does not increment loop index 
           }
              
           newPropValue = "" + migProperty.getDefaultValue();           
           oldPropValue = setProperty(newAppConfig, propName, newPropValue);
           info("\nApplication Configuration Property set: \t" + propName + "\n\tNew Value: " +  newPropValue + "\n\tOld Value: " + oldPropValue);          
       }          
       
    }
    
    public String setProperty(PropertySet propSet, String propName, String propVal)
    {
    	   String oldValue = "";
           CMetadata propObj = joma.getAssocObjByName(propSet, "SetProperties", propName);
           if (propObj == null)
           {
               propObj = joma.newProperty(null, propName);
               propSet.getSetProperties().add(propObj);
           }
           else oldValue = "" + propObj.getDefaultValue();
           
           propObj.setPropertyName(propName);
           propObj.setDefaultValue(propVal);
           propSet.updateMetadataAll();
           return oldValue;
    }  
    
    
    
    public void deleteSPMSoftwareComponent()
    {
      try
        {
            println("----- DELETING " + softwareComponentName + " -----");

            List deleteList = new ArrayList();
            
            String repositoryID = joma.getRepositoryId(JomaSupport.getPerformanceManagementRepositoryName());                      
            CMetadata softwareComponent = joma.getSoftwareComponentByName(repositoryID, softwareComponentName);
            
            if(softwareComponent == null)
                return;

            deleteList.add(softwareComponent);
            
            //----- Actions SoftwareGroups -----
            // JomaSupport.getAssocObjByName(softwareComponent, "SoftwareGroups", "Actions", deleteList);

            //----- Content Tree SoftwareTree -----
            JomaSupport.getAssocObjByName(softwareComponent, "SoftwareTrees", "Content Tree", deleteList);
            
            //----- REGISTERED SAS SOLUTIONS DEPLOYEDCOMPONENT -----
            JomaSupport.addRegisteredSSASSolution(softwareComponent, deleteList);
            
            //----- APPLICATION CONFIGURATION PROPERTY GROUP -----            
            JomaSupport.addApplicationConfiguration(softwareComponent, deleteList);
            
            //----- OBJECTIDENTIFIER SERVICE COMPONENT -----            
            JomaSupport.addObjectIdentifierServiceComponent(softwareComponent, deleteList);

            //----- HistoryIdentifier Property -----
            JomaSupport.getAssocObjByName(softwareComponent, "Properties", "HistoryIdentifier", deleteList);

            showDeleteList(deleteList);
            joma.deleteObjects(deleteList);
            
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }        
        
    }
    
    public void deleteSPMWSSoftwareComponent()
    {
        println("----- DELETING " + wsSoftwareComponentName + " -----");
        List deleteList = new ArrayList();
            
        String repositoryID = joma.getRepositoryId(JomaSupport.getPerformanceManagementRepositoryName());                      
        SoftwareComponent wsSoftwareComponent = joma.getSoftwareComponentByName(repositoryID, wsSoftwareComponentName);
        
        if(wsSoftwareComponent == null)
            return;
        
        deleteList.add(wsSoftwareComponent);
        
        //----- REGISTERED SAS SOLUTIONS DEPLOYEDCOMPONENT -----
        JomaSupport.addRegisteredSSASSolution(wsSoftwareComponent, deleteList);

        showDeleteList(deleteList);
        joma.deleteObjects(deleteList);                                     
    }    
    


    
    public void showDeleteList(List deleteList)
    {
        for(Iterator i = deleteList.iterator(); i.hasNext();)
        {
            CMetadata obj = (CMetadata) i.next();
            println("Deleting: " + obj.getName());            
        }        
    }
    
    
    private void debug(String s)
    {
        if (DEBUG)
        {
             println(s);
        }
    }
    
    private void info(String s)
    {
    	println(s);
    }

}