package deploy.softwareComponents;

import com.sas.metadata.MdObjectFactory;
import com.sas.metadata.remote.SoftwareComponent;
import com.sas.metadata.remote.DeployedComponent;
import com.sas.metadata.remote.Property;
import com.sas.metadata.remote.PropertySet;
import com.sas.metadata.remote.AssociationList;
import com.sas.metadata.remote.CMetadata;
import com.sas.metadata.remote.ServiceComponent;
import com.sas.metadata.remote.TextStore;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import deploy.JomaSupport;

//----- RUN THE UPGRADE CODE -----

/*----------------------------------------------------------------------------*
  Groovy code to upgrade Stm installation release metadata

  The to_release/from_release are passed in as attribute properties for running
  the script.  However every release will require specfic customizations and
  the assumption here is that this will be 5.3 --> 5.4 upgrade.

 *----------------------------------------------------------------------------*/


JomaSupport joma = null;
boolean DEBUG = true;
int rc = 0;



try
{

    String user = attributes.get("admin.user");
    String password = attributes.get("admin.pass");
    String host= attributes.get("omr.host");
    String port = attributes.get("omr.port");
    String spmmidtier_swc_name = attributes.get("spmmidtier.software.component.name");
    String old_rel = attributes.get("from.release");
    String new_rel = attributes.get("to.release");
    String customcontent_images_dir = attributes.get("customcontent.images.dir");

    if(DEBUG)
    {
      println("--- INPUT ATTRIBUTES ---");
      println("USER: " + user);
      println("OMR HOST: " + host);
      println("OMR PORT: " + port);
      println("SPMMIDTIER SWC NAME: " + spmmidtier_swc_name.toString());
      println("FROM RELEASE: " + old_rel);
      println("TO RELEASE: "   + new_rel);
      println("CUSTOM CONTENT IMAGES DIR: "   + customcontent_images_dir);
    }

    joma = new JomaSupport();
    joma.connect(host, port, user, password);

    UpgradeStmMetadataRelease upgrade = new UpgradeStmMetadataRelease(joma);
    upgrade.upgradeAll(joma, old_rel, new_rel, customcontent_images_dir);

}
catch (Exception e)
{
    rc = 1;
    project.setNewProperty("outputproperty", e.toString());
    e.printStackTrace();
}
finally
{
  if(DEBUG) println("DONE.");
  if(joma != null)
    joma.disconnect();
}

project.setNewProperty("resultsproperty", "" + rc);

// ----- END RUN THE UPGRADE CODE -----

class UpgradeStmMetadataRelease
{

    boolean DEBUG = true;
    JomaSupport joma;

    public UpgradeStmMetadataRelease(JomaSupport joma)
    {
         this.joma = joma;
    }

    // This is only supported for 5.1 --> 5.2 and 5.3 --> 5.4, anything else will migrate to 5.3 and then upgrade to 5.4
    public void upgradeAll(JomaSupport joma, String old_rel, String new_rel, String customcontent_images_dir)
    {
       // -- Main stm software component
       SoftwareComponent stmSWC = updateSoftwareComponent(joma, "Strategy Mgmt " + old_rel, old_rel, new_rel, true);

       boolean isError = false;
       ArrayList deleteList = null;
       CMetadata propObj;

       PropertySet stmAppConfig = JomaSupport.getAssocObjByName(stmSWC, "PropertySets", "Application Configuration");

       String propName, oldPropValue, newPropValue;

       if (new_rel.equals("5.4"))
       {

	    // updateSWCNotes(joma, "spmmidtier", old_rel, new_rel, "upgradeSWCNotes");

	    deleteList = new ArrayList(5)
            propObj = joma.getAssocObjByName(stmAppConfig, "SetProperties", 'IMapExport.IMapColLen');
            if (propObj != null) addToDeleteList(propObj, "IMapExport.IMapColLen", deleteList);


            propObj = joma.getAssocObjByName(stmAppConfig, "SetProperties", 'ODCS.Enabled');
            if (propObj != null) addToDeleteList(propObj, "ODCS.Enabled", deleteList);

            propObj = joma.getAssocObjByName(stmAppConfig, "SetProperties", 'ODCS.SoftwareComponentName');
      	    if (propObj != null) addToDeleteList(propObj, "ODCS.SoftwareComponentName", deleteList);

	    propObj = joma.getAssocObjByName(stmAppConfig, "SetProperties", 'ODCS.ConnectionString');
	    if (propObj != null) addToDeleteList(propObj, "ODCS.ConnectionString", deleteList);

 	    showDeleteList(deleteList);
	    joma.deleteObjects(deleteList);


	    propName = "IMapExport.IMapCColLen"; newPropValue = "250";
	    oldPropValue = setProperty(stmAppConfig, propName, newPropValue);
	    info("\nApplication Configuration Property set: \t" + propName + "\n\tNew Value: " +  newPropValue + "\n\tOld Value: " + oldPropValue);

	    propName = "IMapExport.IMapNColLen"; newPropValue = "8";
	    oldPropValue = setProperty(stmAppConfig, propName, newPropValue);
	    info("\nApplication Configuration Property set: \t" + propName + "\n\tNew Value: " +  newPropValue + "\n\tOld Value: " + oldPropValue);

	    propName = "CustomContent.ImagesDir"; newPropValue = customcontent_images_dir;
	    oldPropValue = setProperty(stmAppConfig, propName, newPropValue);
	    info("\nApplication Configuration Property set: \t" + propName + "\n\tNew Value: " +  newPropValue + "\n\tOld Value: " + oldPropValue);

	    propName = "RTL.SupportedLocales"; newPropValue = "ar";
	    oldPropValue = setProperty(stmAppConfig, propName, newPropValue);
	    info("\nApplication Configuration Property set: \t" + propName + "\n\tNew Value: " +  newPropValue + "\n\tOld Value: " + oldPropValue);

        propName = "StM.Formula.Compile.AtStartup"; newPropValue = "true";
        oldPropValue = setProperty(stmAppConfig, propName, newPropValue);
        info("\nApplication Configuration Property set: \t" + propName + "\n\tNew Value: " +  newPropValue + "\n\tOld Value: " + oldPropValue);

	    updateNames(joma, "SASStrategyManagement" + old_rel + " Local Services", "SoftwareComponent", old_rel, new_rel, true);

	    // Strategy Mgmt Tree
	    updateNames(joma, "Strategy Mgmt " + old_rel, "Tree", old_rel, new_rel, true);


 	   updateDirectives(joma, "SPMPortletsDirective", old_rel, new_rel);
 	   updateDirectives(joma, "ScorecardDataEntryForm", old_rel, new_rel);
 	   updateDirectives(joma, "SPM_Manage_TemplatesAndProjects", old_rel, new_rel);
 	   updateDirectives(joma, "SPM_NewProject", old_rel, new_rel);
 	   updateDirectives(joma, "StmWRSServlet", old_rel, new_rel);
 	   updateDirectives(joma, "OpenSPMProject", old_rel, new_rel);
 	   updateDirectives(joma, "SPM_ReportService", old_rel, new_rel);
 	   updateDirectives(joma, "StmSTPOpen", old_rel, new_rel);
 	   updateDirectives(joma, "OpenSPMScorecard", old_rel, new_rel);
 	   updateDirectives(joma, "ViewLink", old_rel, new_rel);


 	   SoftwareComponent web_SWC = joma.getSoftwareComponentByName(null, "Strategy Mgmt " + old_rel + " - WebServices");
	   deleteList = new ArrayList(1);
	   if (web_SWC != null) deleteList.add(web_SWC);
	   showDeleteList(deleteList);
	   joma.deleteObjects(deleteList);

	   SoftwareComponent prefSWC = updateSoftwareComponent(joma, "Strategy Mgnt Prefs XPOLL " + old_rel, old_rel, new_rel, true);
           prefSWC.setSpecVersion(new_rel);
           updateSWCNotes(joma, "spmprefxpoll", old_rel, new_rel, "upgradeSWCNotes");
	   prefSWC.updateMetadataAll();
       }

       if (isError)
       {
          String msg = "An error occured while processing one or more metadata updates check the log for previous error messages.";
          throw new Exception(msg);
       }

    }  // upgradeAll



    public SoftwareComponent updateSoftwareComponent(JomaSupport joma, String softwareComponentName, String old_rel, String new_rel, boolean commit)
    {
       SoftwareComponent softwareComponent = joma.getSoftwareComponentByName(null, softwareComponentName);

       String newName = softwareComponentName.replaceAll(old_rel, new_rel);

       if (softwareComponent == null)
       {  // The name may have changed but its versions may still need to be updated
                softwareComponent = joma.getSoftwareComponentByName(null, newName);
                if (softwareComponent == null)
                {

                   String msg = "\nWARN: " + softwareComponentName + " SoftwareComponent not found.  Was this product component installed successfully?";
                   warn(msg);
                   // throw new Exception(msg);
                   return null;
                }
       }



       softwareComponent.setSoftwareVersion(new_rel);
       // softwareComponent.setSpecVersion(new_rel);  // Let setConfiguredObject ake care of this

       String productName = softwareComponent.getProductName();
       productName.replaceAll(old_rel, new_rel);
       softwareComponent.setProductName(productName);



       String oldName = softwareComponent.getName();
       softwareComponent.setName(newName);


       if (commit)
       {
          softwareComponent.updateMetadataAll();
          newName = softwareComponent.getName();
          info("\nINFO:  SoftwareComponent: " + oldName + " has been updated and name set to: " + newName);
          return softwareComponent;
       }

       return softwareComponent

    } // updateSoftwareComponent


    public List updateNames(JomaSupport joma, String objName, String type, String old_rel, String new_rel, boolean commit)
    {
       String newName = objName.replaceAll(old_rel, new_rel);
       List objects = joma.getObjectsByName(null, type, objName);
       if (objects == null || objects.size() == 0)
       {
           objects = joma.getObjectsByName(null, type, newName);

           if (objects == null)
           {
             String msg = "\nWARN:  Neither old object or new object: " + objName + " / "   + newName +  " of type " +  type + " was found to update.";
             warn(msg);
             throw new Exception(msg);
           }
           else   info("\nINFO: Metadata " + type + " object: " + objName + "  has already been renamed to:  " + newName);

           return objects;  // Return already updated objects if any.
       }


       CMetadata mdObj = null;

       for (Iterator iter = objects.iterator(); iter.hasNext();)
       {
          mdObj = (CMetadata) iter.next();
          if (mdObj == null)
             continue;
          mdObj.setName(newName);
          if (commit)
          {
             mdObj.updateMetadataAll();
             info("\nINFO: " + objName + "of type " + type + " has been renamed to: " + newName);
          }

       }

       return objects;

    }

    public void updateDirectives(JomaSupport joma, String objName, String old_rel, String new_rel)
    {

        String newName = objName.replaceAll(old_rel, new_rel);

        String type = "ServiceComponent";
        List objects = joma.getObjectsByName(null, type, objName);
        if (objects == null || objects.size() == 0)
        {
            warn("\nDirective ServiceComponent: " + objName + " not found");
            return;
        }

       CMetadata mdObj = null;

       for (Iterator iter = objects.iterator(); iter.hasNext();)
       {
          mdObj = (CMetadata) iter.next();
          if (mdObj == null)
          {
             warn("\nDirective ServiceComponent: " + objName + " is null");
             continue;
          }

          PropertySet dPropSet = joma.getAssocObjByName(mdObj, "PropertySets", "Directive");
          if (dPropSet == null)
          {
             warn("\nWARN:  Directive ServiceComponent: " + objName + " has no associated PropertySets");
             continue;
          }

          String propName = "app-name";
          String oldPropValue = getPropertyValue(dPropSet, propName);
          if (oldPropValue == null)
             continue;

          String newPropValue = oldPropValue.replaceAll(old_rel, new_rel);
          if (newPropValue.equals(oldPropValue))
          {
             info("\nDirective: " + objName + " Property set: \t" + propName + "\n\tNew Value: " +  newPropValue + "\n\tOld Value: " + oldPropValue);
             continue;
          }

          oldPropValue = setProperty(dPropSet, "app-name", newPropValue);
          info("\nDirective: " + objName + " Property set: \t" + propName + "\n\tNew Value: " +  newPropValue + "\n\tOld Value: " + oldPropValue);

          if (objName.equals("SPM_ReportService"))
          {  // Change the url for SPM_ReportServic
			  propName = "uri";
			  newPropValue = "/ws1/ReportService";
			  oldPropValue = setProperty(dPropSet, "uri", newPropValue);
              info("\nDirective: " + objName + " Property set: \t" + propName + "\n\tNew Value: " +  newPropValue + "\n\tOld Value: " + oldPropValue);
		  }

       }

       return ;

    }


    public String setProperty(PropertySet propSet, String propName, String propVal)
    {
           String oldValue = "";
           CMetadata propObj = joma.getAssocObjByName(propSet, "SetProperties", propName);
           if (propObj == null)
           {
               propObj = joma.newProperty(null, propName);
               propSet.getSetProperties().add(propObj);
           }
           else oldValue = "" + propObj.getDefaultValue();

           propObj.setPropertyName(propName);
           propObj.setDefaultValue(propVal);
           propSet.updateMetadataAll();
           return oldValue;
    }

    public String getPropertyValue(PropertySet propSet, String propName)
    {

           CMetadata propObj = joma.getAssocObjByName(propSet, "SetProperties", propName);
           if (propObj == null)
           {
               return null;
           }
           String defaultValue = "" + propObj.getDefaultValue();
           return defaultValue;
    }


    public String addToDeleteList(CMetadata mdObj, String mdName, ArrayList deleteList)
    {

        if (mdObj == null || mdName == null || mdName.length() ==0)
        {
            warn("\nWARN: addToDeletList failed for metaObj = " + mdObj + " Name = " +  mdName);
            return mdname;
        }

        String oldName = mdObj.getName();
        String type = mdObj.getCMetadataType();

        if (oldName.equals(mdName))
        {
                // info("\nINFO: Metadata " + type + " object: " + mdName + "  has been added to delete list");
                deleteList.add(mdObj);
        }
        else
        {
            info("\nINFO: Metadata " + type + " object: " + oldName + "  does not match requested deletion for object name: " + mdName);
        }

    }

    public void showDeleteList(List deleteList)
    {
          for(Iterator i = deleteList.iterator(); i.hasNext();)
          {
                CMetadata obj = (CMetadata) i.next();
                info("Deleting: " + obj.getName());
          }
    }

    public void updateSWCNotes(JomaSupport joma, String swcClassId, String old_rel, String new_rel, String change_action)
    {
        int rc = 0;
        SoftwareComponent swc = joma.getSoftwareComponentByClsid("", swcClassId)

        if (swc == null)
        {
                error("updateSWCNotes SftwareComponent for ClassIdentifier : " + swcClassId +  " not found in metadata");
                return;
        }

        String swcName = swc.getName();

        info("Updating SWCNotes for SoftwareComponent: " + swc.getName() + " with ClassIdenfitier " + swcClassId);
        ;

        AssociationList notesList = swc.getNotes();


        if (notesList == null  || notesList.isEmpty())
        {
             warn("updateSWCNotes found no associated Notes for: " + swcName);
             return;

        }


        TextStore textStore;
        String textStr, updTextStr, textStoreName;

        for (Iterator iter = notesList.iterator(); iter.hasNext();)
        {
             textStore = (TextStore) iter.next();
             textStoreName = textStore.getName();
             textStr = textStore.getStoredText();
             updTextStr = textStr.replace(old_rel, new_rel);
             textStore.setStoredText(updTextStr);
             textStore.updateMetadataAll();

             info("Textstore: " + textStoreName + " for: " + swcClassId);
             info("Old TextStore: " + textStr);
             info("Upd TextStore: " + updTextStr);


		}

        info("updateSWCNotes completed rc = " + rc);
    }


   private void debug(String s)
   {
      if (DEBUG)
      {
           println(s);
      }
   }

   private void info(String s)
   {
        println(s);
   }

   private void warn(String s)
   {
        println(s);
   }

   private void error(String s)
   {
        println(s);
   }



}  // class UpgradeStmMetadataRelease

