package deploy.softwareComponents;
import com.sas.metadata.MetadataObjects;
import com.sas.metadata.MdObjectFactory;
import com.sas.metadata.remote.SoftwareComponent;
import com.sas.metadata.remote.DeployedComponent;
import com.sas.metadata.remote.Directory;
import com.sas.metadata.remote.ClassifierMap;
import com.sas.metadata.remote.Property;
import com.sas.metadata.remote.PropertySet;
import com.sas.metadata.remote.RelationalSchema;
import com.sas.metadata.remote.AssociationList;
import com.sas.metadata.remote.CMetadata;
import com.sas.metadata.remote.Tree;
import com.sas.metadata.remote.TypeDefinition;
import com.sas.metadata.remote.PromptGroup;
import com.sas.metadata.remote.ServiceComponent;
import com.sas.metadata.remote.TCPIPConnection;
import com.sas.metadata.remote.impl.ClassifierMapImpl;
import com.sas.metadata.remote.ClassifierMap;
import com.sas.metadata.remote.impl.DirectoryImpl;
import com.sas.metadata.remote.impl.PhysicalTableImpl;
import com.sas.metadata.remote.impl.SASLibraryImpl;
import com.sas.services.information.Factory;
import com.sas.metadata.remote.TextStore;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import deploy.JomaSupport;
import deploy.FolderUtils;
import deploy.SDWPlatformServices;


//----- RUN THE UPGRADE CODE -----

/*----------------------------------------------------------------------------*
  Groovy code to Change release components in part1 of the migration

  The to_release/from_release are passed in as attribute properties for running

  In the 2.4, 5.1, 5.2, 5.3 --> 5.3 framework this part 1 to upgrade and/or rename
  any metadata components that if present will interfere with laying down of the
  current release metadata.



 *----------------------------------------------------------------------------*/


JomaSupport joma = null;

boolean DEBUG = true;
int rc = 0;



try
{

    String user = attributes.get("admin.user");
    String password = attributes.get("admin.pass");
    String sastrustUser = attributes.get("sastrust.user");
    String sastrustPassword = attributes.get("sastrust.pass");
    String host= attributes.get("omr.host");
    String port = attributes.get("omr.port");
    String foundation = attributes.get("foundation.repos.nm");
    // String spmmidtier_swc_name = attributes.get("spmmidtier.software.component.name");
    String old_rel = attributes.get("from.release");
    String new_rel = attributes.get("to.release");
    String change_action = attributes.get("change.action");
    String new_dir = attributes.get("to.dir");

    String dbmsHost = attributes.get("data.dbms.host");
    String dbmsPort = attributes.get("data.dbms.port");
    String midtierHost = attributes.get("midtier.spmmidtier.host");
    String midtierPort = attributes.get("midtier.spmmidtier.port");


    if(DEBUG)
    {
      println("--- INPUT ATTRIBUTES ---");
      println("USER: " + user);
      println("SASTrust USER: " + sastrustUser);
      println("OMR HOST: " + host);
      println("OMR PORT: " + port);
      println("OMR FOUNDATION REPOS NAME: " + foundation);
     //  println("SPMMIDTIER SWC NAME: " + spmmidtier_swc_name.toString());
      println("FROM RELEASE: " + old_rel);
      println("TO RELEASE: "   + new_rel);
      println("CHANGE ACTION: "   + change_action);
      println("NEW DIR: " + new_dir);
      // println("PASSWORD: " + password);
    }

    joma = new JomaSupport();
    joma.connect(host, port, user, password);

    folderUtils = new FolderUtils(joma);

    ChangeReleaseComponents changeComps = new ChangeReleaseComponents(joma);
    if ("renameOld".equals(change_action))
       changeComps.renameOld(joma, old_rel, new_rel, change_action);

    if ("deleteOld".equals(change_action))
        changeComps.deleteOld(joma, old_rel, new_rel, change_action);

    if ("upgradeDatatier".equals(change_action))
    {
        changeComps.upgradeDatatier(joma, old_rel, new_rel, change_action);
        changeComps.updateSWCNotes(joma, "spmdatatier", old_rel, new_rel, "updateDatater");
    }

    if ("moveSWCInstallInfo".equals(change_action))
    {
        changeComps.moveSWCNotes(joma, old_rel, new_rel, change_action);    // Move SoftwareComponent Notes from old release software Componnent to new
        changeComps.copyDependsOnComponents(joma, old_rel, new_rel, change_action);
        changeComps.copyAppConfigProperties(joma, old_rel, new_rel, change_action);
        changeComps.moveTreeMembers(joma, old_rel, new_rel, change_action);

        // set the "StM.Formula.Compile.AtStartup" SoftwareComponent property to "true" so all formulas are compiled
        // on the first startup of StM after this migration.
        changeComps.setStartupCompileProperty(joma, new_rel, "true");
    }

    if ("setODCSEnabled.true".equals (change_action))
       changeComps.setODCSEnabled(joma, new_rel, true)
    if ("setODCSEnabled.false".equals (change_action))
       changeComps.setODCSEnabled(joma, new_rel, false);

    if ("fixExportedStMIMaps".equals(change_action))
      changeComps.fixExportedStMIMaps(joma, new_dir, host, port, foundation, sastrustUser, sastrustPassword);


    if ("fixHostandPort".equals(change_action))
    {
        dbmsHost = attributes.get("data.dbms.host");
        dbmsPort = attributes.get("data.dbms.port");
        midtierHost = attributes.get("midtier.spmmidtier.host");
        midtierPort = attributes.get("midtier.spmmidtier.port");

        println("-----start changeComps.fixHostAndPort-------");
        println("dbmsHost: "       + dbmsHost);
        println("dbmsPort: "       + dbmsPort);
        println("midtierHost: "    + midtierHost);
        println("midtierPort: "    + midtierPort);
        changeComps.fixHostAndPort(joma, new_rel, dbmsHost, dbmsPort, midtierHost, midtierPort);
        println("-----end  changeComps.fixHostAndPort-------");

    }

}
catch (Exception e)
{
    rc = 1;
    project.setNewProperty("outputproperty", e.toString());
    e.printStackTrace();
}
finally
{
  if(DEBUG) println("DONE.");
  if(joma != null)
    joma.disconnect();
}

project.setNewProperty("resultsproperty", "" + rc);

// ----- END RUN THE UPGRADE CODE -----

class ChangeReleaseComponents
{

    boolean DEBUG = true;
    JomaSupport joma;

    def excludedProps = ["WebApp.DocSet":"1",
                         "WebApp.SystemDocSet":"1",
                         "WebApp.Resources":"1",
                         "WebApp.Actions":"1",
                         "WebApp.MenuPrimary":"1",
                         "WebApp.MenuSecondary":"1",
                         "Version.Class":"1",
                         "IMapExport.StoredProcessDirectory": "1",
                         "PortalSoftwareComponentName":"1",
                         "SPMProvider.Midtier.trusted.domain":"1",
                         "SPMProvider.Midtier.trusted.URLs":"1"];



    public ChangeReleaseComponents(JomaSupport joma)
    {
         this.joma = joma;
    }

    public void fixHostAndPort(JomaSupport joma, String new_rel, String dbmsHost, String dbmsPort,
                               String midtierHost, String midtierPort)
    {


        String newSoftwareComponentName = "Strategy Mgmt " + new_rel;
        SoftwareComponent stmSWC = joma.getSoftwareComponentByName(null, newSoftwareComponentName);
        if (stmSWC == null)
        {
              error("ChangeReleaseComponents.fixHostAndPort: SoftwareComponent: " + newSoftwareComponentName +
                            " not found in metadata");
                    return;
        }

        PropertySet stmAppConfig = JomaSupport.getAssocObjByName(stmSWC, "PropertySets", "Application Configuration");

        String propName, oldPropValue, newPropValue;
        propName = "SPMProvider.Midtier.trusted.URLs"; newPropValue = midtierHost + ":" + midtierPort;
        oldPropValue = setProperty(stmAppConfig, propName, newPropValue);
         info("\nApplication Configuration Property set: \t" + propName + "\n\tNew Value: " +  newPropValue + "\n\tOld Value: " + oldPropValue);

        // Fix up TCPIPConnection Components  that startWith /SASScorecord or /SASStrategyManagement e.g. Strategy Mgmt x.y and Webservices

        TCPIPConnection tcpconn = null;
        def connList = ['/SASScorecard', '/SASStrategyManagement'];
        String xmlSelect;
        for (aConn in connList)
        {
           println ("aConn: " + aConn);
           xmlSelect = "@Service=:'" + aConn + "'";   // Select ones that START with either /SASScorecard or /SASStrategyManagement
           List conns = joma.getMetadataObjects(null, "TCPIPConnection", xmlSelect);

           for (Iterator iter = conns.iterator(); iter.hasNext();)
           {
               tcpconn = (TCPIPConnection) iter.next();

               String tcpconnName = tcpconn.getName();
               String tcpconnId =   tcpconn.getId();
               String tcpconnHost = tcpconn.getHostName();
               String tcpconnPort = tcpconn.getPort();
               // println("tcpconnPort:" + tcpconnPort);
               tcpconn.setHostName(midtierHost);
               Integer mp_LONG = Integer.parseInt(midtierPort);
               int mp_long = mp_LONG.intValue();

               tcpconn.setPort(mp_long);
               tcpconn.updateMetadataAll();

               String newHost =  tcpconn.getHostName();
               String newPort =  String.valueOf(tcpconn.getPort());

               println("TCPIPConnection old Values " + tcpconnName + "(" + tcpconnId + ")" + " host: " + tcpconnHost + " port: " + tcpconnPort );
               println("TCPIPConnection new Values " + tcpconnName + "(" + tcpconnId + ")" + " host: " + newHost + " port: " + newPort );

           }

        }

        CMetadata prop = null;

        // Update the Server properties for SPMDS SASLibrary

        xmlSelect = "Property[@PropertyName='Server'][AssociatedObject/SASClientConnection/Libraries/SASLibrary[@Name='SPMDS']]";
        List propList = joma.getMetadataObjects(null, "Property" , xmlSelect);

        for (Iterator iterProp = propList.iterator(); iterProp.hasNext();)
        {
            prop = (Property) iterProp.next();
            String oldServer = "" + prop.getDefaultValue();
            prop.setDefaultValue(dbmsHost);
            prop.updateMetadataAll();
            String newServer = prop.getDefaultValue();

            println("SPMDS Server property updated (new/old): " + newServer + " / " + oldServer);
        }


        // Update the Port properties for SPMDS SASLibrary

        xmlSelect = "Property[@PropertyName='MYSQL_PORT'][AssociatedObject/SASClientConnection/Libraries/SASLibrary[@Name='SPMDS']]";
        List propList2 = joma.getMetadataObjects(null, "Property", xmlSelect);
        for (Iterator iterProp2 = propList2.iterator(); iterProp2.hasNext();)
        {
            prop = (Property) iterProp2.next();
            prop.updateMetadataAll();
            String propName2 = prop.getPropertyName();
            String oldPort = "" + prop.getDefaultValue();
            prop.setDefaultValue(dbmsPort);
            prop.updateMetadataAll();
            String newPort = "" + prop.getDefaultValue();

            println("SPMDS " +  propName2 + " property updated (new/old): " + newPort + " / " + oldPort);

        }

    }

    public void fixExportedStMIMaps(JomaSupport joma, String new_dir, String host, String port, String foundation,
                                    String sastrustUser, String sastrustPassword)
    {
      // Modify the directory location for the stored processes to point to where the sas code actually resides on the new server.
      List directories = joma.getMetadataObjects(null, "Directory", "Directory[@Name='SP Source Directory'][Files/File[@FileName='spmexp2.sas']]");

      if (directories != null && directories.size() > 0) {
         DirectoryImpl directory = (DirectoryImpl) directories.get(0);
         String name = directory.getName();
         String id = directory.getId();
         directory.setDirectoryName(new_dir);
         directory.updateMetadataAll();
      }

      // Modify the metaserver property in all the StM IMap stored procs to contain the current metadata server id, port, foundation repos, sastrust userid and pw.
      List storedProcs = joma.getMetadataObjects(null, "ClassifierMap", "ClassifierMap[@PublicType='StoredProcess'][SourceCode/File[@FileName='spmexp2.sas']]");
      if (storedProcs != null && storedProcs.size() > 0) {
         System.out.println("going to modify metaserver parameter on " + storedProcs.size() + " stored procs.");

         Iterator iter = storedProcs.iterator();
         Factory factory = Factory.getInstance();
         while (iter.hasNext()) {
            ClassifierMapImpl cmi = (ClassifierMapImpl) iter.next();
            System.out.println("    processing stored proc: " + cmi.getName());
            PromptGroup prompt = cmi.getPrompts().get(0);
            String bigString = prompt.getGroupInfo();

            // create the Document object to parse.
            DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = docFactory.newDocumentBuilder();
            Document doc = builder.parse(new InputSource(new ByteArrayInputStream(bigString.getBytes("utf-8"))));

            Element root = doc.getDocumentElement();
            NodeList definitions = root.getElementsByTagName("DefinitionsAndSubgroups");

            boolean gotMetaServer = false;
            boolean gotMetaPort = false;
            boolean gotRepository = false;
            boolean gotmetauser = false;
            boolean gotmetapass = false;

            if (definitions != null) {
               // should only be one.
               Element definition = definitions.item(0);
               NodeList textDefs = root.getElementsByTagName("TextDefinition");
               for (int i = 0; i < textDefs.getLength(); i++) {

                  // go through all the TextDefinition nodes to look for the ones to be substituted.
                  Element el = textDefs.item(i);
                  if (el.getAttribute("name") == "metaserver") {

                     // got the metaserver node, now drill in to it and modify the actual server value.
                     NodeList serverVals = el.getElementsByTagName("DefaultValue");
                     if (serverVals != null) {
                        Element serverVal = serverVals.item(0);
                        NodeList stringVals = serverVal.getElementsByTagName("String");
                        if (stringVals != null) {
                           Element stringVal = stringVals.item(0);
                           // finally, set the new server name...
                           stringVal.setAttribute("value", host);
                           gotMetaServer = true;
                        }
                     }
                  }
                  else if (el.getAttribute("name") == "metaport") {
                     // got the metaport node, now drill in to it and modify the actual metaport value.
                     NodeList serverVals = el.getElementsByTagName("DefaultValue");
                     if (serverVals != null) {
                        Element serverVal = serverVals.item(0);
                        NodeList stringVals = serverVal.getElementsByTagName("String");
                        if (stringVals != null) {
                           Element stringVal = stringVals.item(0);
                           // finally, set the new server name...
                           stringVal.setAttribute("value", port);
                           gotMetaPort = true;
                        }
                     }
                  }
                  else if (el.getAttribute("name") == "repository") {
                     // got the metaport node, now drill in to it and modify the actual repository value.
                     NodeList serverVals = el.getElementsByTagName("DefaultValue");
                     if (serverVals != null) {
                        Element serverVal = serverVals.item(0);
                        NodeList stringVals = serverVal.getElementsByTagName("String");
                        if (stringVals != null) {
                           Element stringVal = stringVals.item(0);
                           // finally, set the new server name...
                           stringVal.setAttribute("value", foundation);
                           gotRepository = true;
                        }
                     }
                  }
                  else if (el.getAttribute("name") == "metauser") {
                     // got the metaport node, now drill in to it and modify the actual metauser value.
                     NodeList serverVals = el.getElementsByTagName("DefaultValue");
                     if (serverVals != null) {
                        Element serverVal = serverVals.item(0);
                        NodeList stringVals = serverVal.getElementsByTagName("String");
                        if (stringVals != null) {
                           Element stringVal = stringVals.item(0);
                           // finally, set the new server name...
                           stringVal.setAttribute("value", sastrustUser);
                           gotmetauser = true;
                        }
                     }
                  }
                  else if (el.getAttribute("name") == "metapass") {
                     // got the metaport node, now drill in to it and modify the actual metapass value.
                     NodeList serverVals = el.getElementsByTagName("DefaultValue");
                     if (serverVals != null) {
                        Element serverVal = serverVals.item(0);
                        NodeList stringVals = serverVal.getElementsByTagName("String");
                        if (stringVals != null) {
                           Element stringVal = stringVals.item(0);
                           // finally, set the new server name...
                           stringVal.setAttribute("value", sastrustPassword);
                           gotmetapass = true;
                        }
                     }
                  }
                  // once everything is set, break out to save time.
                  if (gotMetaServer && gotMetaPort && gotRepository && gotmetauser && gotmetapass)
                     break;
               }
            }

            root = doc.getDocumentElement();

            TransformerFactory transFactory = TransformerFactory.newInstance();
            Transformer transformer = transFactory.newTransformer();
            StringWriter buffer = new StringWriter();
            transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
            transformer.transform(new DOMSource(root), new StreamResult(buffer));
            String newXml = buffer.toString();

            prompt.setGroupInfo(newXml);
            prompt.updateMetadataAll();
         }
      }

      // Associate the GENERICSPMEXPORT PhysicalTable with the SASApp - SPMImapXport SASLibrary.
      List libraries = joma.getMetadataObjects(null, "SASLibrary", "SASLibrary[@Name='SASApp - SPMImapXport']");
      if (libraries != null && libraries.size() > 0) {
         SASLibraryImpl library = (SASLibraryImpl) libraries.get(0);

         List tables = joma.getMetadataObjects(null, "PhysicalTable", "PhysicalTable[@Name='GENERICSPMEXPORT']");


         if (tables != null && tables.size() > 0)
         {
            PhysicalTableImpl table = (PhysicalTableImpl) tables.get(0);

            // check to make sure the association is not already there.
            RelationalSchema test = table.getTablePackage();

            String oldLibName = "";
            if (test != null)
                oldLibName = test.getName();

            println("Old SASLibrary name is: " + oldLibName);


            if ((test == null) ||  (test != null && !oldLibName.equals("SASApp - SPMImapXport")))
            {
               // no library assigned, or its not assigned where we want it, so assign it.
               table.setTablePackage(library);
               table.updateMetadataAll();
               println("Added PhysicalTable  GENERICSPMEXPORT to SASApp - SPMImapXport SASLibrary");
            }
            else println("GENERICSPMEXPORT is already been added to SASApp - SPMImapXport SASLibrary");
         }
         else println("tables list is null, GENERICSPMEXPORT not added");
      }

    }

    public void renameOld(JomaSupport joma, String old_rel, String new_rel, String change_action)
    {
       // -- Main stm software component
       String softwareComponentName = "Strategy Mgmt " + old_rel;
       SoftwareComponent stmSWC = joma.getSoftwareComponentByName(null, softwareComponentName);

       FolderUtils folderUtils = new FolderUtils(joma);

       boolean isError = false;
       int rc = 0;
       // println ("old_rel:" + old_rel + " new_rel:" + new_rel + " change_action: " + change_action);


       Tree appSASCode = null
       if (old_rel.equals("5.1") || old_rel.equals("5.2") || old_rel.equals("5.3"))
       {
           appSASCode = folderUtils.getFolderFromPath("/System/Applications/SAS Strategy Management/Strategy Mgmt " +
                                                              old_rel + "/Application SAS Code");
           ClassifierMap clm = null;
           def clmList = ['SPMExportScorecard', 'MultiReg', 'Corr', 'CorrGroup', 'SingleReg', 'CorrRegGroup'];
           for (aClm in clmList)
           {
                   // println ("aClm: " + aClm);
                   clm = joma.getAssocObjByName(appSASCode, "Members", aClm);
                   if (clm != null)
                   {
                           changeName(clm, aClm+" -" + old_rel);
                       // println ("changeName called,  clm: " + clm)
                   }
                   else info("Old release: " + old_rel + " Classifier Map:  " + aClm + " not found");
           }
       }

       CMetadata cMd = null;
       cMd = joma.getObjectByName(null, "ServiceComponent", "SPMPortletsDirective");
       if (cMd != null)
           changeName(cMd, "SPMPortletsDirective" + " -" + old_rel);

       cMd = joma.getObjectByName(null, "ServiceComponent", "ScorecardDataEntryForm");
       if (cMd != null)
           changeName(cMd, "ScorecardDataEntryForm" + " -" + old_rel);

       cMd = joma.getObjectByName(null, "ServiceComponent", "SPM_Manage_TemplatesAndProjects");
       if (cMd != null)
           changeName(cMd, "SPM_Manage_TemplatesAndProjects" + " -" + old_rel);

       cMd = joma.getObjectByName(null, "ServiceComponent", "SPM_NewProject");
       if (cMd != null)
           changeName(cMd, "SPM_NewProject" + " -" + old_rel);


       cMd = joma.getObjectByName(null, "SASLibrary", "SASApp - SPMImapXport");
       if (cMd != null)
           changeName(cMd, "SASApp - SPMImapXport" + " -" + old_rel);

       cMd = joma.getObjectByName(null, "TypeDefinition", "SPM project");
       if (cMd != null)
           changeName(cMd, "SPM project" + " -" + old_rel);

       cMd = joma.getObjectByName(null, "TypeDefinition", "KPI project");
       if (cMd != null)
           changeName(cMd, "KPI project" + " -" + old_rel);

       cMd = joma.getObjectByName(null, "Prototype", "stm_bmf");
       if (cMd != null)
       {
          ArrayList deleteList = new ArrayList(1)
          addToDeleteList(cMd, "stm_bmf", deleteList);
          showDeleteList(deleteList);
          joma.deleteObjects(deleteList);  // delete old stm_bmf Prototype so new one can be imported
       }

       // Rename the flash portlet template name so it will match the new portlet.xml and make migrated flash portlets visible
       cMd = joma.getObjectByName(null, "Prototype", "StrategyManagement template");
       if (cMd != null)
           changeName(cMd, "PStrategyManagement template");


       Tree stm_Product = folderUtils.getFolderFromPath("/Products/SAS Strategy Management");
       if (stm_Product != null)
          changeName(stm_Product, "SAS Strategy Management" + " -" + old_rel);
       Tree stm_System = folderUtils.getFolderFromPath("/System/Applications/SAS Strategy Management");
       if (stm_System != null)
          changeName(stm_System, "SAS Strategy Management" + " -" + old_rel);


       info("ChangeReleaseComponents.renameOld() completed rc = " + rc);

    }  // changeNames




    public void upgradeDatatier(JomaSupport joma, String old_rel, String new_rel, String change_action)
    {

       CMetadata cMd;
       Property prop;
       int rc = 0;

       if ("5.3".equals(old_rel))
       {
            // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
            // Change the class associated with the SPMProject and KPIProject TypeDefinition metadata objects
            // to be the same SPMProjectType class
            // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


             List typedefList = joma.getMetadataObjects(null, "TypeDefinition", "TypeDefinition[@TypeName='SPMProject']");
             if (typedefList == null || typedefList.size() == 0)
             {
                error("upgradeOldDataTier:  SMProject Typdefinition  not found in metadata");
                throw new Exception("upgradeOldDataTier:  SMProject Typdefinition  not found in metadata, or multiple definitions ");

             }

             for (Iterator iter = typedefList.iterator(); iter.hasNext();)
             {
                cMd = (CMetadata) iter.next();
                prop = joma.getAssocObjByName(cMd, "Properties", "Logical Type Class")
                prop.setDefaultValue("com.sas.solutions.spm.core.metadata.logical.SPMProjectType");
             }

             typedefList = joma.getMetadataObjects(null, "TypeDefinition", "TypeDefinition[@TypeName='KPIProject']");
             if (typedefList == null || typedefList.size() == 0)
             {  // We don't care if the KPI one is gone
                error("upgradeOldDataTier:  KPIroject Typdefinition  not found in metadata");
                // throw new Exception("upgradeOldDataTier:  KPIProject Typdefinition  not found in metadata, or multiple definitions ");

             }


             for (Iterator iter = typedefList.iterator(); iter.hasNext();)
             {
                cMd = (CMetadata) iter.next();
                prop = joma.getAssocObjByName(cMd, "Properties", "Logical Type Class")
                prop.setDefaultValue("com.sas.solutions.spm.core.metadata.logical.SPMProjectType");
             }



        }

        // Should not be need if
        //       String DATATIERSWCPREFIX = "Strategy Mgmt Data Tier ";
        //       String newDataTierSWCName = DATATIERSWCPREFIX + new_rel;
        //       cMd = joma.getSoftwareComponentByClsid(null, "spmdatatier");
        //       if (cMd != null)
        //             changeName(cMd, newDataTierSWCName);


      // Rename the release /System/Applicaiton folder
       FolderUtils folderUtils = new FolderUtils(joma);
       Tree systemTree = folderUtils.getFolderFromPath("/System/Applications/SAS Strategy Management/Strategy Mgmt " + old_rel);
       if (systemTree != null)
          changeName(systemTree, "Strategy Mgmt " + new_rel);

       // Rename the release /Products/...jobs folder
       Tree jobs = folderUtils.getFolderFromPath("/Products/SAS Strategy Management/" + old_rel + " Jobs");
       if (jobs != null)
          changeName(jobs, new_rel + " Jobs");

       cMd = joma.getObjectByName(null, "Prototype", "stm_bmf");
       if (cMd != null)
       {
          ArrayList deleteList = new ArrayList(1)
          addToDeleteList(cMd, "stm_bmf", deleteList);
          showDeleteList(deleteList);
          joma.deleteObjects(deleteList);  // delete old stm_bmf Prototype so new one can be imported
       }

       info("ChangeReleaseComponents.upgradeOldDatatier() completed rc = " + rc);

    }  // upgradeOldDatatier




    public String changeName(CMetadata mdObj, String newName)
    {
        if (mdObj == null || newName == null || newName.length() ==0)
        {
            warn("\nWARN: changeName failed for metaObj = " + mdObj + "NewName = " +  newName);
            return newName;
        }
        String oldName = mdObj.getName();
        mdObj.setName(newName);
        mdObj.updateMetadataAll();
        String type = mdObj.getCMetadataType();
        String retNewName = mdObj.getName();
        info("\nINFO: Metadata " + type + " object: " + oldName + "  has been renamed to:  " + newName);
        return retNewName;
    }


    public void deleteOld(JomaSupport joma, String old_rel, String new_rel, String change_action)
    {
       // -- Main stm software component
       String softwareComponentName = "Strategy Mgmt " + old_rel;
       SoftwareComponent stmSWC = joma.getSoftwareComponentByName(null, softwareComponentName);

       ArrayList deleteList = new ArrayList(32);

       FolderUtils folderUtils = new FolderUtils(joma);

       boolean isError = false;
       int rc = 0;
       // println ("old_rel:" + old_rel + " new_rel:" + new_rel + " change_action: " + change_action);


       Tree appSASCode = null
       if (old_rel.equals("5.1") || old_rel.equals("5.2") || old_rel.equals("5.3"))
       {
           String pathName = "/System/Applications/SAS Strategy Management -" + old_rel + "/Strategy Mgmt " + old_rel + "/Application SAS Code";
           println("PathName is: " + pathName);
           appSASCode = folderUtils.getFolderFromPath(pathName);
           ClassifierMap clm = null;
           def clmList = ['SPMExportScorecard', 'MultiReg', 'Corr', 'CorrGroup', 'SingleReg', 'CorrRegGroup'];
           for (aClm in clmList)
           {
                   // println ("aClm: " + aClm);
                   clm = joma.getAssocObjByName(appSASCode, "Members", aClm+"-" + old_rel);
                   if (clm != null)
                   {
                           addToDeleteList(clm, aClm+" -" + old_rel, deleteList);
                       // println ("changeName called,  clm: " + clm)
                   }
                   else info("Old release: " + old_rel + " Classifier Map:  " + aClm+" -" + old_rel + " not found");
           }
       }

       CMetadata cMd = null;
       String someName;
       cMd = joma.getObjectByName(null, "ServiceComponent", "SPMPortletsDirective" +" -" + old_rel);
       if (cMd != null)
       {
           someName = "SPMPortletsDirective" + " -" + old_rel
           addToDeleteList(cMd, someName, deleteList);
       }

       cMd = joma.getObjectByName(null, "ServiceComponent", "ScorecardDataEntryForm" +" -" + old_rel);
       if (cMd != null)
       {
           someName  = "ScorecardDataEntryForm" + " -" + old_rel;
           addToDeleteList(cMd, someName, deleteList);
       }

       cMd = joma.getObjectByName(null, "ServiceComponent", "SPM_Manage_TemplatesAndProjects" +" -" + old_rel);
       if (cMd != null)
           addToDeleteList(cMd, "SPM_Manage_TemplatesAndProjects" + " -" + old_rel, deleteList);

       cMd = joma.getObjectByName(null, "ServiceComponent", "SPM_NewProject" + " -" + old_rel);
       if (cMd != null)
           addToDeleteList(cMd, "SPM_NewProject" + " -" + old_rel, deleteList);


       cMd = joma.getObjectByName(null, "SASLibrary", "SASApp - SPMImapXport" + " -" + old_rel );
       if (cMd != null)
           addToDeleteList(cMd, "SASApp - SPMImapXport" + " -" + old_rel, deleteList);

       cMd = joma.getObjectByName(null, "TypeDefinition", "SPM project" + " -" + old_rel);
       if (cMd != null)
           addToDeleteList(cMd, "SPM project" + " -" + old_rel, deleteList);

       cMd = joma.getObjectByName(null, "TypeDefinition", "KPI project" +" -" + old_rel);
       if (cMd != null)
           addToDeleteList(cMd, "KPI project" + " -" + old_rel, deleteList);

       cMd = joma.getObjectByName(null, "SoftwareComponent", "Strategy Mgmt "+ old_rel + " - WebServices");
       if ((cMd != null) && (old_rel != new_rel))
           addToDeleteList(cMd, "Strategy Mgmt "+ old_rel + " - WebServices", deleteList);


       String solutionsSWCName = "Solutions Svc " + old_rel;
       SoftwareComponent solutionsSWC = joma.getSoftwareComponentByName(null, solutionsSWCName);
       if ((solutionsSWC != null) && (old_rel != new_rel))
          addToDeleteList(solutionsSWC, solutionsSWCName, deleteList);


       Tree stm_Product = folderUtils.getFolderFromPath("/Products/SAS Strategy Management -" + old_rel);
       if (stm_Product != null)
           addToDeleteList(stm_Product, "SAS Strategy Management" + " -" + old_rel, deleteList);

       if (old_rel != new_rel)
           addToDeleteList(stmSWC, softwareComponentName, deleteList);
       Tree stm_System = folderUtils.getFolderFromPath("/System/Applications/SAS Strategy Management -" + old_rel);
       if (stm_System != null)
          addToDeleteList(stm_System, "SAS Strategy Management" + " -" + old_rel, deleteList);

       showDeleteList(deleteList);
       joma.deleteObjects(deleteList);  // Clean up objects

       info("ChangeReleaseComponents.deleteOld() completed rc = " + rc);

    }  // changeNames

    public String addToDeleteList(CMetadata mdObj, String mdName, ArrayList deleteList)
    {

        if (mdObj == null || mdName == null || mdName.length() ==0)
        {
            warn("\nWARN: addToDeletList failed for metaObj = " + mdObj + " Name = " +  mdName);
            return mdName;
        }

        String oldName = mdObj.getName();
        String type = mdObj.getCMetadataType();

        if (oldName.equals(mdName))
        {
                // info("\nINFO: Metadata " + type + " object: " + mdName + "  has been added to delete list");
                deleteList.add(mdObj);
        }
        else
        {
            info("\nINFO: Metadata " + type + " object: " + oldName + "  does not match requested deletion for object name: " + mdName);
        }

    }

    public void showDeleteList(List deleteList)
    {
          for(Iterator i = deleteList.iterator(); i.hasNext();)
          {
                CMetadata obj = (CMetadata) i.next();
                info("Deleting: " + obj.getName());
          }
    }

    public void moveSWCNotes(JomaSupport joma, String old_rel, String new_rel, String change_action)
    {
            int rc = 0;
        String oldSoftwareComponentName = "Strategy Mgmt " + old_rel;
        String newSoftwareComponentName = "Strategy Mgmt " + new_rel;
        SoftwareComponent oldSWC = joma.getSoftwareComponentByName(null, oldSoftwareComponentName);
        if (oldSWC == null)
        {
                error("ChangeReleaseComponents.moveSWCNotes: old SoftwareComponent: " + oldSoftwareComponentName +
                        " not found in metadata");
                return;
        }
        else info("ChangeReleaseComponents.moveSWCNotes: old SoftwareComponent: " + oldSoftwareComponentName);

        SoftwareComponent newSWC = joma.getSoftwareComponentByName(null, newSoftwareComponentName);
        if (newSWC == null)
        {
                error("ChangeReleaseComponents.moveSWCNotes: new SoftwareComponent: " + newSoftwareComponentName +
                        " not found in metadata");
                return;
        }
        else info("ChangeReleaseComponents.moveSWCNotes: new SoftwareComponent: " + newSoftwareComponentName);

        AssociationList oldNotesList = oldSWC.getNotes();
        AssociationList newNotesList = newSWC.getNotes();


        if (oldNotesList == null  || oldNotesList.isEmpty())
        {
                warn("ChangeReleaseComponents.moveSWCNotes: Old SoftwareComponent: " + oldSoftwareComponentName +
                                " has no associated installation Notes property associations");
                return;
        }


        if (newNotesList != null && !newNotesList.isEmpty())
        {
                warn("ChangeReleaseComponents.moveSWCNotes: New SoftwareComponent: " + newSoftwareComponentName +
                                " already has associated installation Notes property associations");

        }
        else
        {
                CMetadata obj;
                String compName;

                for (Iterator iter = oldNotesList.iterator(); iter.hasNext();)
                {
                        obj = (CMetadata) iter.next();
                        compName = obj.getName();
                        newNotesList.add(obj);
                        info("ChangeReleaseComponents.moveSWCNotes  copied: " + compName);

                }

                newSWC.setNotes(newNotesList);
                newSWC.updateMetadataAll();
        }

        oldNotesList.clear();
        oldSWC.setNotes(oldNotesList);
        oldSWC.updateMetadataAll();

        info("ChangeReleaseComponents.moveSWCNotes() completed rc = " + rc);
    }

    public void moveTreeMembers(JomaSupport joma, String old_rel, String new_rel, String change_action)
    {
        FolderUtils folderUtils = new FolderUtils(joma);

        boolean isError = false;
        int rc = 0;
        // println ("old_rel:" + old_rel + " new_rel:" + new_rel + " change_action: " + change_action);

        String oldFolderPath = "/Products/SAS Strategy Management -" + old_rel + "/Tables";
        String newFolderPath = "/Products/SAS Strategy Management/Tables";

        info("ChangeReleaseComponents.moveTreeMembers:  Old folder path" + oldFolderPath);
        info("ChangeReleaseComponents.moveTreeMembers:  New folder path" + newFolderPath);


        Tree oldTree = folderUtils.getFolderFromPath(oldFolderPath);
        if (oldTree == null)
           error("Old Tree not found: " + oldFolderPath);
        Tree newTree = folderUtils.getFolderFromPath(newFolderPath);
        if (newTree == null)
           error("New Tree not found: " + newFolderPath);

        if (oldTree == null || newTree == null)
        {
                error("ChangeReleaseComponents.moveTreeMembers: old or new Tables folder not found in metadata not found in metadata");
                return;
        }

        AssociationList oldMembersList = oldTree.getMembers();
        AssociationList newMembersList = newTree.getMembers();


        if (oldMembersList == null  || oldMembersList.isEmpty())
        {
                warn("ChangeReleaseComponents.moveTreeMembers: Old folder: " + oldFolderPath +
                                " has no members");
                return;
        }

        if (newMembersList == null)
        {
           newMembersList = new AssociationList("Members");
        }

        CMetadata obj;
        String compName;

        for (Iterator iter = oldMembersList.iterator(); iter.hasNext();)
        {
             obj = (CMetadata) iter.next();
             compName = obj.getName();
             if (compName.startsWith("SASApp - SPMImapXport") ||
                 compName.startsWith("SPMDS"))
                continue;

             newMembersList.add(obj);
             info("ChangeReleaseComponents.moveTreeMembers  moved: " + compName);

        }

        newTree.setMembers(newMembersList);
        newTree.updateMetadataAll();

        oldMembersList.clear();
        oldTree.setMembers(oldMembersList);
        oldTree.updateMetadataAll();

        info("ChangeReleaseComponents.moveTreeMembers() completed rc = " + rc);
    }

    public void setODCSEnabled(JomaSupport joma, String new_rel, boolean bEnabled)
    {
            int rc = 0;
        String newSoftwareComponentName = "Strategy Mgmt " + new_rel;
        SoftwareComponent stmSWC = joma.getSoftwareComponentByName(null, newSoftwareComponentName);
        if (stmSWC == null)
        {
                error("ChangeReleaseComponents.setODCSEnabled: new SoftwareComponent: " + newSoftwareComponentName +
                        " not found in metadata");
                return;
        }

        PropertySet stmAppConfig = JomaSupport.getAssocObjByName(stmSWC, "PropertySets", "Application Configuration");

        String propName, oldPropValue, newPropValue;
        propName = "ODCS.Enabled"; newPropValue = (bEnabled)?"true":"false" ;
        oldPropValue = setProperty(stmAppConfig, propName, newPropValue);
        info("\nApplication Configuration Property set: \t" + propName + "\n\tNew Value: " +  newPropValue + "\n\tOld Value: " + oldPropValue);
    }


    public void setStartupCompileProperty(JomaSupport joma, String new_rel, String newPropValue) {
        int rc = 0;
        String newSoftwareComponentName = "Strategy Mgmt " + new_rel;
        SoftwareComponent stmSWC = joma.getSoftwareComponentByName(null, newSoftwareComponentName);
        if (stmSWC == null) {
                error("ChangeReleaseComponents.setStartupCompileProperty: new SoftwareComponent: " + newSoftwareComponentName +
                        " not found in metadata");
                return;
        }

        PropertySet stmAppConfig = JomaSupport.getAssocObjByName(stmSWC, "PropertySets", "Application Configuration");

        String propName, oldPropValue;
        propName = "StM.Formula.Compile.AtStartup";
        oldPropValue = setProperty(stmAppConfig, propName, newPropValue);
        info("\nApplication Configuration Property set: \t" + propName + "\n\tNew Value: " +  newPropValue + "\n\tOld Value: " + oldPropValue);
    }



    public void copyDependsOnComponents(JomaSupport joma, String old_rel, String new_rel, String change_action)
    {
        String oldSoftwareComponentName = "Strategy Mgmt " + old_rel;
        String newSoftwareComponentName = "Strategy Mgmt " + new_rel;
        String DATATIERSWCPREFIX = "Strategy Mgmt Data Tier ";
        String newDataTierSWCName = DATATIERSWCPREFIX + new_rel;
        SoftwareComponent oldSWC = joma.getSoftwareComponentByName(null, oldSoftwareComponentName);

        int rc = 0;

        if (oldSWC == null)
        {
                error("ChangeReleaseComponents.copyDependsOnComponents: old SoftwareComponent: " + oldSoftwareComponentName +
                        " not found in metadata");
                return;
        }

        SoftwareComponent newSWC = joma.getSoftwareComponentByName(null, newSoftwareComponentName);
        if (newSWC == null)
        {
                error("ChangeReleaseComponents.copyDependsOnComponents: new SoftwareComponent: " + newSoftwareComponentName +
                        " not found in metadata");
                return;
        }

        SoftwareComponent newDTSWC = joma.getSoftwareComponentByName(null, newDataTierSWCName);
        if (newSWC == null)
        {
                error("ChangeReleaseComponents.copyDependsOnComponents: new Data Tier SoftwareComponent: " + newDataTierSWCName +
                        " not found in metadata");
                return;
        }


        AssociationList oldDependsList = oldSWC.getDependsOnComponents();
        AssociationList newDependsList = newSWC.getDependsOnComponents();

        if (newDependsList == null)
        {
                newDependsList = new AssociationList("DependsOnComponents");
        }

        if (oldDependsList == null  || oldDependsList.isEmpty())
        {
                warn("ChangeReleaseComponents.copyDependsOnComponents: Old SoftwareComponent: " + oldSoftwareComponentName +
                                " has no associated installation DependsOnComponents property associations");
                return;
        }

        String compName = null;
        HashMap nameMap = new HashMap();
        for (Iterator iter = newDependsList.iterator(); iter.hasNext();)
        {
                CMetadata obj = (CMetadata) iter.next();
                compName = obj.getName();
                nameMap.put(compName, obj);
        }

        for (Iterator iter = oldDependsList.iterator(); iter.hasNext();)
        {
                CMetadata obj = (CMetadata) iter.next();
                compName = obj.getName();

                if (nameMap.containsKey(compName))
                {
                        info("ChangeReleaseComponents.copyDependsOnComponents:  already contains: " + compName);
                        continue;
                }

                if (compName.startsWith("Solutions"))
                {
                        info("ChangeReleaseComponents.copyDependsOnComponents:  excluded: " + compName);
                        continue;  // Remove Solutions ... dependency
                }

                if (compName.startsWith(DATATIERSWCPREFIX))
                {
                        info("ChangeReleaseComponents.copyDependsOnComponents:  excluded: " + compName);
                        obj = newDTSWC;   // Use new Strategy Mgmt DataTier
                        compName = obj.getName();
                }

                newDependsList.add(obj);
                info("ChangeReleaseComponents.copyDependsOnComponents:  added " + compName);

        }

        newSWC.setDependsOnComponents(newDependsList);
        newSWC.updateMetadataAll();

        info("ChangeReleaseComponents.copyDependsOnComponents() completed rc = " + rc);
    }

    public void copyAppConfigProperties(JomaSupport joma, String old_rel, String new_rel, String change_action)
    {

       String oldSoftwareComponentName = "Strategy Mgmt " + old_rel;
       String newSoftwareComponentName = "Strategy Mgmt " + new_rel;

       //  String pmReposName = JomaSupport.getPerformanceManagementRepositoryName();  // 9.1.3 location
       // String pmReposID = joma.getRepositoryId(pmReposName);
       // CMetadata oldSoftwareComponent = joma.getSoftwareComponentByName(pmReposID, softwareComponentName);
       CMetadata oldSoftwareComponent = joma.getSoftwareComponentByName(null, oldSoftwareComponentName);

       if (oldSoftwareComponent == null)
       {
                String msg = softwareComponentName + " SoftwareComponent not found. A previous migration run may have deleted it.";
                error(msg);
                return;
       }

       List oldPropList = new ArrayList();
       JomaSupport.addApplicationConfiguration(oldSoftwareComponent, oldPropList);

       // debug(oldPropList.toString());

       CMetadata newSoftwareComponent = joma.getSoftwareComponentByName(null, newSoftwareComponentName);
       PropertySet newAppConfig = (PropertySet) JomaSupport.getAssocObjByName(newSoftwareComponent, "PropertySets", "Application Configuration");

       String propName;
       String newPropValue, oldPropValue;
       Property migProperty;
       int pix;

       for (pix=1; pix < oldPropList.size(); pix++)
       {

           migProperty = (Property) oldPropList.get(pix);
           propName = migProperty.getPropertyName();

           // debug("properyName, pix: " + propName + ", " + pix);

           if (propName == null || propName.length() == 0)
              propName = migProperty.getName();

           if (propName == null || propName.length() == 0)
           {
                info("\nCould not obtain migration PropertyName:" + softwareComponentName + ".Application Configuration." + propName);
                pix++; continue;  // Groovy continue does not increment loop index
           }

           if (null != excludedProps.get(propName))
           {
                info("\nApplication Configuration Property: " + propName + " is in the excluded migration property list");
                pix++; continue;  // Groovy continue does not increment loop index
           }

           newPropValue = "" + migProperty.getDefaultValue();
           oldPropValue = setProperty(newAppConfig, propName, newPropValue);
           info("\nApplication Configuration Property set: \t" + propName + "\n\tNew Value: " +  newPropValue + "\n\tOld Value: " + oldPropValue);
       }

    }

    public String setProperty(PropertySet propSet, String propName, String propVal)
    {
           String oldValue = "";
           CMetadata propObj = joma.getAssocObjByName(propSet, "SetProperties", propName);
           if (propObj == null)
           {
               propObj = joma.newProperty(null, propName);
               propSet.getSetProperties().add(propObj);
           }
           else oldValue = "" + propObj.getDefaultValue();

           propObj.setPropertyName(propName);
           propObj.setDefaultValue(propVal);
           propSet.updateMetadataAll();
           return oldValue;
    }

    public String getPropertyValue(PropertySet propSet, String propName)
    {

           CMetadata propObj = joma.getAssocObjByName(propSet, "SetProperties", propName);
           if (propObj == null)
           {
               return null;
           }
           String defaultValue = "" + propObj.getDefaultValue();
           return defaultValue;
    }


    public void updateSWCNotes(JomaSupport joma, String swcClassId, String old_rel, String new_rel, String change_action)
    {
        int rc = 0;
        SoftwareComponent swc = joma.getSoftwareComponentByClsid("", swcClassId)

        if (swc == null)
        {
                error("updateSWCNotes SftwareComponent for ClassIdentifier : " + swcClassId +  " not found in metadata");
                return;
        }

        String swcName = swc.getName();

        info("Updating SWCNotes for SoftwareComponent: " + swc.getName() + " with ClassIdenfitier " + swcClassId);
        ;

        AssociationList notesList = swc.getNotes();


        if (notesList == null  || notesList.isEmpty())
        {
             warn("updateSWCNotes found no associated Notes for: " + swcName);
             return;

        }


        TextStore textStore;
        String textStr, updTextStr, textStoreName;

        for (Iterator iter = notesList.iterator(); iter.hasNext();)
        {
             textStore = (TextStore) iter.next();
             textStoreName = textStore.getName();
             textStr = textStore.getStoredText();
             updTextStr = textStr.replace(old_rel, new_rel);
             textStore.setStoredText(updTextStr);
             textStore.updateMetadataAll();

             info("Textstore: " + textStoreName + " for: " + swcClassId);
             info("Old TextStore: " + textStr);
             info("Upd TextStore: " + updTextStr);


        }

        info("updateSWCNotes completed rc = " + rc);
    }

   private void debug(String s)
   {
      if (DEBUG)
      {
           println(s);
      }
   }

   private void info(String s)
   {
        println(s);
   }

   private void warn(String s)
   {
        println(s);
   }

   private void error(String s)
   {
        println(s);
   }

}  // class ChangeReleaseComponents


