define([
    'dojo/_base/declare',
    'dojo/_base/array',
    'dojo/dom-construct',
    'dojo/store/Cache',
    'dojo/store/JsonRest',
    'dojo/store/Memory',
    'dojo/dom-class',
    'dojo/aspect',
    'dojo/when',
    'dojo/query',
    'dijit/Tree',
    'dijit/tree/ObjectStoreModel',
    'dijit/form/CheckBox',
    'STM/util/ApplicationConstants',
    'STM/util/Util',
    'STM/view/DialogTemplate'
], function(declare, array, domConstruct, Cache, JsonRest, Memory, domClass, aspect, when, query, Tree, ObjectStoreModel,
            CheckBox, GeneralAppConst, STMUtil, DialogTemplate) {

    var createStore = function() {
        var master = new JsonRest({
            target: GeneralAppConst.REST_URL_ROOT + GeneralAppConst.REST_SCORECARDS + '/' + GeneralAppConst.PROJECT_ID + '/',
            getChildren: function(object) {
                return this.get(object.id).then(function(children) {
                    object.children = children;
                    return children;
                });
            }
        });
        var cache = new Memory({
            getChildren: function(object) {
                return object.children;
            }
        });
        var store = new Cache(master, cache);
        store.master = master;
        store.cache = cache;

        return store;
    };

    var createModel = function(store, tree) {
        return new ObjectStoreModel({
            store: store,
            getRoot: function(onItem) {
                var me = this;
                when(me.store.get(''), function(result) {
                    me.root = result[0];
                    when(onItem(result[0]), function() {
                        expandRootChildren(tree);
                    });
                }, function() {
                    new DialogTemplate({
                        type: GeneralAppConst.ERROR_SERVER_DATA_DIALOG
                    }).show();
                });
            },
            mayHaveChildren: function(object) {
                return object.scorecardChildren || object.hasElements;
            }
        });
    };
    
    var expandRootChildren = function(tree) {
        tree.model.getChildren(tree.model.root, function(children) {
            array.forEach(children, function(child) {
                var childNode = tree.getNodesByItem(child)[0];
                when(tree._expandNode(childNode), function() {/**Do nothing on success**/}, function() {
                    new DialogTemplate({
                        type: GeneralAppConst.ERROR_SERVER_DATA_DIALOG
                    }).show();
                });
            });
        }, function() {
            new DialogTemplate({
                type: GeneralAppConst.ERROR_SERVER_DATA_DIALOG
            }).show();
        });
    };
    
    // Handle insertion/removal for the element list array of strings
    var manageArray = function(item, myArray, inserting) {
        var index = array.indexOf(myArray, item);
        
        if (inserting && index === -1) {
            myArray.push(item);
        }
        
        if (!inserting && index !== -1) {
            myArray.splice(index, 1);
        }
    };
    
    var walkTree = function(startNode, callback) {
        // Callback on this node
        callback(startNode);
        
        if (startNode.children && startNode.children.length > 0) {
            array.forEach(startNode.children, function(child) {
                walkTree(child, callback);
            });
        }
    };
    
    return declare(Tree, {
        showRoot: false,
        persist: false,
        // Keep track of whether the tree was created with a list of saved elements
        hasSavedElements: false,
        'class': 'stmScorecardElementTree',
        constructor: function(args) {
            this.setupTree(args['elementTypeId'], args['elementList'], args['scorecardList']);
        },
        setupTree: function(elementTypeId, elementList, scorecardList) {
            var me = this;
            var store = createStore();
            var model = createModel(store, me);

            aspect.after(model, 'getRoot', function() {
                store.master.target = store.master.target + elementTypeId + '/';
            });
            
            me.elementList = elementList || [];
            me.hasSavedElements = me.elementList.length > 0;
            me.scorecardList = scorecardList || [];

            me.store = store;
            me.model = model;
        },
        getIconClass: function(item, opened) {
            return item.element ? 'sasDocumentIcon' : 'sasScorecardIcon';
        },
        // Overrides
        _createTreeNode: function(node) {
            var me = this;
            var item = node.item;
            var treeNode = new Tree._TreeNode(node);
            var checkBox = new CheckBox({
                onChange: function(checked) {
                    var store = treeNode.tree.store;
                    var parentScorecardNode;
                    
                    // Scorecard
                    if (!item.element) {
                        // Add/remove scorecard id from list
                        manageArray(item.id, me.scorecardList, checked);
                        
                        when(store.cache.getChildren(item), function(children) {
                            array.forEach(children, function(child) {
                                if (child.element) {
                                    STMUtil.formSetNoChange(child.checkBoxRef, 'checked', checked);

                                    // Checking or unchecking both result in no child elements in the elementList
                                    manageArray(child.id, me.elementList, false);
                                    me.emit(GeneralAppConst.STM_SC_TREE_ELEMENT_SELECTED);
                                }
                            });
                        }, function() {
                            new DialogTemplate({
                                type: GeneralAppConst.ERROR_SERVER_DATA_DIALOG
                            }).show();
                        });
                    }
                    // Element
                    else {
                        if (checked) {
                            manageArray(item.id, me.elementList, true);
                            me.emit(GeneralAppConst.STM_SC_TREE_ELEMENT_SELECTED);
                        }
                        else {
                            parentScorecardNode = treeNode.getParent();
                            
                            // Is the scorecard node checked?
                            if (array.indexOf(me.scorecardList, parentScorecardNode.item.id) !== -1) {
                                // Remove scorecard from scorecardList
                                manageArray(parentScorecardNode.item.id, me.scorecardList, false);
                                // Uncheck scorecard
                                STMUtil.formSetNoChange(parentScorecardNode.item.checkBoxRef, 'checked', false);
                                
                                // Add all remaining checked elements back into elementList
                                when(store.cache.getChildren(parentScorecardNode.item), function(children) {
                                    array.forEach(children, function(child) {
                                        if (child.element) {
                                            if (child.id !== item.id) {
                                                // Add it to elementList unless it's the item that was just unchecked
                                                manageArray(child.id, me.elementList, true);
                                            }
                                        }
                                    });
                                });
                            }
                            else {
                                manageArray(item.id, me.elementList, false);
                            }

                            me.emit(GeneralAppConst.STM_SC_TREE_ELEMENT_SELECTED);
                        }
                    }
                }
            });
            
            // Key a reference to the checkBox on each item
            item.checkBoxRef = checkBox;
            
            domConstruct.place(checkBox.domNode, treeNode.contentNode, 'first');
            
            if (!item.element) {
                // Have to wait until the treeNode has been added to its parent before we can determine if it should be checked
                treeNode.addChild = function(widget, insertIndex) {
                    var shouldBeChecked, child;
                    
                    Tree._TreeNode.prototype.addChild.call(treeNode, widget, insertIndex);
                    
                    if (widget instanceof Tree._TreeNode) {
                        child = widget.item;
                        shouldBeChecked = array.indexOf(me.elementList, child.id) !== -1 || array.indexOf(me.scorecardList, child.id) !== -1
                            || (child.element && array.indexOf(me.scorecardList, item.id) !== -1);
    
                        STMUtil.formSetNoChange(child.checkBoxRef, 'checked', shouldBeChecked);
                    }
                };
            }
            
            return treeNode;
        },
        // Other
        getSelectedElements: function() {
            return this.elementList;
        },
        getSelectedScorecards: function() {
            return this.scorecardList;
        },
        setSelectedElements: function(elements) {
            var root;
            
            this.model.getRoot(function(root) {
                walkTree(root, function(node) {
                    var inList;
                    if (node.element) {
                        inList = array.some(elements, function(elementId) {
                            return elementId === node.id;
                        });
                        
                        if (inList) {
                            STMUtil.formSetNoChange(node.checkBoxRef, 'checked', true);
                        }
                    }
                });
            });
            
            // Merge elements into elementList
            array.forEach(elements, function(element) {
                if (array.indexOf(this.elementList, element) !== -1) {
                    this.elementList.push(element);
                }
            });
            
        }
    });
    
});