define([
    'dojo/i18n!html/nls/STMResources',
    'dojo/text!html/app/view/dataentry/components/DataEntryGridView.html',
    'dojo/_base/lang',
    'dojo/_base/declare',
    'dojo/string',
    'dojo/query',
    'dojo/dom',
    'dojo/dom-style',
    'dojo/when',
    'dojo/number',
    'dojo/topic',
    'dojo/aspect',
    'dojo/_base/array',
    'dojo/dom-construct',
    'dijit/registry',
    'dijit/popup',
    'dojox/grid/DataGrid',
    'dojo/store/Memory',
    'dojo/data/ObjectStore',
    'dojo/_base/xhr',
    'dijit/form/NumberTextBox',
    'dijit/form/TextBox',
    'dijit/form/Textarea',
    'dijit/form/Select',
    'dijit/form/Button',
    'dijit/layout/ContentPane',
    'STM/util/dataentry/ApplicationConstants',
    'dojo/_base/event',
    'STM/model/dataentry/DataEntryRuleValidateStore',
    'STM/util/Util'
], function(resources, templateString, lang, declare, string, query, dom, domStyle, when, number, topic, aspect, array,
            domConstruct, registry, popup, DataGrid, Memory, ObjectStore, xhr, NumberTextBox, TextBox, Textarea,
            ComboBox, Button, ContentPane, AppConst, event, DataEntryRuleValidateStore, STMUtil) {

    var showNoElementsView = function(view) {
        var noElementsLabel = dom.byId('noElementsLabel');

        // Assuming noElementsLabel === null means we're not coming from a no elements view
        if (!noElementsLabel) {
            // Hide existing content
            STMUtil.hideImmediateChildren(view.domNode);

            // Add the message
            STMUtil.createCenteredTextNode(
                resources['de.portlet.form.datagrid.no.elements.message.txt'],
                'noElementsLabel',
                view.domNode
            );
        }
    };

    var hideNoElementsView = function(view) {
        var noElementsLabel = dom.byId('noElementsLabel');

        // Assuming noElementsLabel === null means we're not coming from a no elements view
        if (noElementsLabel) {
            // Remove the message
            domConstruct.destroy(noElementsLabel);

            // Show existing content
            STMUtil.showImmediateChildren(view.domNode);
        }
    };

    var skipFunctions = function(object, funcNames) {
        // Add single function name to an array if needed to normalize handling
        var funcList = STMUtil.isNonEmptyArray(funcNames) ? funcNames : /* single function */ [funcNames]

        array.forEach(funcNames, function(funcName) {
            aspect.around(object, funcName, function() {
                return function() { /* Do nothing */ }
            });
        });
    };

    var addCursorFix = function(component) {
        component.on('focus', function() {
            var textbox = this.textbox;
            var length = textbox.value.length;
            var range;

            // IE
            if (textbox.createTextRange) {
                range = textbox.createTextRange();
                range.move('character', length);
                range.select();
            }
            else {
                // FF
                if (textbox.setSelectionRange) {
                    textbox.focus();
                    textbox.setSelectionRange(length, length);
                }
                else {
                    textbox.focus;
                }
            }
        });
    };

    var toggleWriteAccessMessage = function(show) {
        var writeAccessMessageNode = dom.byId('writeAccessMessageNode');

        if (show) {
            if (!writeAccessMessageNode) {
                domConstruct.place(
                    '<p id="writeAccessMessageNode" style="display: inline-block;">' +
                        resources['de.portlet.form.datagrid.no.write.access.message.txt'] +
                        '</p>',
                    dom.byId('dataentryView')
                );
            }
        }
        else {
            if (writeAccessMessageNode) {
                domConstruct.destroy(writeAccessMessageNode);
            }
        }
    };

    return declare(ContentPane, {
        //
        // Properties
        //
        id: 'dataentryView',
        changedData: {},
        region: 'center',
        //
        // Overrides
        //
        postCreate: function() {
            var me = this;

            // Do string substitution
            templateString = string.substitute(templateString, {
                saveButtonLabel: resources['save.txt'],
                submitButtonLabel: resources['de.portlet.form.datagrid.submit.txt'],
                discardFieldLabel: resources['de.portlet.form.datagrid.discard.txt']
            });

            me.set('content', templateString);
        },

        createApprovalTable: function(data, isrefresh) {
            //
            // Begin button setup/handling code
            //

            var me = this;
            var approveField;
            var rejectField;

            // Destroy existing buttons
            registry.byId('gridsave').destroyRecursive();
            registry.byId('gridsubmit').destroyRecursive();
            registry.byId('discardField').destroyRecursive();

            // Replace Save and Submit with Approve and Reject
            approveField = new Button({
                id: 'approveField',
                label: resources['de.portlet.form.datagrid.approve.txt'],
                style: 'float: left'
            });
            me.addChild(approveField);

            rejectField = new Button({
                id: 'rejectField',
                label: resources['de.portlet.form.datagrid.reject.txt'],
                style: 'float: left'
            });
            me.addChild(rejectField);

            me.own(
                // Wire up events
                registry.byId('approveField').on('click', function() {
                    topic.publish(AppConst.APPROVE_FORM_CLICK);
                }),
                registry.byId('rejectField').on('click', function() {
                    topic.publish(AppConst.REJECT_FORM_CLICK);
                })
            );

            //
            // End button setup/handling code
            //

            // Redefine the function so that the code above only runs once
            me.createApprovalTable = function(data, isrefresh) {
                var newData = {};
                var grid, store, dataStore;

                if (!STMUtil.isNonEmptyArray(data.rows)) {
                    showNoElementsView(me);
                }
                else {
                    hideNoElementsView(me);
                    // Make sure it is off before processing element rows
                    toggleWriteAccessMessage(false);

                    store = new Memory({
                        data: data.rows,
                        idProperty: "elementId"
                    });


                    dataStore = new ObjectStore({ objectStore: store });
                    //       me.oldData =  lang.clone(store);

                    newData.columns = data.columns;
                    newData.id = AppConst.DATA_ENTRY_ID;
                    var newrows = [];
                    newData.rows = newrows;

                    me.changedData = newData;


                    var gridLayout = [];
                    var gLayout = [];
                    var key, i, colName;

                    var showflag = false;

                    var elementname = "<p>" + resources['de.portlet.form.datagrid.elementname.txt'] + "</p>";
                    gridLayout.push({name: elementname, field: "elementName", styles: 'text-align: left;',
                        get:  function(rowIndex, item) {
                            if (item != null) {
                                return dataStore.getValue(item, "elementName") + "|" + dataStore.getValue(item, "writable");
                            }
                        },
                        formatter: function(item, rowIndex) {
                            var arrays = item.split("|");

                            if(arrays[1] == "false" && showflag == false)
                            {
                                showflag = true;
                                toggleWriteAccessMessage(showflag);
                                registry.byId('approveField').set('disabled', true);
                            }
                            if(arrays[1] == "true")
                                return "<b>" + arrays[0] + "</b>";
                            else
                                return "<b>*" + arrays[0] + "</b>";

                        }
                    });
                    var modified = "<p>" + resources['de.portlet.form.datagrid.modify.txt'] + "</p>";
                    gridLayout.push({name: modified, field: "userid", styles: 'text-align: center;' });

                    var scorecard = "<p>" + resources['de.portlet.form.datagrid.scorecard.txt'] + "</p>";
                    gridLayout.push({name: scorecard, field: "scorecardName", styles: 'text-align: center;' });

                    var period = "<p>" + resources['de.portlet.form.datagrid.period.txt'] + "</p>";
                    gridLayout.push({name: period, field: "periodName", styles: 'text-align: center;' });

                    gLayout.push({ cells: gridLayout, noscroll: true,
                        width: 'auto' });
                    gridLayout = [];

                    for (i = 0; i < data.columns.length; i++) {
                        var colName = "<b>" + data.columns[i].columnName + "<br>";
                        var cfield = data.columns[i].columnId + "value";
                        var ttype = data.columns[i].columnType;
                        if(ttype == "value")
                            colName = "<b>" + data.columns[i].columnName + "<br>" + resources['number.txt'] + "</b>";
                        else
                            colName = "<b>" + data.columns[i].columnName + "<br>" + resources['text.txt'] + "</b>";

                        if(ttype == "value")
                        {
                        	   gridLayout.push(
                                {
                                    name: colName, field: cfield, styles: 'text-align: right;',
                                    formatter: function(item) {
                                        if(item == null || item== "")
                                            return item;
                                        else
                                            return number.format(item);
                                    }});
                        }
                        else
                        {
                        	   gridLayout.push(
                                {
                                    name: colName, field: cfield, styles: 'text-align: right;'
                                });
                        }
                    }
                    gLayout.push({ cells: gridLayout });

                    var rgrid = registry.byId("grid");
                    
                    if (rgrid) {
                        rgrid.set('structure', gLayout);
                        rgrid.setStore(dataStore);
                        me.formgrid = rgrid;
                    }
                    else {
                        rgrid = new DataGrid({
                            store: dataStore,
                            //	items: data.rows,
                            //	query: { id: "*" },
                      //      autoHeight: true,
                            autoWidth: false,
                     //       rowSelector: '10px',
                            columnReordering: true,
                            structure: gLayout
                        }, "grid");
                        rgrid.startup();
                        me.formgrid = rgrid;
                    }
                }
            };

            // Make sure it runs the first time
            me.createApprovalTable(data, isrefresh);
        },

        createAdminTable: function(data, isrefresh) {
            //
            // Begin button setup/handling code
            //

            var me = this;

            // Destroy existing buttons
            registry.byId('gridsave').destroyRecursive();
            registry.byId('gridsubmit').destroyRecursive();
            registry.byId('discardField').destroyRecursive();

            //
            // End button setup/handling code
            //

            me.createAdminTable = function(data, isrefresh) {
                var grid, store, dataStore;

                if (!STMUtil.isNonEmptyArray(data.rows)) {
                    showNoElementsView(me);
                }
                else {
                    hideNoElementsView(me);

                    store = new Memory({
                        data: data.rows,
                        idProperty: "elementId"
                    });


                    dataStore = new ObjectStore({ objectStore: store });

                    me.changedData.columns = data.columns;
                    me.changedData.elementColumnWidth = data.elementColumnWidth;

                    var gridLayout = [];
                    var gLayout = [];
                    var key, i, colName;

                    var elementname = "<p>" + resources['de.portlet.form.datagrid.elementname.txt'] + "</p>";
                    var elementwidth = data.elementColumnWidth;

                    if (elementwidth == "-1")
                        elementwidth = "10";
                    gridLayout.push({name: elementname, field: "elementName", width: elementwidth, styles: 'text-align: left;',

                        formatter: function(item) {
                            return "<b>" + item + "</b>";
                        }});
                    gLayout.push({ cells: gridLayout, noscroll: true,
                        width: 'auto' });
                    gridLayout = [];
                    for (i = 0; i < data.columns.length; i++) {
                        var colName = "<b>" + data.columns[i].columnName + "<br>" + data.columns[i].periodName + "<br>" + data.columns[i].columnType + "</b>";
                        var ttype = data.columns[i].columnType;
                        if(ttype == "value")
                            colName = "<b>" + data.columns[i].columnName + "<br>" + data.columns[i].periodName + "<br>" + resources['number.txt'] + "</b>";
                        else
                            colName = "<b>" + data.columns[i].columnName + "<br>" + data.columns[i].periodName + "<br>" + resources['text.txt'] + "</b>";

                        var cfield = data.columns[i].columnId + "value";
                        var cwidth = data.columns[i].columnWidth;
                        if (cwidth == "-1")
                            cwidth = 10;
                        if(ttype == "value")
                        {
                            gridLayout.push(
                                {
                                    name: colName, field: cfield, styles: 'text-align: right;', width: cwidth,
                                    formatter: function(item) {
                                        if(item == null || item == "")
                                            return item;
                                        else
                                            return number.format(item);
                                    }});
                        }
                        else
                        {
                            gridLayout.push(
                                {
                                    name: colName, field: cfield, styles: 'text-align: right;', width: cwidth
                                });
                        }
                    }
                    gLayout.push({ cells: gridLayout });
                    
                    var rgrid = registry.byId("grid");
                    // Either update or create new if grid doesn't exist yet
                    if (rgrid) {
                        rgrid.set('structure', gLayout);
                        rgrid.setStore(dataStore);
                        me.formgrid = rgrid;
                    }
                    else {
                        rgrid = new DataGrid({
                            store: dataStore,
                            //	items: data.rows,
                            //	query: { id: "*" },
                         //   autoHeight: true,
                            autoWidth: false,
                       //     rowSelector: '10px',
                            columnReordering: true,
                            structure: gLayout
                        }, "grid");
                        rgrid.startup();
                        me.formgrid = rgrid;
                    }
                }
            };

            me.createAdminTable(data, isrefresh);

        },

        createTable: function(data, isrefresh) {
            //
            // Begin button setup/handling code
            //

            var me = this;


            me.own(
                // Wire up events
                registry.byId('gridsubmit').on('click', function() {
                    topic.publish(AppConst.SUBMIT_FORM_CLICK);
                }),
                registry.byId('discardField').on('click', function() {
                    topic.publish(AppConst.DISCARD_CHANGES_CLICK);
                })
            );

            if (me.state === AppConst.NO_WORKFLOW_STATE) {
                registry.byId('gridsave').destroyRecursive();
            }
            else {
                me.own(
                    // Wire up events
                    registry.byId('gridsave').on('click', function() {
                        topic.publish(AppConst.SAVE_FORM_CLICK);
                    })
                );
            }

            //
            // End button setup/handling code
            //

            // Redefine the function so that the code above only runs once
            me.createTable = function(data, isrefresh) {
                var newData = {};
                var grid, store, dataStore;

                store = new Memory({
                    data: data.rows,
                    idProperty: "elementId"
                });


                dataStore = new ObjectStore({ objectStore: store });
                //      me.oldData =  lang.clone(store);

                newData.columns = data.columns;
                newData.id = AppConst.DATA_ENTRY_ID;
                var newrows = [];
                newData.rows = newrows;

                me.changedData = newData;

                // Show the no elements view if we didn't get any element rows
                if (!STMUtil.isNonEmptyArray(data.rows)) {
                    showNoElementsView(me);
                }
                else {
                    hideNoElementsView(me);
                    // Make sure it is off before processing element rows
                    toggleWriteAccessMessage(false);

                    var numbermin;
                    var numbermax;

                    var formatN = function(item, rowIndex) {
                        debugger;


                        //		 if (item.pending)
                        return "<i>" + item + "</i>";
                        //		else
                        //			return item;
                    };

                    var formatterValue = function(item, rowIndex) {

                        return item;
                    };


                    function textWidget(text, length, tname, isrequired, isedit) {
                        var length = length;
                        var props = {
                            name: tname,
                            //     required: isrequired,
                            //     disabled: isedit,
                            placeHolder: isrequired,
                            style: "width:100%",
                            maxLength: length,
                            value: text
                        };

                        return  new dijit.form.ValidationTextBox(props, tname);

                    }

                    function textWidgetnorule(text, tname) {

                        var props = {
                            name: tname,
                            style: "width:100%",
                            maxLength: 65535,
                            value: text
                        };

                        return  new TextBox(props, tname);

                    }

                    function numberWidgetnorule(tid, tvalue) {
                        var props = {
                            name: tid,
                            value: parseFloat(tvalue),
                            style: "width:100%",
                            invalidMessage: resources['de.portlet.form.datagrid.numberinput.txt'],
                            selectOnClick: true
                        };

                        return  new NumberTextBox(props, tid);
                    }

                    function numberWidget(tid, tvalue, minN, maxN, isrequired, isedit) {
                        var props = {
                            name: tid,
                            value: parseFloat(tvalue),
                            //   required: isrequired,
                            //   disabled: isedit,
                            placeHolder: isrequired,
                            style: "width:100%",
                            constraints: { min: minN, max: maxN},
                            selectOnClick: true,
                            invalidMessage: resources['de.portlet.form.datagrid.numberinput.txt'],
                            rangeMessage: string.substitute(resources['de.portlet.form.datagrid.numberoutrangeError.txt'], [minN , maxN])
                        };

                        return  new NumberTextBox(props, tid);
                    }

                    function formulanumberWidget(tid, tvalue, isrequired, isedit) {
                        var props = {
                            name: tid,
                            value: parseFloat(tvalue),
                            //   required: isrequired,
                            //   disabled: isedit,
                            placeHolder: isrequired,
                            style: "width:100%",
                            selectOnClick: true,
                            invalidMessage: resources['de.portlet.form.datagrid.numberinput.txt']
                        };

                        return  new NumberTextBox(props, tid);
                    }

                    /*        function choiceWidget(wid, cvalue, choicelist, isrequired, isedit) {
                     // create store for choice list
                     var valueComboBoxData = [];

                     for (var i = 0; i < choicelist.length; i++) {

                     valueComboBoxData.push({
                     name: choicelist[i],
                     label: choicelist[i]
                     });
                     }

                     var valueStore = new Memory({ identifier: "name", data: valueComboBoxData });
                     var props = {
                     name: wid,
                     //   required: isrequired,
                     //   disabled: isedit,
                     store: valueStore,
                     //   searchAttr: "name",
                     style: "width:100%",
                     value: cvalue

                     };

                     return   new ComboBox(props, wid);

                     }
                     */
                    function choiceWidget(wid, cvalue, choicelist, isrequired, isedit) {
                        // create store for choice list
                        var valueComboBoxData = [];
                        if(!isrequired)
                        {
                            valueComboBoxData.push({
                                label: "",
                                value: ""
                            });
                        }

                        for (var i = 0; i < choicelist.length; i++) {

                            valueComboBoxData.push({
                                label: choicelist[i],
                                value: choicelist[i]
                            });
                        }


                        var props = {
                            name: wid,
                            //       required: isrequired,
                            //   disabled: isedit,
                            options:  valueComboBoxData,
                            //   searchAttr: "name",
                            style: "width:100%",
                            value: cvalue

                        };

                        return   new ComboBox(props, wid);

                    }
                    var gridLayout = [];
                    var gLayout = [];
                    var key, i, colName;
                    var dislist = [];
                    var elementname = "<p>" + resources['de.portlet.form.datagrid.elementname.txt'] + "</p>";
                    var elementwidth = data.elementColumnWidth;
                    if (elementwidth == "-1")
                        elementwidth = "10";

                    gridLayout.push({name: elementname, field: "elementName", width: elementwidth, styles: 'text-align: left;',

                        formatter: function(item) {
                            return "<b>" + item + "</b>";
                        }});


                    gLayout.push({ cells: gridLayout, noscroll: true,
                        width: 'auto' });
                    gridLayout = [];
                    for (i = 0; i < data.columns.length; i++) {

                        key = i;
                        var ttype = data.columns[i].columnType;
                        if(ttype == "value")
                            colName = "<b>" + data.columns[i].columnName + "<br>" + data.columns[i].periodName + "<br>" + resources['number.txt'] + "</b>";
                        else
                            colName = "<b>" + data.columns[i].columnName + "<br>" + data.columns[i].periodName + "<br>" + resources['text.txt'] + "</b>";
                        var cfield = data.columns[i].columnId + "value";
                        var crule = data.columns[i].columnId + "rule";
                        var iseditable = (data.columns[i].editable == true) ? "" : "disabled";
                        var periodid = data.columns[i].periodId;
                        var cwidth = data.columns[i].columnWidth;
                        if (cwidth == "-1")
                            cwidth = 10;
                        var thisitem;

                        if (ttype == "value") {

                            gridLayout.push(
                                {

                                    name: colName, field: cfield, styles: 'text-align: center;', width: cwidth,

                                    get: (function(crule, cfield, iseditable, periodid) {
                                        return function(rowIndex, item) {

                                            if (item != null) {
                                                return dataStore.getValue(item, cfield) + "_" + dataStore.getValue(item, crule) + "_" + dataStore.getValue(item, "elementId")
                                                    + "_" + cfield + "_" + iseditable + "_" + periodid + "_"
                                                    + dataStore.getValue(item, "writable");
                                            }
                                        }
                                    }(crule, cfield, iseditable, periodid)),

                                    formatter: function(item, rowIndex) {

                                        if (item != null) {
                                            var arrays = item.split("_");
                                            var tvalue = arrays[0];
                                            var irule = arrays[1];
                                            var eid = arrays[2];
                                            var cid = arrays[3];
                                            var isedit = arrays[4];
                                            var pid = arrays[5];
                                            var writable = arrays[6];
                                            if (writable == "false") {
                                                //    if (isedit == "")
                                                isedit = "disabled";
                                            }
                                            var ruletype = 6;
                                            var lowrange = 0;
                                            var highrange = 2000;
                                            var requiredEntry = false;
                                            var formulaText = "";
                                            array.forEach(data.rules, function(rules) {

                                                if (rules.id == irule) {
                                                    ruletype = rules.ruleType;
                                                    lowrange = rules.lowRange;
                                                    highrange = rules.highRange;
                                                    testv = rules.highRange;
                                                    requiredEntry = rules.requiredEntry;
                                                    formulaText = rules.formulaMsg;
                                                    //     formulaText
                                                }
                                            });

                                            if (ruletype == 4) {
                                                var tname = rowIndex + "_" + cid;
                                                var placem = "";

                                                if (requiredEntry && isedit == "") {

                                                    if (tvalue === "") {
                                                        placem = resources['de.portlet.form.datagrid.requiredfield.txt'];
                                                        var idx2 = array.indexOf(dislist, tname);

                                                        if (idx2 < 0) {

                                                            dislist.push(tname);

                                                        }
                                                    }
                                                }
                                                var fwid = formulanumberWidget(tname, tvalue, placem, isedit);
                                                addCursorFix(fwid);
                                                if (isedit == "")
                                                    fwid.disabled = isedit;
                                                else
                                                    fwid.set('disabled',isedit);
                                                fwid.set('required', requiredEntry);

                                                if (dislist.length == 0)
                        						    registry.byId('gridsubmit').set('disabled', false);
                        					    else
                                                    registry.byId('gridsubmit').set('disabled', true);

                                                fwid.on('change', function(newValue) {

                                                    if (newValue == "undefined" || newValue == "NaN")
                                                        newValue = "";
                                                    //  if (newValue != "") {
                                                    var vstore = new DataEntryRuleValidateStore({});
                                                    var inputstr = irule + "/" + newValue + "/" + eid
                                                        + "/" + cid + "/" + pid + "/";

                                                    vstore.get(inputstr).then(function(isvalid) {
                                                        var arr = fwid.name.split("_");
                                                        var mygrid = dijit.byId('grid');
                                                        var item = mygrid.getItem(arr[0]);
                                                        var store = mygrid.store;



                                                        if (newValue != null && newValue != "undefined" && newValue != store.getValue(item, arr[1])) {


                                                            if (isvalid) {


                                                                store.setValue(item, arr[1], newValue);
                                                                //mygrid.update();

                                                                var num = 0;
                                                                array.some(newData.rows, function(rdata) {
                                                                    if (rdata[cid] != null && rdata["elementId"] == eid) {
                                                                        newData.rows.splice(num, 1);
                                                                        return true;
                                                                    }
                                                                    num++;
                                                                });

                                                                var object1 = {};
                                                                object1["elementId"] = eid;
                                                                object1[cid] = newValue.toString();
                                                                newData.rows.push(
                                                                    object1
                                                                );
                                                                var idx = array.indexOf(dislist, fwid.get('name'));

                                                                if (idx >= 0) {
                                                                    dislist.splice(idx, 1);
                                                                }

                                                                if (dislist.length == 0)
                                                                    registry.byId('gridsubmit').set('disabled', false);
                                                            }
                                                            else {
                                                            	if(store.getValue(item, arr[1]).toString().trim() !="")
                                                                    fwid.set('value', store.getValue(item, arr[1]));
                                                                store.setValue(item, arr[1], newValue);
                                                                topic.publish(AppConst.FORMULA_VALIDATION, {sender: fwid, formula: formulaText });
                                                                var idx2 = array.indexOf(dislist, fwid.get('name'));

                                                                if (idx2 < 0) {

                                                                    dislist.push(fwid.get('name'));

                                                                }

                                                                if (dislist.length == 0)
                                                                    registry.byId('gridsubmit').set('disabled', false);
                                                                else
                                                                    registry.byId('gridsubmit').set('disabled', true);

                                                                return;
                                                            }
                                                        }
                                                    });
                                                    //   }

                                                });

                                                fwid.on('dblclick', function(evt) {
                                                    event.stop(evt);

                                                });
                                                fwid._destroyOnRemove = true;
                                                return fwid;
                                            }
                                            else if (ruletype == 1) {

                                                var tname = rowIndex + "_" + cid;
                                                var placem = "";

                                                if (requiredEntry && isedit == "" ) {

                                                    if (tvalue === "") {
                                                        placem = resources['de.portlet.form.datagrid.requiredfield.txt'];
                                                        var idx2 = array.indexOf(dislist, tname);

                                                        if (idx2 < 0) {

                                                            dislist.push(tname);

                                                        }
                                                    }
                                                }
                                                if (tvalue != "" && isedit == "") {
                                                    if (tvalue > highrange || tvalue < lowrange) {
                                                        placem = string.substitute(resources['de.portlet.form.datagrid.numberoutrangeError.txt'], [lowrange , highrange]);
                                                        var idx2 = array.indexOf(dislist, tname);

                                                        if (idx2 < 0) {
                                                            dislist.push(tname);

                                                        }
                                                    }
                                                }

                                                var gridComponent = numberWidget(tname, tvalue, lowrange, highrange, placem, isedit);
                                                addCursorFix(gridComponent);
                                                gridComponent.set('required', requiredEntry);
                                                if (isedit == "")
                                                    gridComponent.disabled = isedit;
                                                else
                                                    gridComponent.set('disabled',isedit);
                                                gridComponent.startup();

                                                if (dislist.length == 0)
                                                    registry.byId('gridsubmit').set('disabled', false);
                                                else
                                                    registry.byId('gridsubmit').set('disabled', true);

                                                gridComponent.on('change', function(newValue) {

                                                    var arr = gridComponent.name.split("_");

                                                    var mygrid = dijit.byId('grid');

                                                    var item = mygrid.getItem(arr[0]);


                                                    if(gridComponent.isValid())
                                                    {
                                                        var idx = array.indexOf(dislist, gridComponent.get('name'));

                                                        if (idx >= 0) {
                                                            dislist.splice(idx, 1);
                                                        }

                                                        if (dislist.length == 0)
                                                            registry.byId('gridsubmit').set('disabled', false);

                                                    }
                                                    else
                                                    {
                                                        var idx2 = array.indexOf(dislist, gridComponent.get('name'));

                                                        if (idx2 < 0) {

                                                            dislist.push(gridComponent.get('name'));

                                                        }

                                                        if (dislist.length == 0)
                                                            registry.byId('gridsubmit').set('disabled', false);
                                                        else
                                                            registry.byId('gridsubmit').set('disabled', true);

                                                        return;
                                                    }
                                                    if (newValue == "undefined")
                                                        return;
                                                    /*     if (gridComponent.get('required') && newValue.toString() == "NaN")
                                                     return;
                                                     if (newValue.toString() == "NaN")
                                                     newValue = "";

                                                     if (newValue > highrange || newValue < lowrange)
                                                     return;
                                                     var idx = array.indexOf(dislist, gridComponent.get('name'));

                                                     if (idx >= 0) {
                                                     dislist.splice(idx, 1);
                                                     }

                                                     if (dislist.length == 0)
                                                     registry.byId('gridsubmit').set('disabled', false);
                                                     */
                                                    var store = mygrid.store;
                                                    store.setValue(item, arr[1], newValue);
                                                    //     mygrid.update();

                                                    var num = 0;
                                                    array.some(newData.rows, function(rdata) {
                                                        if (rdata[cid] != null && rdata["elementId"] == eid) {
                                                            newData.rows.splice(num, 1);
                                                            return true;
                                                        }
                                                        num++;
                                                    });

                                                    var object1 = {};
                                                    object1["elementId"] = eid;
                                                    object1[cid] = newValue.toString();
                                                    newData.rows.push(
                                                        object1
                                                    );

                                                });

                                                /*                 gridComponent.on('blur', function() {
                                                 if(gridComponent.isValid())
                                                 {
                                                 var idx = array.indexOf(dislist, gridComponent.get('name'));

                                                 if (idx >= 0) {
                                                 dislist.splice(idx, 1);
                                                 }

                                                 if (dislist.length == 0)
                                                 registry.byId('gridsubmit').set('disabled', false);

                                                 }
                                                 else
                                                 {
                                                 var idx2 = array.indexOf(dislist, gridComponent.get('name'));

                                                 if (idx2 < 0) {

                                                 dislist.push(gridComponent.get('name'));

                                                 }

                                                 if (dislist.length == 0)
                                                 registry.byId('gridsubmit').set('disabled', false);
                                                 else
                                                 registry.byId('gridsubmit').set('disabled', true);

                                                 }
                                                 });
                                                 */
                                                gridComponent._destroyOnRemove = true;


                                                return gridComponent;

                                            }
                                            else {
                                                var tname = rowIndex + "_" + cid;
                                                var gridComponent = numberWidgetnorule(tname, tvalue);

                                                addCursorFix(gridComponent);

                                                //     gridComponent.required = requiredEntry;
                                                if (isedit == "")
                                                    gridComponent.disabled = isedit;
                                                else
                                                    gridComponent.set('disabled',isedit);
                                                //    gridComponent.startup();
                                                gridComponent.on('change', function(newValue) {

                                                    var arr = gridComponent.name.split("_");

                                                    var mygrid = dijit.byId('grid');


                                                    var item = mygrid.getItem(arr[0]);

                                                    if(gridComponent.isValid())
                                                    {
                                                        var idx = array.indexOf(dislist, gridComponent.get('name'));

                                                        if (idx >= 0) {
                                                            dislist.splice(idx, 1);
                                                        }

                                                        if (dislist.length == 0)
                                                            registry.byId('gridsubmit').set('disabled', false);

                                                    }
                                                    else
                                                    {
                                                        var idx2 = array.indexOf(dislist, gridComponent.get('name'));

                                                        if (idx2 < 0) {

                                                            dislist.push(gridComponent.get('name'));

                                                        }

                                                        if (dislist.length == 0)
                                                            registry.byId('gridsubmit').set('disabled', false);
                                                        else
                                                            registry.byId('gridsubmit').set('disabled', true);

                                                        return;
                                                    }
                                                    if (newValue == "undefined")
                                                        return;

                                                    var store = mygrid.store;
                                                    store.setValue(item, arr[1], newValue);
                                                    //mygrid.update();

                                                    var num = 0;
                                                    array.some(newData.rows, function(rdata) {
                                                        if (rdata[cid] != null && rdata["elementId"] == eid) {
                                                            newData.rows.splice(num, 1);
                                                            return true;
                                                        }
                                                        num++;
                                                    });

                                                    var object1 = {};
                                                    object1["elementId"] = eid;
                                                    object1[cid] = newValue.toString();
                                                    newData.rows.push(
                                                        object1
                                                    );

                                                });


                                                gridComponent._destroyOnRemove = true;


                                                return gridComponent;
                                            }

                                        }
                                    }

                                });

                        }
                        else {
                            gridLayout.push(
                                {
                                    name: colName, field: cfield, width: cwidth, styles: 'text-align: center;',


                                    get: (function(crule, cfield, iseditable, periodid) {
                                        return function(rowIndex, item) {
                                            if (item != null)
                                                return  dataStore.getValue(item, cfield) + "_" + dataStore.getValue(item, crule) + "_" +
                                                    dataStore.getValue(item, "elementId") + "_" + cfield + "_" + iseditable + "_" + periodid
                                                    + "_" + dataStore.getValue(item, "writable");
                                            ;
                                        }
                                    }(crule, cfield, iseditable, periodid)),
                                    formatter: function(item, rowIndex) {
                                        if (item != null) {
                                            var arrays = item.split("_");
                                            var currentvalue = arrays[0];
                                            var irule = arrays[1];
                                            var eid = arrays[2];
                                            var ccfield = arrays[3];
                                            var isedit = arrays[4];
                                            var pid = arrays[5];
                                            var writable = arrays[6];
                                            if (writable == "false")  {
                                                //     if (isedit == "")
                                                isedit = "disabled";
                                            }
                                            var ruletype = 6;
                                            var maxlength = 65535;
                                            var choiceList;
                                            var formulaText;
                                            var requiredEntry = false;
                                            array.forEach(data.rules, function(rules) {

                                                if (rules.id == irule) {
                                                    ruletype = rules.ruleType;
                                                    maxlength = rules.maxLength;
                                                    if (maxlength > 65535)
                                                        maxlength = 65535;
                                                    choiceList = rules.choiceList;
                                                    formulaText = rules.formulaMsg;
                                                    requiredEntry = rules.requiredEntry;
                                                }
                                            });


                                            if (ruletype == 3) {

                                                var wid = rowIndex + "_" + ccfield;

                                                // Parse on semicolon, or either of the UNICODE semicolons
                                                if (choiceList.indexOf(';') > -1)
                                                    delimiter = ";";
                                                else if (choiceList.indexOf('\u003B') > -1)
                                                    delimiter = '\u003B';
                                                else if (choiceList.indexOf('\uFF1B') > -1)
                                                    delimiter = '\uFF1B';
                                                else
                                                    delimiter = ";";

                                                var choicelist = choiceList.split(delimiter);

                                                var selects = choiceWidget(wid, currentvalue, choicelist, requiredEntry, isedit);
                                                //    selects.set('disabled', isedit);
                                                selects.set('required', requiredEntry);
                                                if (isedit == "")
                                                    selects.disabled = isedit;
                                                else
                                                    selects.set('disabled',isedit);
                                                selects.startup();
                                                selects.on('change', function(newValue) {
                                                    var arr = selects.name.split("_");
                                                    var mygrid = dijit.byId('grid');
                                                    var item = mygrid.getItem(arr[0]);
                                                    var store = mygrid.store;
                                                    store.setValue(item, arr[1], newValue);
                                                    //     mygrid.update();
                                                    var num = 0;
                                                    array.some(newData.rows, function(rdata) {
                                                        if (rdata[ccfield] != null && rdata["elementId"] == eid) {
                                                            newData.rows.splice(num, 1);
                                                            return true;
                                                        }
                                                        num++;
                                                    });
                                                    var object1 = {};
                                                    object1["elementId"] = eid;
                                                    object1[ccfield] = newValue.toString();
                                                    newData.rows.push(
                                                        object1
                                                    );
                                                });

                                                selects.on('dblclick', function(evt) {
                                                    event.stop(evt);

                                                });
                                                selects._destroyOnRemove = true;

                                                return  selects;
                                            }
                                            else if (ruletype == 4) {
                                                var tname = rowIndex + "_" + ccfield;
                                                var placem = "";
                                                if (requiredEntry && isedit == "") {

                                                    if (currentvalue === "") {
                                                        placem = resources['de.portlet.form.datagrid.requiredfield.txt'];
                                                        var idx2 = array.indexOf(dislist, tname);

                                                        if (idx2 < 0) {

                                                            dislist.push(tname);

                                                        }
                                                    }
                                                }
                                                var fwid = textWidget(currentvalue, "800", tname, placem, isedit);
                                                if (isedit == "")
                                                    fwid.disabled = isedit;
                                                else
                                                    fwid.set('disabled',isedit);
                                                fwid.required = requiredEntry;
                                                
                                                if (dislist.length == 0)
                        						    registry.byId('gridsubmit').set('disabled', false);
                        						else
                                                    registry.byId('gridsubmit').set('disabled', true);

                                                fwid.on('blur', function() {
                                                    popup.close();
                                                    //     fwid.focus();
                                                });
                                                fwid.on('click', function(event) {

                                                    var editor = new Textarea({
                                                        name: fwid.get('name'),
                                                        value: fwid.get('value'),
                                                        maxLength: 500,
                                                        cols: 40,
                                                        rows: 8
                                                    });
                                                    editor.startup();

                                                    popup.open({
                                                        parent: this,
                                                        popup: editor,
                                                        x: event.clientX,
                                                        y: event.clientY,
                                                        around: this.domNode,
                                                        onCancel: function() {

                                                        },
                                                        onClose: function() {
                                                            fwid.set('value', editor.get('value'));
                                                            editor.destroy();
                                                            popup.close(editor);
                                                        },

                                                        onExecute: function() {
                                                            fwid.set('value', editor.get('value'));
                                                            popup.close(editor);
                                                            editor.destroy();
                                                            fwid.focus();
                                                        }

                                                    });

                                                });
                                                fwid.on('change', function(newValue) {
                                                    if (fwid.get('required') && newValue.toString() == "NaN")
                                                        return;
                                                    if (newValue.toString() == "NaN")
                                                        newValue = "";
                                                    if (newValue != "") {
                                                        var vstore = new DataEntryRuleValidateStore({});
                                                        var inputstr = irule + "/" + newValue + "/" + eid
                                                            + "/" + ccfield + "/" + pid + "/";
                                                        var arr = fwid.name.split("_");
                                                        var mygrid = dijit.byId('grid');
                                                        var item = mygrid.getItem(arr[0]);
                                                        var store = mygrid.store;
                                                        vstore.get(inputstr).then(function(isvalid) {
                                                            if (newValue != null && newValue != store.getValue(item, arr[1])) {
                                                                //console.log("valid " ,isvalid);
                                                                if (isvalid) {

                                                                    store.setValue(item, arr[1], newValue);
                                                                    //       mygrid.update();

                                                                    var num = 0;
                                                                    array.some(newData.rows, function(rdata) {
                                                                        if (rdata[ccfield] != null && rdata["elementId"] == eid) {
                                                                            newData.rows.splice(num, 1);
                                                                            return true;
                                                                        }
                                                                        num++;
                                                                    });
                                                                    var object1 = {};
                                                                    object1["elementId"] = eid;
                                                                    object1[ccfield] = newValue.toString();
                                                                    newData.rows.push(
                                                                        object1
                                                                    );

                                                                    var idx = array.indexOf(dislist, fwid.get('name'));

                                                                    if (idx >= 0) {
                                                                        dislist.splice(idx, 1);
                                                                    }

                                                                    if (dislist.length == 0)
                                                                        registry.byId('gridsubmit').set('disabled', false);

                                                                }
                                                                else {
                                                                	if(store.getValue(item, arr[1]).toString().trim() !="")
                                                                       fwid.set('value', store.getValue(item, arr[1]));
                                                                    store.setValue(item, arr[1], newValue);
                                                                    topic.publish(AppConst.FORMULA_VALIDATION, {sender: fwid, formula: formulaText });
                                                                    var idx2 = array.indexOf(dislist, fwid.get('name'));

                                                                    if (idx2 < 0) {

                                                                        dislist.push(fwid.get('name'));

                                                                    }

                                                                    if (dislist.length == 0)
                                                                        registry.byId('gridsubmit').set('disabled', false);
                                                                    else
                                                                        registry.byId('gridsubmit').set('disabled', true);

                                                                    return;
                                                                }
                                                            }
                                                        });
                                                    }
                                                });

                                                fwid.on('dblclick', function(evt) {
                                                    event.stop(evt);

                                                });
                                                fwid._destroyOnRemove = true;
                                                return fwid;
                                            }
                                            else if (ruletype == 2) {

                                                var tname = rowIndex + "_" + ccfield;
                                                var placem = "";
                                                if (requiredEntry && isedit == "") {

                                                    if (currentvalue === "") {
                                                        placem = resources['de.portlet.form.datagrid.requiredfield.txt'];
                                                        var idx2 = array.indexOf(dislist, tname);

                                                        if (idx2 < 0) {

                                                            dislist.push(tname);

                                                        }
                                                    }
                                                }
                                                var twid = textWidget(currentvalue, maxlength, tname, placem, isedit);
                                                if (isedit == "")
                                                    twid.disabled = isedit;
                                                else
                                                    twid.set('disabled',isedit);
                                                twid.set('required', requiredEntry);
                                                if (dislist.length == 0)
                        						    registry.byId('gridsubmit').set('disabled', false);
                        						else
                                                    registry.byId('gridsubmit').set('disabled', true);

                                                twid.on('blur', function() {
                                                    popup.close();
                                                    //          twid.focus();
                                                });
                                                twid.on('click', function(event) {

                                                    var editor = new Textarea({
                                                        name: twid.get('name'),
                                                        value: twid.get('value'),
                                                        maxLength: twid.get('maxLength'),
                                                        cols: 40,
                                                        rows: 8

                                                    });
                                                    editor.startup();
                                                    editor.focus();
                                                    popup.open({
                                                        parent: this,
                                                        popup: editor,
                                                        x: event.clientX,
                                                        y: event.clientY,
                                                        around: this.domNode,
                                                        onCancel: function() {

                                                        },
                                                        onClose: function() {
                                                            twid.set('value', editor.get('value'));
                                                            editor.destroy();
                                                            popup.close();
                                                        },

                                                        onExecute: function() {
                                                            twid.set('value', editor.get('value'));
                                                            popup.close(editor);
                                                            editor.destroy();
                                                            twid.focus();
                                                        }

                                                    });

                                                });

                                                twid.on('change', function(newValue) {

                                                    var arr = twid.name.split("_");
                                                    var mygrid = dijit.byId('grid');
                                                    var item = mygrid.getItem(arr[0]);
                                                    var store = mygrid.store;

                                                    //     mygrid.update();
                                                    if(twid.isValid())
                                                    {
                                                        var idx = array.indexOf(dislist, twid.get('name'));

                                                        if (idx >= 0) {
                                                            dislist.splice(idx, 1);
                                                        }

                                                        if (dislist.length == 0)
                                                            registry.byId('gridsubmit').set('disabled', false);

                                                    }
                                                    else
                                                    {
                                                        var idx2 = array.indexOf(dislist, twid.get('name'));

                                                        if (idx2 < 0) {

                                                            dislist.push(twid.get('name'));

                                                        }

                                                        if (dislist.length == 0)
                                                            registry.byId('gridsubmit').set('disabled', false);
                                                        else
                                                            registry.byId('gridsubmit').set('disabled', true);

                                                        return;
                                                    }
                                                    if (newValue.toString() == "NaN")
                                                        return;
                                                    store.setValue(item, arr[1], newValue);
                                                    var num = 0;
                                                    array.some(newData.rows, function(rdata) {
                                                        if (rdata[ccfield] != null && rdata["elementId"] == eid) {
                                                            newData.rows.splice(num, 1);
                                                            return true;
                                                        }
                                                        num++;
                                                    });
                                                    var object1 = {};
                                                    object1["elementId"] = eid;
                                                    object1[ccfield] = newValue.toString();
                                                    newData.rows.push(
                                                        object1
                                                    );
                                                });

                                                /*    twid.on('dblclick', function(evt){
                                                 event.stop(evt);

                                                 });
                                                 */

                                                twid._destroyOnRemove = true;
                                                return twid;
                                            }
                                            else {

                                                var tname = rowIndex + "_" + ccfield;
                                                var twid = textWidgetnorule(currentvalue, tname);
                                                if (isedit == "")
                                                    twid.disabled = isedit;
                                                else
                                                    twid.set('disabled',isedit);
                                                twid.on('blur', function() {
                                                    popup.close();
                                                });
                                                twid.on('click', function(event) {

                                                    var editor = new Textarea({
                                                        name: twid.get('name'),
                                                        value: twid.get('value'),
                                                        maxLength: twid.get('maxLength'),
                                                        cols: 40,
                                                        rows: 8

                                                    });
                                                    editor.startup();

                                                    popup.open({
                                                        parent: this,
                                                        popup: editor,
                                                        x: event.clientX,
                                                        y: event.clientY,
                                                        around: this.domNode,
                                                        onCancel: function() {

                                                        },
                                                        onClose: function() {
                                                            twid.set('value', editor.get('value'));
                                                            editor.destroy();
                                                            popup.close(editor);
                                                        },

                                                        onExecute: function() {
                                                            twid.set('value', editor.get('value'));
                                                            popup.close(editor);
                                                            //console.log("herewwww " , editor);
                                                            editor.destroy();
                                                            twid.focus();
                                                        }

                                                    });
                                                    editor.focus();

                                                });

                                                twid.on('change', function(newValue) {

                                                    var arr = twid.name.split("_");
                                                    var mygrid = dijit.byId('grid');
                                                    var item = mygrid.getItem(arr[0]);
                                                    var store = mygrid.store;
                                                    store.setValue(item, arr[1], newValue);
                                                    //    mygrid.update();

                                                    var num = 0;
                                                    array.some(newData.rows, function(rdata) {
                                                        if (rdata[ccfield] != null && rdata["elementId"] == eid) {
                                                            newData.rows.splice(num, 1);
                                                            return true;
                                                        }
                                                        num++;
                                                    });
                                                    var object1 = {};
                                                    object1["elementId"] = eid;
                                                    object1[ccfield] = newValue.toString();
                                                    newData.rows.push(
                                                        object1
                                                    );
                                                });

                                                /*    twid.on('dblclick', function(evt){
                                                 event.stop(evt);

                                                 });
                                                 */

                                                twid._destroyOnRemove = true;
                                                return twid;
                                            }
                                            //     else
                                            //         return currentvalue;

                                        }
                                    }




                                });
                        }

                    }

                    gLayout.push({ cells: gridLayout });

                    var agrid = registry.byId("grid");
                    
                    if (agrid) {
                        agrid.set('structure', gLayout);
                        agrid.setStore(dataStore);
                        me.formgrid = agrid;
                    }
                    else {
                        agrid = new DataGrid({
                            store: dataStore,
                            //	items: data.rows,
                            //	query: { id: "*" },
                        //    autoHeight: true,
                            autoWidth: false,
                     //       rowSelector: '10px',
                            columnReordering: true,
                            structure: gLayout,
                            singleClickEdit: false,
                            selectionMode: 'none'
                        }, "grid");

                        // "Turn off" problematic grid handler functions
                        skipFunctions(agrid, [
                            'onCellClick',
                            'onRowClick',
                            'onCellDblClick',
                            'onRowDblClick',
                            'onKeyEvent'
                        ]);

                        agrid.startup();
                        me.formgrid = agrid;
                    }
                }
            };

            // Make sure it runs the first time
            me.createTable(data, isrefresh);
        }


    });

});