define([
    'dojo/i18n!html/nls/STMResources',
    'dojo/text!html/app/view/manageforms/formpropertypages/ElementsPageView.html',
    'dojo/_base/declare',
    'dojo/_base/lang',
    'dojo/when',
    'dojo/on',
    'dojo/dom',
    'dojo/string',
    'dojo/aspect',
    'dojo/dom-construct',
    'dojo/topic',
    'dojo/store/Memory',
    'dijit/registry',
    'dijit/layout/ContentPane',
    'dijit/tree/ObjectStoreModel',
    'dijit/form/FilteringSelect',
    'STM/util/ApplicationConstants',
    'STM/model/ElementTypeStore',
    'STM/view/DialogTemplate',
    'STM/view/PageNavigatorPage',
    'STM/view/ScorecardElementTree',
    'STM/util/manageforms/ApplicationConstants'
], function(resources, templateString, declare, lang, when, on, dom, string, aspect, domConstruct, topic, Memory, registry, 
            ContentPane, ObjectStoreModel, FilteringSelect, GeneralAppConst, ElementTypeStore, DialogTemplate, PageNavigatorPage, 
            ScorecardElementTree, AppConst) {
    
    return declare([ContentPane, PageNavigatorPage], {
        postCreate: function() {
            var me = this;
            
            me.set('title', resources['de.form.elementspage.title.txt']);
            
            templateString = string.substitute(templateString, {
                helpText: resources['de.form.elementspage.helptip.txt'],
                elementTypeLabel: resources['de.form.elementspage.elementtype.txt'],
                scorecardElementTreeLabel: resources['de.form.elementspage.elements.txt']
            });
        },
        onShow: function() {
            this.initUI();

            // Init code should only run once
            this.onShow = function() {};
        },
        initUI: function() {
            var me = this;
            
            when(me.set('content', templateString), function() {
                me.listSubscriptions();

                when(new ElementTypeStore({}).query(''), function(elementTypes) {
                    var defaultElementTypeId = elementTypes.length > 0 ? elementTypes[0].id : null;
                    var elementTypeId = me.sharedData.selectedElementTypeId || defaultElementTypeId;

                    var elementTypeField = new FilteringSelect({
                        placeHolder: resources['de.form.elementspage.elementtype.empty.txt'],
                        store: new Memory({data: elementTypes}),
                        required: false,
                        value: elementTypeId
                    }, 'elementTypeField');

                    // Flag the page as dirty if we are setting default values
                    if (!me.sharedData.selectedElementTypeId) {
                        me.setIsPageDirty(true);
                    }

                    var treeField;

                    elementTypeField.startup();

                    //TODO Should handle setting page to dirty if the user expands their first node without having any previously saved elements
                    if (elementTypeId) {
                        treeField = new ScorecardElementTree({
                            elementTypeId: elementTypeId,
                            elementList: me.sharedData.selectedElementIds || [],
                            scorecardList: me.sharedData.selectedScorecardIds || []
                        }, 'scorecardElementTree');

                        treeField.startup();

                        me.setDirtyListeners([
                            treeField
                        ], GeneralAppConst.STM_SC_TREE_ELEMENT_SELECTED);
                    }

                    me.setDirtyListeners([
                        elementTypeField
                    ]);

                    me.own(
                        elementTypeField.on('change', function(newValue) {
                            topic.publish(AppConst.FORM_ELEMENTTYPE_SELECTED, newValue);
                        })
                    );
                }, function() {
                    new DialogTemplate({
                        type: GeneralAppConst.ERROR_SERVER_DATA_DIALOG
                    }).show();
                });
            });
        },
        listSubscriptions: function() {
            this.own(
                topic.subscribe(AppConst.FORM_ELEMENTTYPE_SELECTED, this.onElementtypeSelected)
            );
        },
        onHide: function() {
            this.saveDirtyData();
        },
        getValuesToSave: function() {
            var elementTypeId = registry.byId('elementTypeField').get('value');
            var elements = registry.byId('scorecardElementTree').getSelectedElements();
            var scorecards = registry.byId('scorecardElementTree').getSelectedScorecards();

            return {
                selectedElementTypeId: elementTypeId,
                selectedElementIds: elements,
                selectedScorecardIds: scorecards
            };
        },
        getInvalidFields: function() {
            var elementTypeValue = registry.byId('elementTypeField').get('value');
            var selectedElementsValue = registry.byId('scorecardElementTree').getSelectedElements();
            var selectedScorecardsValue = registry.byId('scorecardElementTree').getSelectedScorecards();
            var invalidFields = [];

            if (elementTypeValue === null || elementTypeValue === '') {
                invalidFields.push(resources['de.form.elementspage.elements.type.invalid.txt']);
            }

            if (selectedElementsValue.length < 1 && selectedScorecardsValue.length < 1) {
                invalidFields.push(resources['de.form.elementspage.elements.invalid.txt']);
            }

            return invalidFields;
        },
        onElementtypeSelected: function(elementTypeId) {
            var newTreeNode = domConstruct.toDom('<div id="scorecardElementTree"></div>');
            var cellNode = dom.byId('elementTreeCell');

            registry.byId('scorecardElementTree').destroyRecursive();
            domConstruct.place(newTreeNode, cellNode);

            new ScorecardElementTree({
                elementTypeId: elementTypeId
            }, 'scorecardElementTree').startup();
        }
    });

});