define([
    'dojo/i18n!html/nls/STMResources',
    'dojo/text!html/app/view/managerules/ManageRulesPropertiesView.html',
    'dojo/_base/declare',
    'dojo/_base/array',
    'dojo/query',
    'dojo/string',
    'dojo/when',
    'dojo/topic',
    'dijit/registry',
    'dijit/form/Button',
    'dijit/layout/TabContainer',
    'dijit/layout/ContentPane',
    'STM/view/PageNavigator',
    'STM/view/managerules/rulepropertypages/IdentificationPageView',
    'STM/view/managerules/rulepropertypages/RulePropertiesPageView',
    'STM/util/ApplicationConstants',
    'STM/util/managerules/ApplicationConstants'
], function(resources, templateString, declare, array, query, string, when, topic, registry, Button, TabContainer,
            ContentPane, PageNavigator, IdentificationPageView, RulePropertiesPageView, GeneralAppConst, AppConst) {

    //
    // Private getters for convenience
    //

    var getSaveButton = function() {
        return registry.byId('rulePropertiesSaveButton');
    };

    var getCancelButton = function() {
        return registry.byId('rulePropertiesCancelButton');
    };

    return declare([ContentPane, PageNavigator], {
        id: 'propertiesView',
        region: 'bottom',
        splitter: true,
   //     'class': 'stmPropertiesPanel',
        topic: topic,
        postCreate: function() {
            var me = this;

            templateString = string.substitute(templateString, {
                saveButtonLabel: resources['save.txt'],
                cancelButtonLabel: resources['cancel.txt']
            });

            when(me.set('content', templateString), function() {
                me.initUI();
            });
        },
        initUI: function() {
            var me = this;

            // Find all of the pages under this navigator, convert to widgets, bind to the navigator
            var pages = registry.byId('pageContainer').getChildren();

            array.forEach(pages, function(page ) {
                me.bindPageToNavigator(page);
            });

            if (!GeneralAppConst.CAN_UPDATE) {
                me.setSaveButtonEnabled(false);
            }
            else {
                me.own(
                    getSaveButton().on('click', function() {
                        topic.publish(AppConst.SAVE_RULE_CLICK);
                    })
                );
            }

            me.own(
                // Convert the PageNavigator events into notifications that the controller can subscribe to
                me.on(GeneralAppConst.STM_CLOSE_NAVIGATOR, function(callback) {
                    topic.publish(GeneralAppConst.STM_CLOSE_NAVIGATOR, callback);
                }),

                me.on(GeneralAppConst.STM_CLOSE_DIRTY_NAVIGATOR, function(callback) {
                    topic.publish(GeneralAppConst.STM_CLOSE_DIRTY_NAVIGATOR, callback);
                }),

                me.on(GeneralAppConst.STM_PAGE_NAVIGATOR_DIRTY, function() {
                    topic.publish(GeneralAppConst.STM_PAGE_NAVIGATOR_DIRTY, me);
                }),

                // Wire up events
                getCancelButton().on('click', function() {
                    // 'this' = the button
                    this.getParent().closeNavigator();
                })
            );

            // The controller can listen to this to set listeners that depend on instantiation
            topic.publish(AppConst.PROPERTIES_VIEW_CREATED, this);
        },
        setSaveButtonEnabled: function(enabled) {
            getSaveButton().set('disabled', !enabled);
        }
    })

});

