define([
    'dojo/i18n!html/nls/STMResources',
    'dojo/text!html/app/view/managerules/rulepropertypages/IdentificationPageView.html',
    'dojo/_base/declare',
    'dojo/_base/lang',
    'dojo/when',
    'dojo/on',
    'dojo/dom',
    'dojo/string',
    'dojo/store/Memory',
    'dijit/registry',
    'dijit/layout/ContentPane',
    'dijit/form/ValidationTextBox',
    'dijit/form/SimpleTextarea',
    'STM/view/PageNavigatorPage',
    'STM/view/DialogTemplate',
	'STM/util/ApplicationConstants'
], function(resources, templateString, declare, lang, when, on, dom, string, Memory, registry, ContentPane,
            ValidationTextBox, SimpleTextarea, PageNavigatorPage, DialogTemplate, GeneralAppConst) {

    return declare([ContentPane, PageNavigatorPage], {
        postCreate: function() {
            var me = this;

            me.set('title', resources['rule.identificationpage.title.txt']);
            templateString = string.substitute(templateString, {
                helpText: resources['rule.identificationpage.helptip.txt'],
                helplabel: resources['help.txt'],
                helpTeaser: resources['rule.identificationpage.teaser.txt'],
                helpContext: resources['rule.identificationpage.popup.txt'],
                nameLabel: resources['rule.identificationpage.name.txt'],
                descriptionLabel: resources['rule.identificationpage.description.txt'],
                requiredEntryLabel: resources['rule.identificationpage.requiredentry.txt']
            });
        },
        onShow: function() {
            this.initUI();

            // Init code should only run once
            this.onShow = function() {};
        },
        initUI: function() {
            var me = this;

            when(me.set('content', templateString), function() {
                var nameField = registry.byId('nameField');
                var descriptionField = registry.byId('descriptionField');
                var requiredEntryField = registry.byId('requiredEntryField');

                nameField.set('value', me.sharedData.name || '');
                descriptionField.set('value', me.sharedData.description || '');
                requiredEntryField.set('checked', me.sharedData.requiredEntry);

				nameField.focus();

                me.setDirtyListeners([
                    nameField,
                    descriptionField
                ], 'keyup');

                me.setDirtyListeners([
                    requiredEntryField
                ], 'click');
            });
        },
        onHide: function() {
            this.saveDirtyData();
        },
        getValuesToSave: function() {
            var ruleName = registry.byId('nameField').get('value');
            var description = registry.byId('descriptionField').get('value');
            var requiredEntry = registry.byId('requiredEntryField').get('checked');

            return {
                name: ruleName,
                description: description,
                requiredEntry: requiredEntry
            };
        },
        //TODO Think about adding native validation highlighting
        getInvalidFields: function() {
			var obj = registry.byId('nameField');
            var nameValue = "";
            if (obj !== null)
            	nameValue = obj.get('value');

            if (nameValue === null || nameValue === '') {
                return [resources['rule.identificationpage.name.invalid.txt']];
            }
            else {
                return [];
            }
        }
    });
});